@@ -1,6 +1,6 @@
 /* valaccodememberaccess.vala
  *
- * Copyright (C) 2006  Jürg Billeter
+ * Copyright (C) 2006-2008  Jürg Billeter
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -21,7 +21,6 @@
  */
 
 #include <ccode/valaccodememberaccess.h>
-#include <ccode/valaccodenode.h>
 
 
 
@@ -44,9 +43,9 @@ static void vala_ccode_member_access_finalize (ValaCCodeNode* obj);
 
 ValaCCodeMemberAccess* vala_ccode_member_access_construct (GType object_type, ValaCCodeExpression* container, const char* member, gboolean pointer) {
 	ValaCCodeMemberAccess* self;
-	g_return_val_if_fail (VALA_IS_CCODE_EXPRESSION (container), NULL);
+	g_return_val_if_fail (container != NULL, NULL);
 	g_return_val_if_fail (member != NULL, NULL);
-	self = ((ValaCCodeMemberAccess*) (g_type_create_instance (object_type)));
+	self = (ValaCCodeMemberAccess*) g_type_create_instance (object_type);
 	vala_ccode_member_access_set_inner (self, container);
 	vala_ccode_member_access_set_member_name (self, member);
 	vala_ccode_member_access_set_is_pointer (self, pointer);
@@ -61,9 +60,9 @@ ValaCCodeMemberAccess* vala_ccode_member_access_new (ValaCCodeExpression* contai
 
 ValaCCodeMemberAccess* vala_ccode_member_access_construct_pointer (GType object_type, ValaCCodeExpression* container, const char* member) {
 	ValaCCodeMemberAccess* self;
-	g_return_val_if_fail (VALA_IS_CCODE_EXPRESSION (container), NULL);
+	g_return_val_if_fail (container != NULL, NULL);
 	g_return_val_if_fail (member != NULL, NULL);
-	self = ((ValaCCodeMemberAccess*) (g_type_create_instance (object_type)));
+	self = (ValaCCodeMemberAccess*) g_type_create_instance (object_type);
 	vala_ccode_member_access_set_inner (self, container);
 	vala_ccode_member_access_set_member_name (self, member);
 	vala_ccode_member_access_set_is_pointer (self, TRUE);
@@ -78,9 +77,9 @@ ValaCCodeMemberAccess* vala_ccode_member_access_new_pointer (ValaCCodeExpression
 
 static void vala_ccode_member_access_real_write (ValaCCodeNode* base, ValaCCodeWriter* writer) {
 	ValaCCodeMemberAccess * self;
-	self = VALA_CCODE_MEMBER_ACCESS (base);
-	g_return_if_fail (VALA_IS_CCODE_WRITER (writer));
-	vala_ccode_node_write (VALA_CCODE_NODE (self->priv->_inner), writer);
+	self = (ValaCCodeMemberAccess*) base;
+	g_return_if_fail (writer != NULL);
+	vala_ccode_expression_write_inner (self->priv->_inner, writer);
 	if (self->priv->_is_pointer) {
 		vala_ccode_writer_write_string (writer, "->");
 	} else {
@@ -91,7 +90,7 @@ static void vala_ccode_member_access_real_write (ValaCCodeNode* base, ValaCCodeW
 
 
 ValaCCodeExpression* vala_ccode_member_access_get_inner (ValaCCodeMemberAccess* self) {
-	g_return_val_if_fail (VALA_IS_CCODE_MEMBER_ACCESS (self), NULL);
+	g_return_val_if_fail (self != NULL, NULL);
 	return self->priv->_inner;
 }
 
@@ -99,15 +98,15 @@ ValaCCodeExpression* vala_ccode_member_access_get_inner (ValaCCodeMemberAccess*
 void vala_ccode_member_access_set_inner (ValaCCodeMemberAccess* self, ValaCCodeExpression* value) {
 	ValaCCodeExpression* _tmp2;
 	ValaCCodeExpression* _tmp1;
-	g_return_if_fail (VALA_IS_CCODE_MEMBER_ACCESS (self));
+	g_return_if_fail (self != NULL);
 	_tmp2 = NULL;
 	_tmp1 = NULL;
-	self->priv->_inner = (_tmp2 = (_tmp1 = value, (_tmp1 == NULL ? NULL : vala_ccode_node_ref (_tmp1))), (self->priv->_inner == NULL ? NULL : (self->priv->_inner = (vala_ccode_node_unref (self->priv->_inner), NULL))), _tmp2);
+	self->priv->_inner = (_tmp2 = (_tmp1 = value, (_tmp1 == NULL) ? NULL : vala_ccode_node_ref (_tmp1)), (self->priv->_inner == NULL) ? NULL : (self->priv->_inner = (vala_ccode_node_unref (self->priv->_inner), NULL)), _tmp2);
 }
 
 
 const char* vala_ccode_member_access_get_member_name (ValaCCodeMemberAccess* self) {
-	g_return_val_if_fail (VALA_IS_CCODE_MEMBER_ACCESS (self), NULL);
+	g_return_val_if_fail (self != NULL, NULL);
 	return self->priv->_member_name;
 }
 
@@ -115,21 +114,21 @@ const char* vala_ccode_member_access_get_member_name (ValaCCodeMemberAccess* sel
 void vala_ccode_member_access_set_member_name (ValaCCodeMemberAccess* self, const char* value) {
 	char* _tmp2;
 	const char* _tmp1;
-	g_return_if_fail (VALA_IS_CCODE_MEMBER_ACCESS (self));
+	g_return_if_fail (self != NULL);
 	_tmp2 = NULL;
 	_tmp1 = NULL;
-	self->priv->_member_name = (_tmp2 = (_tmp1 = value, (_tmp1 == NULL ? NULL : g_strdup (_tmp1))), (self->priv->_member_name = (g_free (self->priv->_member_name), NULL)), _tmp2);
+	self->priv->_member_name = (_tmp2 = (_tmp1 = value, (_tmp1 == NULL) ? NULL : g_strdup (_tmp1)), self->priv->_member_name = (g_free (self->priv->_member_name), NULL), _tmp2);
 }
 
 
 gboolean vala_ccode_member_access_get_is_pointer (ValaCCodeMemberAccess* self) {
-	g_return_val_if_fail (VALA_IS_CCODE_MEMBER_ACCESS (self), FALSE);
+	g_return_val_if_fail (self != NULL, FALSE);
 	return self->priv->_is_pointer;
 }
 
 
 void vala_ccode_member_access_set_is_pointer (ValaCCodeMemberAccess* self, gboolean value) {
-	g_return_if_fail (VALA_IS_CCODE_MEMBER_ACCESS (self));
+	g_return_if_fail (self != NULL);
 	self->priv->_is_pointer = value;
 }
 
@@ -150,7 +149,7 @@ static void vala_ccode_member_access_instance_init (ValaCCodeMemberAccess * self
 static void vala_ccode_member_access_finalize (ValaCCodeNode* obj) {
 	ValaCCodeMemberAccess * self;
 	self = VALA_CCODE_MEMBER_ACCESS (obj);
-	(self->priv->_inner == NULL ? NULL : (self->priv->_inner = (vala_ccode_node_unref (self->priv->_inner), NULL)));
+	(self->priv->_inner == NULL) ? NULL : (self->priv->_inner = (vala_ccode_node_unref (self->priv->_inner), NULL));
 	self->priv->_member_name = (g_free (self->priv->_member_name), NULL);
 	VALA_CCODE_NODE_CLASS (vala_ccode_member_access_parent_class)->finalize (obj);
 }
