@@ -35,8 +35,8 @@ static gpointer vala_ccode_newline_parent_class = NULL;
 
 static void vala_ccode_newline_real_write (ValaCCodeNode* base, ValaCCodeWriter* writer) {
 	ValaCCodeNewline * self;
-	self = VALA_CCODE_NEWLINE (base);
-	g_return_if_fail (VALA_IS_CCODE_WRITER (writer));
+	self = (ValaCCodeNewline*) base;
+	g_return_if_fail (writer != NULL);
 	vala_ccode_writer_write_newline (writer);
 }
 
@@ -46,7 +46,7 @@ static void vala_ccode_newline_real_write (ValaCCodeNode* base, ValaCCodeWriter*
  */
 ValaCCodeNewline* vala_ccode_newline_construct (GType object_type) {
 	ValaCCodeNewline* self;
-	self = ((ValaCCodeNewline*) (g_type_create_instance (object_type)));
+	self = (ValaCCodeNewline*) g_type_create_instance (object_type);
 	return self;
 }
 
