@@ -92,7 +92,7 @@ static void vala_ccode_variable_declarator_real_write_initialization (ValaCCodeD
 	g_return_if_fail (writer != NULL);
 	_tmp0 = FALSE;
 	if (self->priv->_initializer != NULL) {
-		_tmp0 = !(VALA_IS_CCODE_INITIALIZER_LIST (self->priv->_initializer));
+		_tmp0 = !VALA_IS_CCODE_INITIALIZER_LIST (self->priv->_initializer);
 	} else {
 		_tmp0 = FALSE;
 	}
