@@ -25,6 +25,7 @@
 
 #include <glib.h>
 #include <glib-object.h>
+#include <gee/collectionobject.h>
 #include <gee/iterable.h>
 #include <gee/collection.h>
 #include <gee/iterator.h>
@@ -47,17 +48,21 @@ typedef struct _GeeReadOnlyCollectionPrivate GeeReadOnlyCollectionPrivate;
  * Represents a read-only collection of items.
  */
 struct _GeeReadOnlyCollection {
-	GObject parent_instance;
+	GeeCollectionObject parent_instance;
 	GeeReadOnlyCollectionPrivate * priv;
 };
+
 struct _GeeReadOnlyCollectionClass {
-	GObjectClass parent_class;
+	GeeCollectionObjectClass parent_class;
 };
 
+
+GeeReadOnlyCollection* gee_read_only_collection_construct (GType object_type, GType g_type, GBoxedCopyFunc g_dup_func, GDestroyNotify g_destroy_func, GeeCollection* collection);
 GeeReadOnlyCollection* gee_read_only_collection_new (GType g_type, GBoxedCopyFunc g_dup_func, GDestroyNotify g_destroy_func, GeeCollection* collection);
 void gee_read_only_collection_set_collection (GeeReadOnlyCollection* self, GeeCollection* value);
 GType gee_read_only_collection_get_type (void);
 
+
 G_END_DECLS
 
 #endif
