@@ -29,14 +29,15 @@
 #include <vala/valatypesymbol.h>
 #include <gee/iterable.h>
 #include <gee/iterator.h>
-#include <vala/valaformalparameter.h>
 #include <ccode/valaccodeformalparameter.h>
 #include <vala/valaarraytype.h>
 #include <vala/valasymbol.h>
 #include <gee/collection.h>
 #include <ccode/valaccodetypedefinition.h>
 #include <ccode/valaccodedeclarator.h>
+#include <vala/valasourcereference.h>
 #include <ccode/valaccodefragment.h>
+#include <ccode/valaccodecomment.h>
 #include <ccode/valaccodenode.h>
 #include <vala/valaunaryexpression.h>
 #include <vala/valamethodcall.h>
@@ -57,7 +58,6 @@
 #include <ccode/valaccodefunction.h>
 #include <ccode/valaccodemodifiers.h>
 #include <gee/hashmap.h>
-#include <gee/map.h>
 #include <float.h>
 #include <math.h>
 #include <gee/list.h>
@@ -67,6 +67,7 @@
 #include <ccode/valaccodeexpressionstatement.h>
 #include <ccode/valaccodereturnstatement.h>
 #include <gobject/valaccodebasemodule.h>
+#include <gobject/valaccodemethodmodule.h>
 
 
 
@@ -80,6 +81,7 @@ static ValaCCodeExpression* vala_ccode_delegate_module_real_get_delegate_target_
 static char* vala_ccode_delegate_module_real_get_delegate_target_destroy_notify_cname (ValaCCodeBaseModule* base, const char* delegate_cname);
 static ValaCCodeExpression* vala_ccode_delegate_module_real_get_implicit_cast_expression (ValaCCodeBaseModule* base, ValaCCodeExpression* source_cexpr, ValaDataType* expression_type, ValaDataType* target_type, ValaExpression* expr);
 static char* vala_ccode_delegate_module_generate_delegate_wrapper (ValaCCodeDelegateModule* self, ValaMethod* m, ValaDelegate* d);
+static void vala_ccode_delegate_module_real_generate_parameter (ValaCCodeMethodModule* base, ValaFormalParameter* param, GeeMap* cparam_map, GeeMap* carg_map);
 static gpointer vala_ccode_delegate_module_parent_class = NULL;
 
 
@@ -211,8 +213,34 @@ static void vala_ccode_delegate_module_real_visit_delegate (ValaCCodeModule* bas
 	_tmp16 = NULL;
 	ctypedef = (_tmp16 = vala_ccode_type_definition_new (_tmp15 = vala_data_type_get_cname (vala_delegate_get_return_type (d)), (ValaCCodeDeclarator*) cfundecl), _tmp15 = (g_free (_tmp15), NULL), _tmp16);
 	if (!vala_symbol_is_internal_symbol ((ValaSymbol*) d)) {
+		gboolean _tmp17;
+		_tmp17 = FALSE;
+		if (vala_code_node_get_source_reference ((ValaCodeNode*) d) != NULL) {
+			_tmp17 = vala_source_reference_get_comment (vala_code_node_get_source_reference ((ValaCodeNode*) d)) != NULL;
+		} else {
+			_tmp17 = FALSE;
+		}
+		if (_tmp17) {
+			ValaCCodeComment* _tmp18;
+			_tmp18 = NULL;
+			vala_ccode_fragment_append (((ValaCCodeBaseModule*) self)->header_type_declaration, (ValaCCodeNode*) (_tmp18 = vala_ccode_comment_new (vala_source_reference_get_comment (vala_code_node_get_source_reference ((ValaCodeNode*) d)))));
+			(_tmp18 == NULL) ? NULL : (_tmp18 = (vala_ccode_node_unref (_tmp18), NULL));
+		}
 		vala_ccode_fragment_append (((ValaCCodeBaseModule*) self)->header_type_declaration, (ValaCCodeNode*) ctypedef);
 	} else {
+		gboolean _tmp19;
+		_tmp19 = FALSE;
+		if (vala_code_node_get_source_reference ((ValaCodeNode*) d) != NULL) {
+			_tmp19 = vala_source_reference_get_comment (vala_code_node_get_source_reference ((ValaCodeNode*) d)) != NULL;
+		} else {
+			_tmp19 = FALSE;
+		}
+		if (_tmp19) {
+			ValaCCodeComment* _tmp20;
+			_tmp20 = NULL;
+			vala_ccode_fragment_append (((ValaCCodeBaseModule*) self)->source_type_declaration, (ValaCCodeNode*) (_tmp20 = vala_ccode_comment_new (vala_source_reference_get_comment (vala_code_node_get_source_reference ((ValaCodeNode*) d)))));
+			(_tmp20 == NULL) ? NULL : (_tmp20 = (vala_ccode_node_unref (_tmp20), NULL));
+		}
 		vala_ccode_fragment_append (((ValaCCodeBaseModule*) self)->source_type_declaration, (ValaCCodeNode*) ctypedef);
 	}
 	(cfundecl == NULL) ? NULL : (cfundecl = (vala_ccode_node_unref (cfundecl), NULL));
@@ -273,7 +301,7 @@ static ValaCCodeExpression* vala_ccode_delegate_module_real_get_delegate_target_
 			} else {
 				_tmp6 = FALSE;
 			}
-			if ((_tmp6)) {
+			if (_tmp6) {
 				_tmp5 = TRUE;
 			} else {
 				_tmp5 = ((ValaCCodeBaseModule*) self)->in_constructor;
@@ -341,67 +369,71 @@ static ValaCCodeExpression* vala_ccode_delegate_module_real_get_delegate_target_
 						if (VALA_IS_FIELD (vala_expression_get_symbol_reference (delegate_expr))) {
 							ValaField* _tmp20;
 							ValaField* field;
+							char* _tmp21;
+							char* _tmp22;
 							char* target_cname;
-							ValaMemberAccess* _tmp21;
+							ValaMemberAccess* _tmp23;
 							ValaMemberAccess* ma;
 							ValaCCodeExpression* target_expr;
 							_tmp20 = NULL;
 							field = (_tmp20 = VALA_FIELD (vala_expression_get_symbol_reference (delegate_expr)), (_tmp20 == NULL) ? NULL : vala_code_node_ref (_tmp20));
-							target_cname = vala_ccode_base_module_get_delegate_target_cname ((ValaCCodeBaseModule*) self, vala_symbol_get_name ((ValaSymbol*) field));
 							_tmp21 = NULL;
-							ma = (_tmp21 = VALA_MEMBER_ACCESS (delegate_expr), (_tmp21 == NULL) ? NULL : vala_code_node_ref (_tmp21));
+							_tmp22 = NULL;
+							target_cname = (_tmp22 = vala_ccode_base_module_get_delegate_target_cname ((ValaCCodeBaseModule*) self, _tmp21 = vala_field_get_cname (field)), _tmp21 = (g_free (_tmp21), NULL), _tmp22);
+							_tmp23 = NULL;
+							ma = (_tmp23 = VALA_MEMBER_ACCESS (delegate_expr), (_tmp23 == NULL) ? NULL : vala_code_node_ref (_tmp23));
 							target_expr = NULL;
 							if (vala_field_get_binding (field) == MEMBER_BINDING_INSTANCE) {
-								ValaDataType* _tmp22;
+								ValaDataType* _tmp24;
 								ValaDataType* instance_expression_type;
 								ValaDataType* instance_target_type;
 								ValaCCodeExpression* pub_inst;
 								ValaCCodeExpression* typed_inst;
 								ValaCCodeExpression* inst;
-								_tmp22 = NULL;
-								instance_expression_type = (_tmp22 = vala_expression_get_value_type (vala_member_access_get_inner (ma)), (_tmp22 == NULL) ? NULL : vala_code_node_ref (_tmp22));
+								_tmp24 = NULL;
+								instance_expression_type = (_tmp24 = vala_expression_get_value_type (vala_member_access_get_inner (ma)), (_tmp24 == NULL) ? NULL : vala_code_node_ref (_tmp24));
 								instance_target_type = vala_ccode_base_module_get_data_type_for_symbol (VALA_TYPESYMBOL (vala_symbol_get_parent_symbol ((ValaSymbol*) field)));
 								pub_inst = VALA_CCODE_EXPRESSION (vala_ccode_base_module_get_ccodenode ((ValaCCodeBaseModule*) self, (ValaCodeNode*) vala_member_access_get_inner (ma)));
 								typed_inst = vala_ccode_base_module_transform_expression ((ValaCCodeBaseModule*) self, pub_inst, instance_expression_type, instance_target_type, NULL);
 								inst = NULL;
 								if (vala_symbol_get_access ((ValaSymbol*) field) == VALA_SYMBOL_ACCESSIBILITY_PRIVATE) {
-									ValaCCodeExpression* _tmp23;
-									_tmp23 = NULL;
-									inst = (_tmp23 = (ValaCCodeExpression*) vala_ccode_member_access_new_pointer (typed_inst, "priv"), (inst == NULL) ? NULL : (inst = (vala_ccode_node_unref (inst), NULL)), _tmp23);
-								} else {
 									ValaCCodeExpression* _tmp25;
-									ValaCCodeExpression* _tmp24;
 									_tmp25 = NULL;
-									_tmp24 = NULL;
-									inst = (_tmp25 = (_tmp24 = typed_inst, (_tmp24 == NULL) ? NULL : vala_ccode_node_ref (_tmp24)), (inst == NULL) ? NULL : (inst = (vala_ccode_node_unref (inst), NULL)), _tmp25);
-								}
-								if (vala_typesymbol_is_reference_type ((VALA_TYPESYMBOL (vala_symbol_get_parent_symbol ((ValaSymbol*) field))))) {
-									ValaCCodeExpression* _tmp26;
-									_tmp26 = NULL;
-									target_expr = (_tmp26 = (ValaCCodeExpression*) vala_ccode_member_access_new_pointer (inst, target_cname), (target_expr == NULL) ? NULL : (target_expr = (vala_ccode_node_unref (target_expr), NULL)), _tmp26);
+									inst = (_tmp25 = (ValaCCodeExpression*) vala_ccode_member_access_new_pointer (typed_inst, "priv"), (inst == NULL) ? NULL : (inst = (vala_ccode_node_unref (inst), NULL)), _tmp25);
 								} else {
 									ValaCCodeExpression* _tmp27;
+									ValaCCodeExpression* _tmp26;
 									_tmp27 = NULL;
-									target_expr = (_tmp27 = (ValaCCodeExpression*) vala_ccode_member_access_new (inst, target_cname, FALSE), (target_expr == NULL) ? NULL : (target_expr = (vala_ccode_node_unref (target_expr), NULL)), _tmp27);
+									_tmp26 = NULL;
+									inst = (_tmp27 = (_tmp26 = typed_inst, (_tmp26 == NULL) ? NULL : vala_ccode_node_ref (_tmp26)), (inst == NULL) ? NULL : (inst = (vala_ccode_node_unref (inst), NULL)), _tmp27);
+								}
+								if (vala_typesymbol_is_reference_type (VALA_TYPESYMBOL (vala_symbol_get_parent_symbol ((ValaSymbol*) field)))) {
+									ValaCCodeExpression* _tmp28;
+									_tmp28 = NULL;
+									target_expr = (_tmp28 = (ValaCCodeExpression*) vala_ccode_member_access_new_pointer (inst, target_cname), (target_expr == NULL) ? NULL : (target_expr = (vala_ccode_node_unref (target_expr), NULL)), _tmp28);
+								} else {
+									ValaCCodeExpression* _tmp29;
+									_tmp29 = NULL;
+									target_expr = (_tmp29 = (ValaCCodeExpression*) vala_ccode_member_access_new (inst, target_cname, FALSE), (target_expr == NULL) ? NULL : (target_expr = (vala_ccode_node_unref (target_expr), NULL)), _tmp29);
 								}
 								(instance_expression_type == NULL) ? NULL : (instance_expression_type = (vala_code_node_unref (instance_expression_type), NULL));
 								(instance_target_type == NULL) ? NULL : (instance_target_type = (vala_code_node_unref (instance_target_type), NULL));
 								(pub_inst == NULL) ? NULL : (pub_inst = (vala_ccode_node_unref (pub_inst), NULL));
 								(typed_inst == NULL) ? NULL : (typed_inst = (vala_ccode_node_unref (typed_inst), NULL));
 								(inst == NULL) ? NULL : (inst = (vala_ccode_node_unref (inst), NULL));
-							} else {
-								ValaCCodeExpression* _tmp28;
-								_tmp28 = NULL;
-								target_expr = (_tmp28 = (ValaCCodeExpression*) vala_ccode_identifier_new (target_cname), (target_expr == NULL) ? NULL : (target_expr = (vala_ccode_node_unref (target_expr), NULL)), _tmp28);
-							}
-							if (is_out) {
-								ValaCCodeExpression* _tmp29;
-								_tmp29 = NULL;
-								return (_tmp29 = (ValaCCodeExpression*) vala_ccode_unary_expression_new (VALA_CCODE_UNARY_OPERATOR_ADDRESS_OF, target_expr), (field == NULL) ? NULL : (field = (vala_code_node_unref (field), NULL)), target_cname = (g_free (target_cname), NULL), (ma == NULL) ? NULL : (ma = (vala_code_node_unref (ma), NULL)), (target_expr == NULL) ? NULL : (target_expr = (vala_ccode_node_unref (target_expr), NULL)), _tmp29);
 							} else {
 								ValaCCodeExpression* _tmp30;
 								_tmp30 = NULL;
-								return (_tmp30 = target_expr, (field == NULL) ? NULL : (field = (vala_code_node_unref (field), NULL)), target_cname = (g_free (target_cname), NULL), (ma == NULL) ? NULL : (ma = (vala_code_node_unref (ma), NULL)), _tmp30);
+								target_expr = (_tmp30 = (ValaCCodeExpression*) vala_ccode_identifier_new (target_cname), (target_expr == NULL) ? NULL : (target_expr = (vala_ccode_node_unref (target_expr), NULL)), _tmp30);
+							}
+							if (is_out) {
+								ValaCCodeExpression* _tmp31;
+								_tmp31 = NULL;
+								return (_tmp31 = (ValaCCodeExpression*) vala_ccode_unary_expression_new (VALA_CCODE_UNARY_OPERATOR_ADDRESS_OF, target_expr), (field == NULL) ? NULL : (field = (vala_code_node_unref (field), NULL)), target_cname = (g_free (target_cname), NULL), (ma == NULL) ? NULL : (ma = (vala_code_node_unref (ma), NULL)), (target_expr == NULL) ? NULL : (target_expr = (vala_ccode_node_unref (target_expr), NULL)), _tmp31);
+							} else {
+								ValaCCodeExpression* _tmp32;
+								_tmp32 = NULL;
+								return (_tmp32 = target_expr, (field == NULL) ? NULL : (field = (vala_code_node_unref (field), NULL)), target_cname = (g_free (target_cname), NULL), (ma == NULL) ? NULL : (ma = (vala_code_node_unref (ma), NULL)), _tmp32);
 							}
 							(field == NULL) ? NULL : (field = (vala_code_node_unref (field), NULL));
 							target_cname = (g_free (target_cname), NULL);
@@ -409,22 +441,22 @@ static ValaCCodeExpression* vala_ccode_delegate_module_real_get_delegate_target_
 							(target_expr == NULL) ? NULL : (target_expr = (vala_ccode_node_unref (target_expr), NULL));
 						} else {
 							if (VALA_IS_METHOD (vala_expression_get_symbol_reference (delegate_expr))) {
-								ValaMethod* _tmp31;
+								ValaMethod* _tmp33;
 								ValaMethod* m;
-								ValaMemberAccess* _tmp32;
+								ValaMemberAccess* _tmp34;
 								ValaMemberAccess* ma;
-								_tmp31 = NULL;
-								m = (_tmp31 = VALA_METHOD (vala_expression_get_symbol_reference (delegate_expr)), (_tmp31 == NULL) ? NULL : vala_code_node_ref (_tmp31));
-								_tmp32 = NULL;
-								ma = (_tmp32 = VALA_MEMBER_ACCESS (delegate_expr), (_tmp32 == NULL) ? NULL : vala_code_node_ref (_tmp32));
+								_tmp33 = NULL;
+								m = (_tmp33 = VALA_METHOD (vala_expression_get_symbol_reference (delegate_expr)), (_tmp33 == NULL) ? NULL : vala_code_node_ref (_tmp33));
+								_tmp34 = NULL;
+								ma = (_tmp34 = VALA_MEMBER_ACCESS (delegate_expr), (_tmp34 == NULL) ? NULL : vala_code_node_ref (_tmp34));
 								if (vala_method_get_binding (m) == MEMBER_BINDING_STATIC) {
-									ValaCCodeExpression* _tmp33;
-									_tmp33 = NULL;
-									return (_tmp33 = (ValaCCodeExpression*) vala_ccode_constant_new ("NULL"), (m == NULL) ? NULL : (m = (vala_code_node_unref (m), NULL)), (ma == NULL) ? NULL : (ma = (vala_code_node_unref (ma), NULL)), _tmp33);
+									ValaCCodeExpression* _tmp35;
+									_tmp35 = NULL;
+									return (_tmp35 = (ValaCCodeExpression*) vala_ccode_constant_new ("NULL"), (m == NULL) ? NULL : (m = (vala_code_node_unref (m), NULL)), (ma == NULL) ? NULL : (ma = (vala_code_node_unref (ma), NULL)), _tmp35);
 								} else {
-									ValaCCodeExpression* _tmp34;
-									_tmp34 = NULL;
-									return (_tmp34 = VALA_CCODE_EXPRESSION (vala_ccode_base_module_get_ccodenode ((ValaCCodeBaseModule*) self, (ValaCodeNode*) vala_member_access_get_inner (ma))), (m == NULL) ? NULL : (m = (vala_code_node_unref (m), NULL)), (ma == NULL) ? NULL : (ma = (vala_code_node_unref (ma), NULL)), _tmp34);
+									ValaCCodeExpression* _tmp36;
+									_tmp36 = NULL;
+									return (_tmp36 = VALA_CCODE_EXPRESSION (vala_ccode_base_module_get_ccodenode ((ValaCCodeBaseModule*) self, (ValaCodeNode*) vala_member_access_get_inner (ma))), (m == NULL) ? NULL : (m = (vala_code_node_unref (m), NULL)), (ma == NULL) ? NULL : (ma = (vala_code_node_unref (ma), NULL)), _tmp36);
 								}
 								(m == NULL) ? NULL : (m = (vala_code_node_unref (m), NULL));
 								(ma == NULL) ? NULL : (ma = (vala_code_node_unref (ma), NULL));
@@ -700,7 +732,7 @@ static char* vala_ccode_delegate_module_generate_delegate_wrapper (ValaCCodeDele
 					} else {
 						_tmp29 = pos < min_pos;
 					}
-					_tmp28 = (_tmp29);
+					_tmp28 = _tmp29;
 				} else {
 					_tmp28 = FALSE;
 				}
@@ -736,7 +768,7 @@ static char* vala_ccode_delegate_module_generate_delegate_wrapper (ValaCCodeDele
 			_tmp34 = NULL;
 			_tmp32 = NULL;
 			_tmp33 = NULL;
-			arg = (_tmp34 = (ValaCCodeExpression*) vala_ccode_identifier_new (vala_ccode_formal_parameter_get_name (((_tmp33 = vala_code_node_get_ccodenode ((ValaCodeNode*) (_tmp32 = (ValaFormalParameter*) gee_list_get (d_params, 0))), VALA_IS_CCODE_FORMAL_PARAMETER (_tmp33) ? ((ValaCCodeFormalParameter*) _tmp33) : NULL)))), (arg == NULL) ? NULL : (arg = (vala_ccode_node_unref (arg), NULL)), _tmp34);
+			arg = (_tmp34 = (ValaCCodeExpression*) vala_ccode_identifier_new (vala_ccode_formal_parameter_get_name ((_tmp33 = vala_code_node_get_ccodenode ((ValaCodeNode*) (_tmp32 = (ValaFormalParameter*) gee_list_get (d_params, 0))), VALA_IS_CCODE_FORMAL_PARAMETER (_tmp33) ? ((ValaCCodeFormalParameter*) _tmp33) : NULL))), (arg == NULL) ? NULL : (arg = (vala_ccode_node_unref (arg), NULL)), _tmp34);
 			(_tmp32 == NULL) ? NULL : (_tmp32 = (vala_code_node_unref (_tmp32), NULL));
 			i = 1;
 		}
@@ -762,7 +794,7 @@ static char* vala_ccode_delegate_module_generate_delegate_wrapper (ValaCCodeDele
 			_tmp39 = NULL;
 			_tmp37 = NULL;
 			_tmp38 = NULL;
-			arg = (_tmp39 = (ValaCCodeExpression*) vala_ccode_identifier_new (vala_ccode_formal_parameter_get_name (((_tmp38 = vala_code_node_get_ccodenode ((ValaCodeNode*) (_tmp37 = (ValaFormalParameter*) gee_list_get (d_params, i))), VALA_IS_CCODE_FORMAL_PARAMETER (_tmp38) ? ((ValaCCodeFormalParameter*) _tmp38) : NULL)))), (arg == NULL) ? NULL : (arg = (vala_ccode_node_unref (arg), NULL)), _tmp39);
+			arg = (_tmp39 = (ValaCCodeExpression*) vala_ccode_identifier_new (vala_ccode_formal_parameter_get_name ((_tmp38 = vala_code_node_get_ccodenode ((ValaCodeNode*) (_tmp37 = (ValaFormalParameter*) gee_list_get (d_params, i))), VALA_IS_CCODE_FORMAL_PARAMETER (_tmp38) ? ((ValaCCodeFormalParameter*) _tmp38) : NULL))), (arg == NULL) ? NULL : (arg = (vala_ccode_node_unref (arg), NULL)), _tmp39);
 			(_tmp37 == NULL) ? NULL : (_tmp37 = (vala_code_node_unref (_tmp37), NULL));
 			gee_map_set ((GeeMap*) carg_map, GINT_TO_POINTER (vala_ccode_base_module_get_param_pos ((ValaCCodeBaseModule*) self, vala_formal_parameter_get_cparameter_position (param), FALSE)), arg);
 			_tmp40 = FALSE;
@@ -910,7 +942,7 @@ static char* vala_ccode_delegate_module_generate_delegate_wrapper (ValaCCodeDele
 					} else {
 						_tmp70 = pos < min_pos;
 					}
-					_tmp69 = (_tmp70);
+					_tmp69 = _tmp70;
 				} else {
 					_tmp69 = FALSE;
 				}
@@ -951,6 +983,92 @@ static char* vala_ccode_delegate_module_generate_delegate_wrapper (ValaCCodeDele
 }
 
 
+static void vala_ccode_delegate_module_real_generate_parameter (ValaCCodeMethodModule* base, ValaFormalParameter* param, GeeMap* cparam_map, GeeMap* carg_map) {
+	ValaCCodeDelegateModule * self;
+	gboolean _tmp0;
+	self = (ValaCCodeDelegateModule*) base;
+	g_return_if_fail (param != NULL);
+	g_return_if_fail (cparam_map != NULL);
+	_tmp0 = FALSE;
+	if (VALA_IS_DELEGATE_TYPE (vala_formal_parameter_get_parameter_type (param))) {
+		_tmp0 = TRUE;
+	} else {
+		_tmp0 = VALA_IS_METHOD_TYPE (vala_formal_parameter_get_parameter_type (param));
+	}
+	if (!_tmp0) {
+		VALA_CCODE_METHOD_MODULE_CLASS (vala_ccode_delegate_module_parent_class)->generate_parameter ((ValaCCodeMethodModule*) VALA_CCODE_ARRAY_MODULE (self), param, cparam_map, carg_map);
+		return;
+	}
+	gee_map_set (cparam_map, GINT_TO_POINTER (vala_ccode_base_module_get_param_pos ((ValaCCodeBaseModule*) self, vala_formal_parameter_get_cparameter_position (param), FALSE)), VALA_CCODE_FORMAL_PARAMETER (vala_code_node_get_ccodenode ((ValaCodeNode*) param)));
+	if (carg_map != NULL) {
+		ValaCCodeIdentifier* _tmp1;
+		_tmp1 = NULL;
+		gee_map_set (carg_map, GINT_TO_POINTER (vala_ccode_base_module_get_param_pos ((ValaCCodeBaseModule*) self, vala_formal_parameter_get_cparameter_position (param), FALSE)), (ValaCCodeExpression*) (_tmp1 = vala_ccode_identifier_new (vala_symbol_get_name ((ValaSymbol*) param))));
+		(_tmp1 == NULL) ? NULL : (_tmp1 = (vala_ccode_node_unref (_tmp1), NULL));
+	}
+	if (VALA_IS_DELEGATE_TYPE (vala_formal_parameter_get_parameter_type (param))) {
+		ValaDelegateType* _tmp2;
+		ValaDelegateType* deleg_type;
+		ValaDelegate* _tmp3;
+		ValaDelegate* d;
+		_tmp2 = NULL;
+		deleg_type = (_tmp2 = VALA_DELEGATE_TYPE (vala_formal_parameter_get_parameter_type (param)), (_tmp2 == NULL) ? NULL : vala_code_node_ref (_tmp2));
+		_tmp3 = NULL;
+		d = (_tmp3 = vala_delegate_type_get_delegate_symbol (deleg_type), (_tmp3 == NULL) ? NULL : vala_code_node_ref (_tmp3));
+		if (vala_delegate_get_has_target (d)) {
+			char* _tmp4;
+			ValaCCodeFormalParameter* _tmp5;
+			ValaCCodeFormalParameter* cparam;
+			_tmp4 = NULL;
+			_tmp5 = NULL;
+			cparam = (_tmp5 = vala_ccode_formal_parameter_new (_tmp4 = vala_ccode_base_module_get_delegate_target_cname ((ValaCCodeBaseModule*) self, vala_symbol_get_name ((ValaSymbol*) param)), "void*"), _tmp4 = (g_free (_tmp4), NULL), _tmp5);
+			gee_map_set (cparam_map, GINT_TO_POINTER (vala_ccode_base_module_get_param_pos ((ValaCCodeBaseModule*) self, vala_formal_parameter_get_cdelegate_target_parameter_position (param), FALSE)), cparam);
+			if (carg_map != NULL) {
+				ValaCCodeIdentifier* _tmp6;
+				_tmp6 = NULL;
+				gee_map_set (carg_map, GINT_TO_POINTER (vala_ccode_base_module_get_param_pos ((ValaCCodeBaseModule*) self, vala_formal_parameter_get_cdelegate_target_parameter_position (param), FALSE)), (ValaCCodeExpression*) (_tmp6 = vala_ccode_identifier_new (vala_ccode_formal_parameter_get_name (cparam))));
+				(_tmp6 == NULL) ? NULL : (_tmp6 = (vala_ccode_node_unref (_tmp6), NULL));
+			}
+			if (vala_data_type_get_value_owned ((ValaDataType*) deleg_type)) {
+				ValaCCodeFormalParameter* _tmp8;
+				char* _tmp7;
+				_tmp8 = NULL;
+				_tmp7 = NULL;
+				cparam = (_tmp8 = vala_ccode_formal_parameter_new (_tmp7 = vala_ccode_base_module_get_delegate_target_destroy_notify_cname ((ValaCCodeBaseModule*) self, vala_symbol_get_name ((ValaSymbol*) param)), "GDestroyNotify"), (cparam == NULL) ? NULL : (cparam = (vala_ccode_node_unref (cparam), NULL)), _tmp8);
+				_tmp7 = (g_free (_tmp7), NULL);
+				gee_map_set (cparam_map, GINT_TO_POINTER (vala_ccode_base_module_get_param_pos ((ValaCCodeBaseModule*) self, vala_formal_parameter_get_cdelegate_target_parameter_position (param) + 0.01, FALSE)), cparam);
+				if (carg_map != NULL) {
+					ValaCCodeIdentifier* _tmp9;
+					_tmp9 = NULL;
+					gee_map_set (carg_map, GINT_TO_POINTER (vala_ccode_base_module_get_param_pos ((ValaCCodeBaseModule*) self, vala_formal_parameter_get_cdelegate_target_parameter_position (param) + 0.01, FALSE)), (ValaCCodeExpression*) (_tmp9 = vala_ccode_identifier_new (vala_ccode_formal_parameter_get_name (cparam))));
+					(_tmp9 == NULL) ? NULL : (_tmp9 = (vala_ccode_node_unref (_tmp9), NULL));
+				}
+			}
+			(cparam == NULL) ? NULL : (cparam = (vala_ccode_node_unref (cparam), NULL));
+		}
+		(deleg_type == NULL) ? NULL : (deleg_type = (vala_code_node_unref (deleg_type), NULL));
+		(d == NULL) ? NULL : (d = (vala_code_node_unref (d), NULL));
+	} else {
+		if (VALA_IS_METHOD_TYPE (vala_formal_parameter_get_parameter_type (param))) {
+			char* _tmp10;
+			ValaCCodeFormalParameter* _tmp11;
+			ValaCCodeFormalParameter* cparam;
+			_tmp10 = NULL;
+			_tmp11 = NULL;
+			cparam = (_tmp11 = vala_ccode_formal_parameter_new (_tmp10 = vala_ccode_base_module_get_delegate_target_cname ((ValaCCodeBaseModule*) self, vala_symbol_get_name ((ValaSymbol*) param)), "void*"), _tmp10 = (g_free (_tmp10), NULL), _tmp11);
+			gee_map_set (cparam_map, GINT_TO_POINTER (vala_ccode_base_module_get_param_pos ((ValaCCodeBaseModule*) self, vala_formal_parameter_get_cdelegate_target_parameter_position (param), FALSE)), cparam);
+			if (carg_map != NULL) {
+				ValaCCodeIdentifier* _tmp12;
+				_tmp12 = NULL;
+				gee_map_set (carg_map, GINT_TO_POINTER (vala_ccode_base_module_get_param_pos ((ValaCCodeBaseModule*) self, vala_formal_parameter_get_cdelegate_target_parameter_position (param), FALSE)), (ValaCCodeExpression*) (_tmp12 = vala_ccode_identifier_new (vala_ccode_formal_parameter_get_name (cparam))));
+				(_tmp12 == NULL) ? NULL : (_tmp12 = (vala_ccode_node_unref (_tmp12), NULL));
+			}
+			(cparam == NULL) ? NULL : (cparam = (vala_ccode_node_unref (cparam), NULL));
+		}
+	}
+}
+
+
 static void vala_ccode_delegate_module_class_init (ValaCCodeDelegateModuleClass * klass) {
 	vala_ccode_delegate_module_parent_class = g_type_class_peek_parent (klass);
 	VALA_CCODE_MODULE_CLASS (klass)->visit_delegate = vala_ccode_delegate_module_real_visit_delegate;
@@ -958,6 +1076,7 @@ static void vala_ccode_delegate_module_class_init (ValaCCodeDelegateModuleClass
 	VALA_CCODE_BASE_MODULE_CLASS (klass)->get_delegate_target_cexpression = vala_ccode_delegate_module_real_get_delegate_target_cexpression;
 	VALA_CCODE_BASE_MODULE_CLASS (klass)->get_delegate_target_destroy_notify_cname = vala_ccode_delegate_module_real_get_delegate_target_destroy_notify_cname;
 	VALA_CCODE_BASE_MODULE_CLASS (klass)->get_implicit_cast_expression = vala_ccode_delegate_module_real_get_implicit_cast_expression;
+	VALA_CCODE_METHOD_MODULE_CLASS (klass)->generate_parameter = vala_ccode_delegate_module_real_generate_parameter;
 }
 
 
