@@ -89,7 +89,6 @@ static void vala_ccode_module_real_visit_integer_literal (ValaCCodeModule* self,
 static void vala_ccode_module_real_visit_real_literal (ValaCCodeModule* self, ValaRealLiteral* expr);
 static void vala_ccode_module_real_visit_string_literal (ValaCCodeModule* self, ValaStringLiteral* expr);
 static void vala_ccode_module_real_visit_null_literal (ValaCCodeModule* self, ValaNullLiteral* expr);
-static void vala_ccode_module_real_visit_parenthesized_expression (ValaCCodeModule* self, ValaParenthesizedExpression* expr);
 static void vala_ccode_module_real_visit_member_access (ValaCCodeModule* self, ValaMemberAccess* expr);
 static void vala_ccode_module_real_visit_method_call (ValaCCodeModule* self, ValaMethodCall* expr);
 static void vala_ccode_module_real_visit_element_access (ValaCCodeModule* self, ValaElementAccess* expr);
@@ -107,7 +106,7 @@ static void vala_ccode_module_real_visit_binary_expression (ValaCCodeModule* sel
 static void vala_ccode_module_real_visit_type_check (ValaCCodeModule* self, ValaTypeCheck* expr);
 static void vala_ccode_module_real_visit_lambda_expression (ValaCCodeModule* self, ValaLambdaExpression* l);
 static void vala_ccode_module_real_visit_assignment (ValaCCodeModule* self, ValaAssignment* a);
-static void vala_ccode_module_real_generate_cparameters (ValaCCodeModule* self, ValaMethod* m, ValaDataType* creturn_type, gboolean in_gtypeinstance_creation_method, GeeMap* cparam_map, ValaCCodeFunction* func, ValaCCodeFunctionDeclarator* vdeclarator, GeeMap* carg_map, ValaCCodeFunctionCall* vcall, gint direction);
+static void vala_ccode_module_real_generate_cparameters (ValaCCodeModule* self, ValaMethod* m, GeeMap* cparam_map, ValaCCodeFunction* func, ValaCCodeFunctionDeclarator* vdeclarator, GeeMap* carg_map, ValaCCodeFunctionCall* vcall, gint direction);
 static char* vala_ccode_module_real_get_custom_creturn_type (ValaCCodeModule* self, ValaMethod* m);
 static void vala_ccode_module_real_generate_dynamic_method_wrapper (ValaCCodeModule* self, ValaDynamicMethod* method);
 static gboolean vala_ccode_module_real_method_has_wrapper (ValaCCodeModule* self, ValaMethod* method);
@@ -791,18 +790,6 @@ void vala_ccode_module_visit_null_literal (ValaCCodeModule* self, ValaNullLitera
 }
 
 
-static void vala_ccode_module_real_visit_parenthesized_expression (ValaCCodeModule* self, ValaParenthesizedExpression* expr) {
-	g_return_if_fail (self != NULL);
-	g_return_if_fail (expr != NULL);
-	vala_ccode_module_visit_parenthesized_expression (self->priv->next, expr);
-}
-
-
-void vala_ccode_module_visit_parenthesized_expression (ValaCCodeModule* self, ValaParenthesizedExpression* expr) {
-	VALA_CCODE_MODULE_GET_CLASS (self)->visit_parenthesized_expression (self, expr);
-}
-
-
 static void vala_ccode_module_real_visit_member_access (ValaCCodeModule* self, ValaMemberAccess* expr) {
 	g_return_if_fail (self != NULL);
 	g_return_if_fail (expr != NULL);
@@ -1007,18 +994,17 @@ void vala_ccode_module_visit_assignment (ValaCCodeModule* self, ValaAssignment*
 }
 
 
-static void vala_ccode_module_real_generate_cparameters (ValaCCodeModule* self, ValaMethod* m, ValaDataType* creturn_type, gboolean in_gtypeinstance_creation_method, GeeMap* cparam_map, ValaCCodeFunction* func, ValaCCodeFunctionDeclarator* vdeclarator, GeeMap* carg_map, ValaCCodeFunctionCall* vcall, gint direction) {
+static void vala_ccode_module_real_generate_cparameters (ValaCCodeModule* self, ValaMethod* m, GeeMap* cparam_map, ValaCCodeFunction* func, ValaCCodeFunctionDeclarator* vdeclarator, GeeMap* carg_map, ValaCCodeFunctionCall* vcall, gint direction) {
 	g_return_if_fail (self != NULL);
 	g_return_if_fail (m != NULL);
-	g_return_if_fail (creturn_type != NULL);
 	g_return_if_fail (cparam_map != NULL);
 	g_return_if_fail (func != NULL);
-	vala_ccode_module_generate_cparameters (self->priv->next, m, creturn_type, in_gtypeinstance_creation_method, cparam_map, func, vdeclarator, carg_map, vcall, direction);
+	vala_ccode_module_generate_cparameters (self->priv->next, m, cparam_map, func, vdeclarator, carg_map, vcall, direction);
 }
 
 
-void vala_ccode_module_generate_cparameters (ValaCCodeModule* self, ValaMethod* m, ValaDataType* creturn_type, gboolean in_gtypeinstance_creation_method, GeeMap* cparam_map, ValaCCodeFunction* func, ValaCCodeFunctionDeclarator* vdeclarator, GeeMap* carg_map, ValaCCodeFunctionCall* vcall, gint direction) {
-	VALA_CCODE_MODULE_GET_CLASS (self)->generate_cparameters (self, m, creturn_type, in_gtypeinstance_creation_method, cparam_map, func, vdeclarator, carg_map, vcall, direction);
+void vala_ccode_module_generate_cparameters (ValaCCodeModule* self, ValaMethod* m, GeeMap* cparam_map, ValaCCodeFunction* func, ValaCCodeFunctionDeclarator* vdeclarator, GeeMap* carg_map, ValaCCodeFunctionCall* vcall, gint direction) {
+	VALA_CCODE_MODULE_GET_CLASS (self)->generate_cparameters (self, m, cparam_map, func, vdeclarator, carg_map, vcall, direction);
 }
 
 
@@ -1446,7 +1432,6 @@ static void vala_ccode_module_class_init (ValaCCodeModuleClass * klass) {
 	VALA_CCODE_MODULE_CLASS (klass)->visit_real_literal = vala_ccode_module_real_visit_real_literal;
 	VALA_CCODE_MODULE_CLASS (klass)->visit_string_literal = vala_ccode_module_real_visit_string_literal;
 	VALA_CCODE_MODULE_CLASS (klass)->visit_null_literal = vala_ccode_module_real_visit_null_literal;
-	VALA_CCODE_MODULE_CLASS (klass)->visit_parenthesized_expression = vala_ccode_module_real_visit_parenthesized_expression;
 	VALA_CCODE_MODULE_CLASS (klass)->visit_member_access = vala_ccode_module_real_visit_member_access;
 	VALA_CCODE_MODULE_CLASS (klass)->visit_method_call = vala_ccode_module_real_visit_method_call;
 	VALA_CCODE_MODULE_CLASS (klass)->visit_element_access = vala_ccode_module_real_visit_element_access;
