@@ -37,18 +37,27 @@
 #include <vala/valasignal.h>
 #include <vala/valatypesymbol.h>
 #include <vala/valaobjecttypesymbol.h>
+#include <vala/valastruct.h>
 #include <ccode/valaccodeidentifier.h>
 #include <vala/valadatatype.h>
 #include <vala/valadynamicproperty.h>
 #include <gee/list.h>
+#include <vala/valaelementaccess.h>
 #include <ccode/valaccodeconstant.h>
+#include <vala/valaassignment.h>
 #include <vala/valamethod.h>
+#include <vala/valamemberaccess.h>
+#include <gee/map.h>
+#include <ccode/valaccodefunction.h>
+#include <ccode/valaccodefunctiondeclarator.h>
+#include <vala/valareturnstatement.h>
+#include <vala/valacastexpression.h>
+#include <vala/valainterface.h>
 #include <vala/valadynamicmethod.h>
+#include <vala/valamethodcall.h>
 #include <vala/valacodecontext.h>
 #include <vala/valasourcefile.h>
 #include <vala/valaclass.h>
-#include <vala/valainterface.h>
-#include <vala/valastruct.h>
 #include <vala/valaenum.h>
 #include <vala/valaenumvalue.h>
 #include <vala/valaerrordomain.h>
@@ -78,7 +87,6 @@
 #include <vala/valaforeachstatement.h>
 #include <vala/valabreakstatement.h>
 #include <vala/valacontinuestatement.h>
-#include <vala/valareturnstatement.h>
 #include <vala/valayieldstatement.h>
 #include <vala/valathrowstatement.h>
 #include <vala/valatrystatement.h>
@@ -92,27 +100,18 @@
 #include <vala/valarealliteral.h>
 #include <vala/valastringliteral.h>
 #include <vala/valanullliteral.h>
-#include <vala/valaparenthesizedexpression.h>
-#include <vala/valamemberaccess.h>
-#include <vala/valamethodcall.h>
-#include <vala/valaelementaccess.h>
 #include <vala/valabaseaccess.h>
 #include <vala/valapostfixexpression.h>
 #include <vala/valaobjectcreationexpression.h>
 #include <vala/valasizeofexpression.h>
 #include <vala/valatypeofexpression.h>
 #include <vala/valaunaryexpression.h>
-#include <vala/valacastexpression.h>
 #include <vala/valapointerindirection.h>
 #include <vala/valaaddressofexpression.h>
 #include <vala/valareferencetransferexpression.h>
 #include <vala/valabinaryexpression.h>
 #include <vala/valatypecheck.h>
 #include <vala/valalambdaexpression.h>
-#include <vala/valaassignment.h>
-#include <gee/map.h>
-#include <ccode/valaccodefunction.h>
-#include <ccode/valaccodefunctiondeclarator.h>
 #include <gobject/valaccodegenerator.h>
 
 G_BEGIN_DECLS
@@ -193,7 +192,6 @@ struct _ValaCCodeModuleClass {
 	void (*visit_real_literal) (ValaCCodeModule* self, ValaRealLiteral* expr);
 	void (*visit_string_literal) (ValaCCodeModule* self, ValaStringLiteral* expr);
 	void (*visit_null_literal) (ValaCCodeModule* self, ValaNullLiteral* expr);
-	void (*visit_parenthesized_expression) (ValaCCodeModule* self, ValaParenthesizedExpression* expr);
 	void (*visit_member_access) (ValaCCodeModule* self, ValaMemberAccess* expr);
 	void (*visit_method_call) (ValaCCodeModule* self, ValaMethodCall* expr);
 	void (*visit_element_access) (ValaCCodeModule* self, ValaElementAccess* expr);
@@ -211,7 +209,7 @@ struct _ValaCCodeModuleClass {
 	void (*visit_type_check) (ValaCCodeModule* self, ValaTypeCheck* expr);
 	void (*visit_lambda_expression) (ValaCCodeModule* self, ValaLambdaExpression* l);
 	void (*visit_assignment) (ValaCCodeModule* self, ValaAssignment* a);
-	void (*generate_cparameters) (ValaCCodeModule* self, ValaMethod* m, ValaDataType* creturn_type, gboolean in_gtypeinstance_creation_method, GeeMap* cparam_map, ValaCCodeFunction* func, ValaCCodeFunctionDeclarator* vdeclarator, GeeMap* carg_map, ValaCCodeFunctionCall* vcall, gint direction);
+	void (*generate_cparameters) (ValaCCodeModule* self, ValaMethod* m, GeeMap* cparam_map, ValaCCodeFunction* func, ValaCCodeFunctionDeclarator* vdeclarator, GeeMap* carg_map, ValaCCodeFunctionCall* vcall, gint direction);
 	char* (*get_custom_creturn_type) (ValaCCodeModule* self, ValaMethod* m);
 	void (*generate_dynamic_method_wrapper) (ValaCCodeModule* self, ValaDynamicMethod* method);
 	gboolean (*method_has_wrapper) (ValaCCodeModule* self, ValaMethod* method);
@@ -294,7 +292,6 @@ void vala_ccode_module_visit_integer_literal (ValaCCodeModule* self, ValaInteger
 void vala_ccode_module_visit_real_literal (ValaCCodeModule* self, ValaRealLiteral* expr);
 void vala_ccode_module_visit_string_literal (ValaCCodeModule* self, ValaStringLiteral* expr);
 void vala_ccode_module_visit_null_literal (ValaCCodeModule* self, ValaNullLiteral* expr);
-void vala_ccode_module_visit_parenthesized_expression (ValaCCodeModule* self, ValaParenthesizedExpression* expr);
 void vala_ccode_module_visit_member_access (ValaCCodeModule* self, ValaMemberAccess* expr);
 void vala_ccode_module_visit_method_call (ValaCCodeModule* self, ValaMethodCall* expr);
 void vala_ccode_module_visit_element_access (ValaCCodeModule* self, ValaElementAccess* expr);
@@ -312,7 +309,7 @@ void vala_ccode_module_visit_binary_expression (ValaCCodeModule* self, ValaBinar
 void vala_ccode_module_visit_type_check (ValaCCodeModule* self, ValaTypeCheck* expr);
 void vala_ccode_module_visit_lambda_expression (ValaCCodeModule* self, ValaLambdaExpression* l);
 void vala_ccode_module_visit_assignment (ValaCCodeModule* self, ValaAssignment* a);
-void vala_ccode_module_generate_cparameters (ValaCCodeModule* self, ValaMethod* m, ValaDataType* creturn_type, gboolean in_gtypeinstance_creation_method, GeeMap* cparam_map, ValaCCodeFunction* func, ValaCCodeFunctionDeclarator* vdeclarator, GeeMap* carg_map, ValaCCodeFunctionCall* vcall, gint direction);
+void vala_ccode_module_generate_cparameters (ValaCCodeModule* self, ValaMethod* m, GeeMap* cparam_map, ValaCCodeFunction* func, ValaCCodeFunctionDeclarator* vdeclarator, GeeMap* carg_map, ValaCCodeFunctionCall* vcall, gint direction);
 char* vala_ccode_module_get_custom_creturn_type (ValaCCodeModule* self, ValaMethod* m);
 void vala_ccode_module_generate_dynamic_method_wrapper (ValaCCodeModule* self, ValaDynamicMethod* method);
 gboolean vala_ccode_module_method_has_wrapper (ValaCCodeModule* self, ValaMethod* method);
