@@ -36,7 +36,6 @@
 #include <ccode/valaccodeblock.h>
 #include <ccode/valaccodefunctioncall.h>
 #include <ccode/valaccodeidentifier.h>
-#include <ccode/valaccodeexpression.h>
 #include <ccode/valaccodeconstant.h>
 #include <ccode/valaccodereturnstatement.h>
 #include <ccode/valaccodeenumvalue.h>
@@ -45,6 +44,7 @@
 #include <vala/valaexpression.h>
 #include <vala/valastatement.h>
 #include <gee/list.h>
+#include <vala/valavoidtype.h>
 #include <ccode/valaccodememberaccess.h>
 #include <ccode/valaccodeunaryexpression.h>
 #include <gee/iterable.h>
@@ -52,9 +52,7 @@
 #include <ccode/valaccodegotostatement.h>
 #include <ccode/valaccodebinaryexpression.h>
 #include <ccode/valaccodeifstatement.h>
-#include <ccode/valaccodestatement.h>
 #include <gee/collection.h>
-#include <vala/valavoidtype.h>
 #include <ccode/valaccodelabel.h>
 #include <vala/valamethod.h>
 #include <ccode/valaccodestruct.h>
@@ -78,6 +76,7 @@ enum  {
 static void vala_gerror_module_real_visit_error_domain (ValaCCodeModule* base, ValaErrorDomain* edomain);
 static void vala_gerror_module_real_visit_error_code (ValaCCodeModule* base, ValaErrorCode* ecode);
 static void vala_gerror_module_real_visit_throw_statement (ValaCCodeModule* base, ValaThrowStatement* stmt);
+static ValaCCodeStatement* vala_gerror_module_real_return_with_exception (ValaGErrorModule* self, ValaCCodeExpression* error_expr);
 static void vala_gerror_module_real_add_simple_check (ValaCCodeModule* base, ValaCodeNode* node, ValaCCodeFragment* cfrag);
 static void vala_gerror_module_real_visit_try_statement (ValaCCodeModule* base, ValaTryStatement* stmt);
 static void vala_gerror_module_real_visit_catch_clause (ValaCCodeModule* base, ValaCatchClause* clause);
@@ -229,8 +228,59 @@ static void vala_gerror_module_real_visit_throw_statement (ValaCCodeModule* base
 }
 
 
+static ValaCCodeStatement* vala_gerror_module_real_return_with_exception (ValaGErrorModule* self, ValaCCodeExpression* error_expr) {
+	ValaCCodeIdentifier* _tmp0;
+	ValaCCodeFunctionCall* _tmp1;
+	ValaCCodeFunctionCall* cpropagate;
+	ValaCCodeExpression* _tmp2;
+	ValaCCodeBlock* cerror_block;
+	ValaCCodeExpressionStatement* _tmp3;
+	ValaCCodeFragment* free_frag;
+	ValaCCodeStatement* _tmp7;
+	g_return_val_if_fail (self != NULL, NULL);
+	g_return_val_if_fail (error_expr != NULL, NULL);
+	_tmp0 = NULL;
+	_tmp1 = NULL;
+	cpropagate = (_tmp1 = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp0 = vala_ccode_identifier_new ("g_propagate_error"))), (_tmp0 == NULL) ? NULL : (_tmp0 = (vala_ccode_node_unref (_tmp0), NULL)), _tmp1);
+	_tmp2 = NULL;
+	vala_ccode_function_call_add_argument (cpropagate, _tmp2 = vala_ccode_base_module_get_variable_cexpression ((ValaCCodeBaseModule*) self, "error"));
+	(_tmp2 == NULL) ? NULL : (_tmp2 = (vala_ccode_node_unref (_tmp2), NULL));
+	vala_ccode_function_call_add_argument (cpropagate, error_expr);
+	cerror_block = vala_ccode_block_new ();
+	_tmp3 = NULL;
+	vala_ccode_block_add_statement (cerror_block, (ValaCCodeNode*) (_tmp3 = vala_ccode_expression_statement_new ((ValaCCodeExpression*) cpropagate)));
+	(_tmp3 == NULL) ? NULL : (_tmp3 = (vala_ccode_node_unref (_tmp3), NULL));
+	/* free local variables*/
+	free_frag = vala_ccode_fragment_new ();
+	vala_ccode_base_module_append_local_free ((ValaCCodeBaseModule*) self, ((ValaCCodeBaseModule*) self)->current_symbol, free_frag, FALSE);
+	vala_ccode_block_add_statement (cerror_block, (ValaCCodeNode*) free_frag);
+	if (VALA_IS_VOID_TYPE (((ValaCCodeBaseModule*) self)->current_return_type)) {
+		ValaCCodeReturnStatement* _tmp4;
+		_tmp4 = NULL;
+		vala_ccode_block_add_statement (cerror_block, (ValaCCodeNode*) (_tmp4 = vala_ccode_return_statement_new (NULL)));
+		(_tmp4 == NULL) ? NULL : (_tmp4 = (vala_ccode_node_unref (_tmp4), NULL));
+	} else {
+		ValaCCodeReturnStatement* _tmp6;
+		ValaCCodeExpression* _tmp5;
+		_tmp6 = NULL;
+		_tmp5 = NULL;
+		vala_ccode_block_add_statement (cerror_block, (ValaCCodeNode*) (_tmp6 = vala_ccode_return_statement_new (_tmp5 = vala_ccode_base_module_default_value_for_type ((ValaCCodeBaseModule*) self, ((ValaCCodeBaseModule*) self)->current_return_type, FALSE))));
+		(_tmp6 == NULL) ? NULL : (_tmp6 = (vala_ccode_node_unref (_tmp6), NULL));
+		(_tmp5 == NULL) ? NULL : (_tmp5 = (vala_ccode_node_unref (_tmp5), NULL));
+	}
+	_tmp7 = NULL;
+	return (_tmp7 = (ValaCCodeStatement*) cerror_block, (cpropagate == NULL) ? NULL : (cpropagate = (vala_ccode_node_unref (cpropagate), NULL)), (free_frag == NULL) ? NULL : (free_frag = (vala_ccode_node_unref (free_frag), NULL)), _tmp7);
+}
+
+
+ValaCCodeStatement* vala_gerror_module_return_with_exception (ValaGErrorModule* self, ValaCCodeExpression* error_expr) {
+	return VALA_GERROR_MODULE_GET_CLASS (self)->return_with_exception (self, error_expr);
+}
+
+
 static void vala_gerror_module_real_add_simple_check (ValaCCodeModule* base, ValaCodeNode* node, ValaCCodeFragment* cfrag) {
 	ValaGErrorModule * self;
+	ValaCCodeExpression* inner_error;
 	ValaCCodeFragment* cprint_frag;
 	ValaCCodeIdentifier* _tmp0;
 	ValaCCodeFunctionCall* _tmp1;
@@ -238,19 +288,23 @@ static void vala_gerror_module_real_add_simple_check (ValaCCodeModule* base, Val
 	ValaCCodeConstant* _tmp2;
 	ValaCCodeConstant* _tmp3;
 	ValaCCodeConstant* _tmp4;
-	ValaCCodeMemberAccess* _tmp6;
-	ValaCCodeExpression* _tmp5;
-	ValaCCodeExpressionStatement* _tmp7;
-	ValaCCodeIdentifier* _tmp8;
-	ValaCCodeFunctionCall* _tmp9;
+	ValaCCodeMemberAccess* _tmp5;
+	ValaCCodeExpressionStatement* _tmp6;
+	ValaCCodeIdentifier* _tmp7;
+	ValaCCodeFunctionCall* _tmp8;
 	ValaCCodeFunctionCall* cclear;
-	ValaCCodeUnaryExpression* _tmp11;
-	ValaCCodeExpression* _tmp10;
-	ValaCCodeExpressionStatement* _tmp12;
+	ValaCCodeUnaryExpression* _tmp9;
+	ValaCCodeExpressionStatement* _tmp10;
+	ValaCCodeStatement* cerror_handler;
+	ValaCCodeConstant* _tmp30;
+	ValaCCodeBinaryExpression* _tmp31;
+	ValaCCodeBinaryExpression* ccond;
+	ValaCCodeIfStatement* _tmp32;
 	self = (ValaGErrorModule*) base;
 	g_return_if_fail (node != NULL);
 	g_return_if_fail (cfrag != NULL);
 	((ValaCCodeBaseModule*) self)->current_method_inner_error = TRUE;
+	inner_error = vala_ccode_base_module_get_variable_cexpression ((ValaCCodeBaseModule*) self, "inner_error");
 	cprint_frag = vala_ccode_fragment_new ();
 	_tmp0 = NULL;
 	_tmp1 = NULL;
@@ -264,35 +318,29 @@ static void vala_gerror_module_real_add_simple_check (ValaCCodeModule* base, Val
 	_tmp4 = NULL;
 	vala_ccode_function_call_add_argument (ccritical, (ValaCCodeExpression*) (_tmp4 = vala_ccode_constant_new ("__LINE__")));
 	(_tmp4 == NULL) ? NULL : (_tmp4 = (vala_ccode_node_unref (_tmp4), NULL));
-	_tmp6 = NULL;
 	_tmp5 = NULL;
-	vala_ccode_function_call_add_argument (ccritical, (ValaCCodeExpression*) (_tmp6 = vala_ccode_member_access_new_pointer (_tmp5 = vala_ccode_base_module_get_variable_cexpression ((ValaCCodeBaseModule*) self, "inner_error"), "message")));
-	(_tmp6 == NULL) ? NULL : (_tmp6 = (vala_ccode_node_unref (_tmp6), NULL));
+	vala_ccode_function_call_add_argument (ccritical, (ValaCCodeExpression*) (_tmp5 = vala_ccode_member_access_new_pointer (inner_error, "message")));
 	(_tmp5 == NULL) ? NULL : (_tmp5 = (vala_ccode_node_unref (_tmp5), NULL));
+	_tmp6 = NULL;
+	vala_ccode_fragment_append (cprint_frag, (ValaCCodeNode*) (_tmp6 = vala_ccode_expression_statement_new ((ValaCCodeExpression*) ccritical)));
+	(_tmp6 == NULL) ? NULL : (_tmp6 = (vala_ccode_node_unref (_tmp6), NULL));
 	_tmp7 = NULL;
-	vala_ccode_fragment_append (cprint_frag, (ValaCCodeNode*) (_tmp7 = vala_ccode_expression_statement_new ((ValaCCodeExpression*) ccritical)));
-	(_tmp7 == NULL) ? NULL : (_tmp7 = (vala_ccode_node_unref (_tmp7), NULL));
 	_tmp8 = NULL;
+	cclear = (_tmp8 = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp7 = vala_ccode_identifier_new ("g_clear_error"))), (_tmp7 == NULL) ? NULL : (_tmp7 = (vala_ccode_node_unref (_tmp7), NULL)), _tmp8);
 	_tmp9 = NULL;
-	cclear = (_tmp9 = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp8 = vala_ccode_identifier_new ("g_clear_error"))), (_tmp8 == NULL) ? NULL : (_tmp8 = (vala_ccode_node_unref (_tmp8), NULL)), _tmp9);
-	_tmp11 = NULL;
+	vala_ccode_function_call_add_argument (cclear, (ValaCCodeExpression*) (_tmp9 = vala_ccode_unary_expression_new (VALA_CCODE_UNARY_OPERATOR_ADDRESS_OF, inner_error)));
+	(_tmp9 == NULL) ? NULL : (_tmp9 = (vala_ccode_node_unref (_tmp9), NULL));
 	_tmp10 = NULL;
-	vala_ccode_function_call_add_argument (cclear, (ValaCCodeExpression*) (_tmp11 = vala_ccode_unary_expression_new (VALA_CCODE_UNARY_OPERATOR_ADDRESS_OF, _tmp10 = vala_ccode_base_module_get_variable_cexpression ((ValaCCodeBaseModule*) self, "inner_error"))));
-	(_tmp11 == NULL) ? NULL : (_tmp11 = (vala_ccode_node_unref (_tmp11), NULL));
+	vala_ccode_fragment_append (cprint_frag, (ValaCCodeNode*) (_tmp10 = vala_ccode_expression_statement_new ((ValaCCodeExpression*) cclear)));
 	(_tmp10 == NULL) ? NULL : (_tmp10 = (vala_ccode_node_unref (_tmp10), NULL));
-	_tmp12 = NULL;
-	vala_ccode_fragment_append (cprint_frag, (ValaCCodeNode*) (_tmp12 = vala_ccode_expression_statement_new ((ValaCCodeExpression*) cclear)));
-	(_tmp12 == NULL) ? NULL : (_tmp12 = (vala_ccode_node_unref (_tmp12), NULL));
+	cerror_handler = NULL;
 	if (((ValaCCodeBaseModule*) self)->current_try != NULL) {
 		ValaCCodeBlock* cerror_block;
 		ValaCCodeFragment* free_frag;
-		ValaCCodeGotoStatement* _tmp22;
-		char* _tmp21;
-		ValaCCodeConstant* _tmp24;
-		ValaCCodeExpression* _tmp23;
-		ValaCCodeBinaryExpression* _tmp25;
-		ValaCCodeBinaryExpression* ccond;
-		ValaCCodeIfStatement* _tmp26;
+		ValaCCodeGotoStatement* _tmp19;
+		char* _tmp18;
+		ValaCCodeStatement* _tmp21;
+		ValaCCodeStatement* _tmp20;
 		/* surrounding try found*/
 		cerror_block = vala_ccode_block_new ();
 		/* free local variables*/
@@ -300,12 +348,12 @@ static void vala_gerror_module_real_add_simple_check (ValaCCodeModule* base, Val
 		vala_ccode_base_module_append_error_free ((ValaCCodeBaseModule*) self, ((ValaCCodeBaseModule*) self)->current_symbol, free_frag, ((ValaCCodeBaseModule*) self)->current_try);
 		vala_ccode_block_add_statement (cerror_block, (ValaCCodeNode*) free_frag);
 		{
-			GeeList* _tmp13;
-			GeeIterator* _tmp14;
+			GeeList* _tmp11;
+			GeeIterator* _tmp12;
 			GeeIterator* _clause_it;
-			_tmp13 = NULL;
-			_tmp14 = NULL;
-			_clause_it = (_tmp14 = gee_iterable_iterator ((GeeIterable*) (_tmp13 = vala_try_statement_get_catch_clauses (((ValaCCodeBaseModule*) self)->current_try))), (_tmp13 == NULL) ? NULL : (_tmp13 = (gee_collection_object_unref (_tmp13), NULL)), _tmp14);
+			_tmp11 = NULL;
+			_tmp12 = NULL;
+			_clause_it = (_tmp12 = gee_iterable_iterator ((GeeIterable*) (_tmp11 = vala_try_statement_get_catch_clauses (((ValaCCodeBaseModule*) self)->current_try))), (_tmp11 == NULL) ? NULL : (_tmp11 = (gee_collection_object_unref (_tmp11), NULL)), _tmp12);
 			while (gee_iterator_next (_clause_it)) {
 				ValaCatchClause* clause;
 				ValaCCodeGotoStatement* cgoto_stmt;
@@ -316,25 +364,23 @@ static void vala_gerror_module_real_add_simple_check (ValaCCodeModule* base, Val
 					/* general catch clause*/
 					vala_ccode_block_add_statement (cerror_block, (ValaCCodeNode*) cgoto_stmt);
 				} else {
-					ValaCCodeIdentifier* _tmp18;
-					char* _tmp17;
-					ValaCCodeMemberAccess* _tmp16;
-					ValaCCodeExpression* _tmp15;
-					ValaCCodeBinaryExpression* _tmp19;
+					ValaCCodeIdentifier* _tmp15;
+					char* _tmp14;
+					ValaCCodeMemberAccess* _tmp13;
+					ValaCCodeBinaryExpression* _tmp16;
 					ValaCCodeBinaryExpression* ccond;
 					ValaCCodeBlock* cgoto_block;
-					ValaCCodeIfStatement* _tmp20;
-					_tmp18 = NULL;
-					_tmp17 = NULL;
-					_tmp16 = NULL;
+					ValaCCodeIfStatement* _tmp17;
 					_tmp15 = NULL;
-					_tmp19 = NULL;
-					ccond = (_tmp19 = vala_ccode_binary_expression_new (VALA_CCODE_BINARY_OPERATOR_EQUALITY, (ValaCCodeExpression*) (_tmp16 = vala_ccode_member_access_new_pointer (_tmp15 = vala_ccode_base_module_get_variable_cexpression ((ValaCCodeBaseModule*) self, "inner_error"), "domain")), (ValaCCodeExpression*) (_tmp18 = vala_ccode_identifier_new (_tmp17 = vala_typesymbol_get_upper_case_cname (vala_data_type_get_data_type (vala_catch_clause_get_error_type (clause)), NULL)))), (_tmp18 == NULL) ? NULL : (_tmp18 = (vala_ccode_node_unref (_tmp18), NULL)), _tmp17 = (g_free (_tmp17), NULL), (_tmp16 == NULL) ? NULL : (_tmp16 = (vala_ccode_node_unref (_tmp16), NULL)), (_tmp15 == NULL) ? NULL : (_tmp15 = (vala_ccode_node_unref (_tmp15), NULL)), _tmp19);
+					_tmp14 = NULL;
+					_tmp13 = NULL;
+					_tmp16 = NULL;
+					ccond = (_tmp16 = vala_ccode_binary_expression_new (VALA_CCODE_BINARY_OPERATOR_EQUALITY, (ValaCCodeExpression*) (_tmp13 = vala_ccode_member_access_new_pointer (inner_error, "domain")), (ValaCCodeExpression*) (_tmp15 = vala_ccode_identifier_new (_tmp14 = vala_typesymbol_get_upper_case_cname (vala_data_type_get_data_type (vala_catch_clause_get_error_type (clause)), NULL)))), (_tmp15 == NULL) ? NULL : (_tmp15 = (vala_ccode_node_unref (_tmp15), NULL)), _tmp14 = (g_free (_tmp14), NULL), (_tmp13 == NULL) ? NULL : (_tmp13 = (vala_ccode_node_unref (_tmp13), NULL)), _tmp16);
 					cgoto_block = vala_ccode_block_new ();
 					vala_ccode_block_add_statement (cgoto_block, (ValaCCodeNode*) cgoto_stmt);
-					_tmp20 = NULL;
-					vala_ccode_block_add_statement (cerror_block, (ValaCCodeNode*) (_tmp20 = vala_ccode_if_statement_new ((ValaCCodeExpression*) ccond, (ValaCCodeStatement*) cgoto_block, NULL)));
-					(_tmp20 == NULL) ? NULL : (_tmp20 = (vala_ccode_node_unref (_tmp20), NULL));
+					_tmp17 = NULL;
+					vala_ccode_block_add_statement (cerror_block, (ValaCCodeNode*) (_tmp17 = vala_ccode_if_statement_new ((ValaCCodeExpression*) ccond, (ValaCCodeStatement*) cgoto_block, NULL)));
+					(_tmp17 == NULL) ? NULL : (_tmp17 = (vala_ccode_node_unref (_tmp17), NULL));
 					(ccond == NULL) ? NULL : (ccond = (vala_ccode_node_unref (ccond), NULL));
 					(cgoto_block == NULL) ? NULL : (cgoto_block = (vala_ccode_node_unref (cgoto_block), NULL));
 				}
@@ -344,99 +390,38 @@ static void vala_gerror_module_real_add_simple_check (ValaCCodeModule* base, Val
 			(_clause_it == NULL) ? NULL : (_clause_it = (gee_collection_object_unref (_clause_it), NULL));
 		}
 		/* go to finally clause if no catch clause matches*/
-		_tmp22 = NULL;
+		_tmp19 = NULL;
+		_tmp18 = NULL;
+		vala_ccode_block_add_statement (cerror_block, (ValaCCodeNode*) (_tmp19 = vala_ccode_goto_statement_new (_tmp18 = g_strdup_printf ("__finally%d", self->priv->current_try_id))));
+		(_tmp19 == NULL) ? NULL : (_tmp19 = (vala_ccode_node_unref (_tmp19), NULL));
+		_tmp18 = (g_free (_tmp18), NULL);
 		_tmp21 = NULL;
-		vala_ccode_block_add_statement (cerror_block, (ValaCCodeNode*) (_tmp22 = vala_ccode_goto_statement_new (_tmp21 = g_strdup_printf ("__finally%d", self->priv->current_try_id))));
-		(_tmp22 == NULL) ? NULL : (_tmp22 = (vala_ccode_node_unref (_tmp22), NULL));
-		_tmp21 = (g_free (_tmp21), NULL);
-		/* check error domain if expression failed*/
-		_tmp24 = NULL;
-		_tmp23 = NULL;
-		_tmp25 = NULL;
-		ccond = (_tmp25 = vala_ccode_binary_expression_new (VALA_CCODE_BINARY_OPERATOR_INEQUALITY, _tmp23 = vala_ccode_base_module_get_variable_cexpression ((ValaCCodeBaseModule*) self, "inner_error"), (ValaCCodeExpression*) (_tmp24 = vala_ccode_constant_new ("NULL"))), (_tmp24 == NULL) ? NULL : (_tmp24 = (vala_ccode_node_unref (_tmp24), NULL)), (_tmp23 == NULL) ? NULL : (_tmp23 = (vala_ccode_node_unref (_tmp23), NULL)), _tmp25);
-		_tmp26 = NULL;
-		vala_ccode_fragment_append (cfrag, (ValaCCodeNode*) (_tmp26 = vala_ccode_if_statement_new ((ValaCCodeExpression*) ccond, (ValaCCodeStatement*) cerror_block, NULL)));
-		(_tmp26 == NULL) ? NULL : (_tmp26 = (vala_ccode_node_unref (_tmp26), NULL));
+		_tmp20 = NULL;
+		cerror_handler = (_tmp21 = (_tmp20 = (ValaCCodeStatement*) cerror_block, (_tmp20 == NULL) ? NULL : vala_ccode_node_ref (_tmp20)), (cerror_handler == NULL) ? NULL : (cerror_handler = (vala_ccode_node_unref (cerror_handler), NULL)), _tmp21);
 		(cerror_block == NULL) ? NULL : (cerror_block = (vala_ccode_node_unref (cerror_block), NULL));
 		(free_frag == NULL) ? NULL : (free_frag = (vala_ccode_node_unref (free_frag), NULL));
-		(ccond == NULL) ? NULL : (ccond = (vala_ccode_node_unref (ccond), NULL));
 	} else {
-		gboolean _tmp27;
-		_tmp27 = FALSE;
+		gboolean _tmp22;
+		_tmp22 = FALSE;
 		if (((ValaCCodeBaseModule*) self)->current_method != NULL) {
-			GeeList* _tmp28;
-			_tmp28 = NULL;
-			_tmp27 = gee_collection_get_size ((GeeCollection*) (_tmp28 = vala_code_node_get_error_types ((ValaCodeNode*) ((ValaCCodeBaseModule*) self)->current_method))) > 0;
-			(_tmp28 == NULL) ? NULL : (_tmp28 = (gee_collection_object_unref (_tmp28), NULL));
+			GeeList* _tmp23;
+			_tmp23 = NULL;
+			_tmp22 = gee_collection_get_size ((GeeCollection*) (_tmp23 = vala_code_node_get_error_types ((ValaCodeNode*) ((ValaCCodeBaseModule*) self)->current_method))) > 0;
+			(_tmp23 == NULL) ? NULL : (_tmp23 = (gee_collection_object_unref (_tmp23), NULL));
 		} else {
-			_tmp27 = FALSE;
+			_tmp22 = FALSE;
 		}
-		if (_tmp27) {
-			ValaCCodeIdentifier* _tmp29;
-			ValaCCodeFunctionCall* _tmp30;
-			ValaCCodeFunctionCall* cpropagate;
-			ValaCCodeExpression* _tmp31;
-			ValaCCodeExpression* _tmp32;
-			ValaCCodeBlock* cerror_block;
-			ValaCCodeExpressionStatement* _tmp33;
-			ValaCCodeFragment* free_frag;
-			ValaCCodeConstant* _tmp38;
-			ValaCCodeExpression* _tmp37;
-			ValaCCodeBinaryExpression* _tmp39;
-			ValaCCodeBinaryExpression* ccond;
-			ValaCCodeIfStatement* _tmp40;
+		if (_tmp22) {
+			ValaCCodeStatement* _tmp24;
 			/* current method can fail, propagate error
 			 TODO ensure one of the error domains matches*/
-			_tmp29 = NULL;
-			_tmp30 = NULL;
-			cpropagate = (_tmp30 = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp29 = vala_ccode_identifier_new ("g_propagate_error"))), (_tmp29 == NULL) ? NULL : (_tmp29 = (vala_ccode_node_unref (_tmp29), NULL)), _tmp30);
-			_tmp31 = NULL;
-			vala_ccode_function_call_add_argument (cpropagate, _tmp31 = vala_ccode_base_module_get_variable_cexpression ((ValaCCodeBaseModule*) self, "error"));
-			(_tmp31 == NULL) ? NULL : (_tmp31 = (vala_ccode_node_unref (_tmp31), NULL));
-			_tmp32 = NULL;
-			vala_ccode_function_call_add_argument (cpropagate, _tmp32 = vala_ccode_base_module_get_variable_cexpression ((ValaCCodeBaseModule*) self, "inner_error"));
-			(_tmp32 == NULL) ? NULL : (_tmp32 = (vala_ccode_node_unref (_tmp32), NULL));
-			cerror_block = vala_ccode_block_new ();
-			_tmp33 = NULL;
-			vala_ccode_block_add_statement (cerror_block, (ValaCCodeNode*) (_tmp33 = vala_ccode_expression_statement_new ((ValaCCodeExpression*) cpropagate)));
-			(_tmp33 == NULL) ? NULL : (_tmp33 = (vala_ccode_node_unref (_tmp33), NULL));
-			/* free local variables*/
-			free_frag = vala_ccode_fragment_new ();
-			vala_ccode_base_module_append_local_free ((ValaCCodeBaseModule*) self, ((ValaCCodeBaseModule*) self)->current_symbol, free_frag, FALSE);
-			vala_ccode_block_add_statement (cerror_block, (ValaCCodeNode*) free_frag);
-			if (VALA_IS_VOID_TYPE (((ValaCCodeBaseModule*) self)->current_return_type)) {
-				ValaCCodeReturnStatement* _tmp34;
-				_tmp34 = NULL;
-				vala_ccode_block_add_statement (cerror_block, (ValaCCodeNode*) (_tmp34 = vala_ccode_return_statement_new (NULL)));
-				(_tmp34 == NULL) ? NULL : (_tmp34 = (vala_ccode_node_unref (_tmp34), NULL));
-			} else {
-				ValaCCodeReturnStatement* _tmp36;
-				ValaCCodeExpression* _tmp35;
-				_tmp36 = NULL;
-				_tmp35 = NULL;
-				vala_ccode_block_add_statement (cerror_block, (ValaCCodeNode*) (_tmp36 = vala_ccode_return_statement_new (_tmp35 = vala_ccode_base_module_default_value_for_type ((ValaCCodeBaseModule*) self, ((ValaCCodeBaseModule*) self)->current_return_type, FALSE))));
-				(_tmp36 == NULL) ? NULL : (_tmp36 = (vala_ccode_node_unref (_tmp36), NULL));
-				(_tmp35 == NULL) ? NULL : (_tmp35 = (vala_ccode_node_unref (_tmp35), NULL));
-			}
-			_tmp38 = NULL;
-			_tmp37 = NULL;
-			_tmp39 = NULL;
-			ccond = (_tmp39 = vala_ccode_binary_expression_new (VALA_CCODE_BINARY_OPERATOR_INEQUALITY, _tmp37 = vala_ccode_base_module_get_variable_cexpression ((ValaCCodeBaseModule*) self, "inner_error"), (ValaCCodeExpression*) (_tmp38 = vala_ccode_constant_new ("NULL"))), (_tmp38 == NULL) ? NULL : (_tmp38 = (vala_ccode_node_unref (_tmp38), NULL)), (_tmp37 == NULL) ? NULL : (_tmp37 = (vala_ccode_node_unref (_tmp37), NULL)), _tmp39);
-			_tmp40 = NULL;
-			vala_ccode_fragment_append (cfrag, (ValaCCodeNode*) (_tmp40 = vala_ccode_if_statement_new ((ValaCCodeExpression*) ccond, (ValaCCodeStatement*) cerror_block, NULL)));
-			(_tmp40 == NULL) ? NULL : (_tmp40 = (vala_ccode_node_unref (_tmp40), NULL));
-			(cpropagate == NULL) ? NULL : (cpropagate = (vala_ccode_node_unref (cpropagate), NULL));
-			(cerror_block == NULL) ? NULL : (cerror_block = (vala_ccode_node_unref (cerror_block), NULL));
-			(free_frag == NULL) ? NULL : (free_frag = (vala_ccode_node_unref (free_frag), NULL));
-			(ccond == NULL) ? NULL : (ccond = (vala_ccode_node_unref (ccond), NULL));
+			_tmp24 = NULL;
+			cerror_handler = (_tmp24 = vala_gerror_module_return_with_exception (self, inner_error), (cerror_handler == NULL) ? NULL : (cerror_handler = (vala_ccode_node_unref (cerror_handler), NULL)), _tmp24);
 		} else {
 			ValaCCodeBlock* cerror_block;
 			ValaCCodeFragment* free_frag;
-			ValaCCodeConstant* _tmp45;
-			ValaCCodeExpression* _tmp44;
-			ValaCCodeBinaryExpression* _tmp46;
-			ValaCCodeBinaryExpression* ccond;
-			ValaCCodeIfStatement* _tmp47;
+			ValaCCodeStatement* _tmp29;
+			ValaCCodeStatement* _tmp28;
 			/* unhandled error*/
 			cerror_block = vala_ccode_block_new ();
 			/* free local variables*/
@@ -446,37 +431,40 @@ static void vala_gerror_module_real_add_simple_check (ValaCCodeModule* base, Val
 			/* print critical message*/
 			vala_ccode_block_add_statement (cerror_block, (ValaCCodeNode*) cprint_frag);
 			if (VALA_IS_VOID_TYPE (((ValaCCodeBaseModule*) self)->current_return_type)) {
-				ValaCCodeReturnStatement* _tmp41;
-				_tmp41 = NULL;
-				vala_ccode_block_add_statement (cerror_block, (ValaCCodeNode*) (_tmp41 = vala_ccode_return_statement_new (NULL)));
-				(_tmp41 == NULL) ? NULL : (_tmp41 = (vala_ccode_node_unref (_tmp41), NULL));
+				ValaCCodeReturnStatement* _tmp25;
+				_tmp25 = NULL;
+				vala_ccode_block_add_statement (cerror_block, (ValaCCodeNode*) (_tmp25 = vala_ccode_return_statement_new (NULL)));
+				(_tmp25 == NULL) ? NULL : (_tmp25 = (vala_ccode_node_unref (_tmp25), NULL));
 			} else {
 				if (((ValaCCodeBaseModule*) self)->current_return_type != NULL) {
-					ValaCCodeReturnStatement* _tmp43;
-					ValaCCodeExpression* _tmp42;
-					_tmp43 = NULL;
-					_tmp42 = NULL;
-					vala_ccode_block_add_statement (cerror_block, (ValaCCodeNode*) (_tmp43 = vala_ccode_return_statement_new (_tmp42 = vala_ccode_base_module_default_value_for_type ((ValaCCodeBaseModule*) self, ((ValaCCodeBaseModule*) self)->current_return_type, FALSE))));
-					(_tmp43 == NULL) ? NULL : (_tmp43 = (vala_ccode_node_unref (_tmp43), NULL));
-					(_tmp42 == NULL) ? NULL : (_tmp42 = (vala_ccode_node_unref (_tmp42), NULL));
+					ValaCCodeReturnStatement* _tmp27;
+					ValaCCodeExpression* _tmp26;
+					_tmp27 = NULL;
+					_tmp26 = NULL;
+					vala_ccode_block_add_statement (cerror_block, (ValaCCodeNode*) (_tmp27 = vala_ccode_return_statement_new (_tmp26 = vala_ccode_base_module_default_value_for_type ((ValaCCodeBaseModule*) self, ((ValaCCodeBaseModule*) self)->current_return_type, FALSE))));
+					(_tmp27 == NULL) ? NULL : (_tmp27 = (vala_ccode_node_unref (_tmp27), NULL));
+					(_tmp26 == NULL) ? NULL : (_tmp26 = (vala_ccode_node_unref (_tmp26), NULL));
 				}
 			}
-			/* check error domain if expression failed*/
-			_tmp45 = NULL;
-			_tmp44 = NULL;
-			_tmp46 = NULL;
-			ccond = (_tmp46 = vala_ccode_binary_expression_new (VALA_CCODE_BINARY_OPERATOR_INEQUALITY, _tmp44 = vala_ccode_base_module_get_variable_cexpression ((ValaCCodeBaseModule*) self, "inner_error"), (ValaCCodeExpression*) (_tmp45 = vala_ccode_constant_new ("NULL"))), (_tmp45 == NULL) ? NULL : (_tmp45 = (vala_ccode_node_unref (_tmp45), NULL)), (_tmp44 == NULL) ? NULL : (_tmp44 = (vala_ccode_node_unref (_tmp44), NULL)), _tmp46);
-			_tmp47 = NULL;
-			vala_ccode_fragment_append (cfrag, (ValaCCodeNode*) (_tmp47 = vala_ccode_if_statement_new ((ValaCCodeExpression*) ccond, (ValaCCodeStatement*) cerror_block, NULL)));
-			(_tmp47 == NULL) ? NULL : (_tmp47 = (vala_ccode_node_unref (_tmp47), NULL));
+			_tmp29 = NULL;
+			_tmp28 = NULL;
+			cerror_handler = (_tmp29 = (_tmp28 = (ValaCCodeStatement*) cerror_block, (_tmp28 == NULL) ? NULL : vala_ccode_node_ref (_tmp28)), (cerror_handler == NULL) ? NULL : (cerror_handler = (vala_ccode_node_unref (cerror_handler), NULL)), _tmp29);
 			(cerror_block == NULL) ? NULL : (cerror_block = (vala_ccode_node_unref (cerror_block), NULL));
 			(free_frag == NULL) ? NULL : (free_frag = (vala_ccode_node_unref (free_frag), NULL));
-			(ccond == NULL) ? NULL : (ccond = (vala_ccode_node_unref (ccond), NULL));
 		}
 	}
+	_tmp30 = NULL;
+	_tmp31 = NULL;
+	ccond = (_tmp31 = vala_ccode_binary_expression_new (VALA_CCODE_BINARY_OPERATOR_INEQUALITY, inner_error, (ValaCCodeExpression*) (_tmp30 = vala_ccode_constant_new ("NULL"))), (_tmp30 == NULL) ? NULL : (_tmp30 = (vala_ccode_node_unref (_tmp30), NULL)), _tmp31);
+	_tmp32 = NULL;
+	vala_ccode_fragment_append (cfrag, (ValaCCodeNode*) (_tmp32 = vala_ccode_if_statement_new ((ValaCCodeExpression*) ccond, cerror_handler, NULL)));
+	(_tmp32 == NULL) ? NULL : (_tmp32 = (vala_ccode_node_unref (_tmp32), NULL));
+	(inner_error == NULL) ? NULL : (inner_error = (vala_ccode_node_unref (inner_error), NULL));
 	(cprint_frag == NULL) ? NULL : (cprint_frag = (vala_ccode_node_unref (cprint_frag), NULL));
 	(ccritical == NULL) ? NULL : (ccritical = (vala_ccode_node_unref (ccritical), NULL));
 	(cclear == NULL) ? NULL : (cclear = (vala_ccode_node_unref (cclear), NULL));
+	(cerror_handler == NULL) ? NULL : (cerror_handler = (vala_ccode_node_unref (cerror_handler), NULL));
+	(ccond == NULL) ? NULL : (ccond = (vala_ccode_node_unref (ccond), NULL));
 }
 
 
@@ -681,6 +669,7 @@ static void vala_gerror_module_class_init (ValaGErrorModuleClass * klass) {
 	VALA_CCODE_MODULE_CLASS (klass)->visit_error_domain = vala_gerror_module_real_visit_error_domain;
 	VALA_CCODE_MODULE_CLASS (klass)->visit_error_code = vala_gerror_module_real_visit_error_code;
 	VALA_CCODE_MODULE_CLASS (klass)->visit_throw_statement = vala_gerror_module_real_visit_throw_statement;
+	VALA_GERROR_MODULE_CLASS (klass)->return_with_exception = vala_gerror_module_real_return_with_exception;
 	VALA_CCODE_MODULE_CLASS (klass)->add_simple_check = vala_gerror_module_real_add_simple_check;
 	VALA_CCODE_MODULE_CLASS (klass)->visit_try_statement = vala_gerror_module_real_visit_try_statement;
 	VALA_CCODE_MODULE_CLASS (klass)->visit_catch_clause = vala_gerror_module_real_visit_catch_clause;
