@@ -574,7 +574,7 @@ static void vala_gir_writer_real_visit_method (ValaCodeVisitor* base, ValaMethod
 		} else {
 			_tmp2 = FALSE;
 		}
-		_tmp0 = (_tmp2);
+		_tmp0 = _tmp2;
 	}
 	/* don't write interface implementation unless it's an abstract or virtual method*/
 	if (_tmp0) {
@@ -664,7 +664,7 @@ static void vala_gir_writer_real_visit_property (ValaCodeVisitor* base, ValaProp
 		} else {
 			_tmp2 = FALSE;
 		}
-		_tmp0 = (_tmp2);
+		_tmp0 = _tmp2;
 	}
 	if (_tmp0) {
 		return;
