@@ -1,6 +1,7 @@
 /* valagobjectmodule.vala
  *
- * Copyright (C) 2006-2008  Jürg Billeter, Raffaele Sandrini
+ * Copyright (C) 2006-2009  Jürg Billeter
+ * Copyright (C) 2006-2008  Raffaele Sandrini
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -26,6 +27,8 @@
 
 #include <glib.h>
 #include <glib-object.h>
+#include <vala/valaformalparameter.h>
+#include <gee/map.h>
 #include <vala/valaclass.h>
 #include <ccode/valaccodefunctioncall.h>
 #include <vala/valaproperty.h>
