@@ -67,7 +67,6 @@
 #include <gobject/valaccodebasemodule.h>
 #include <gobject/valainterfaceregisterfunction.h>
 #include <gobject/valatyperegisterfunction.h>
-#include <gobject/valaccodestructmodule.h>
 #include <gobject/valastructregisterfunction.h>
 
 
@@ -350,7 +349,7 @@ static void vala_gtype_module_add_interface_base_init_function (ValaGTypeModule*
 	_tmp7 = NULL;
 	_tmp8 = NULL;
 	decl = (_tmp8 = vala_ccode_declaration_new (_tmp7 = vala_data_type_get_cname (((ValaCCodeBaseModule*) self)->bool_type)), _tmp7 = (g_free (_tmp7), NULL), _tmp8);
-	vala_ccode_declaration_set_modifiers (decl, vala_ccode_declaration_get_modifiers (decl) | (VALA_CCODE_MODIFIERS_STATIC));
+	vala_ccode_declaration_set_modifiers (decl, vala_ccode_declaration_get_modifiers (decl) | VALA_CCODE_MODIFIERS_STATIC);
 	_tmp10 = NULL;
 	_tmp9 = NULL;
 	vala_ccode_declaration_add_declarator (decl, (ValaCCodeDeclarator*) (_tmp10 = vala_ccode_variable_declarator_new ("initialized", (ValaCCodeExpression*) (_tmp9 = vala_ccode_constant_new ("FALSE")))));
