@@ -172,7 +172,7 @@ void vala_typeregister_function_init_from_type (ValaTypeRegisterFunction* self,
 	}
 	vala_ccode_declaration_set_modifiers (cdecl_, VALA_CCODE_MODIFIERS_STATIC);
 	if (use_thread_safe) {
-		vala_ccode_declaration_set_modifiers (cdecl_, vala_ccode_declaration_get_modifiers (cdecl_) | (VALA_CCODE_MODIFIERS_VOLATILE));
+		vala_ccode_declaration_set_modifiers (cdecl_, vala_ccode_declaration_get_modifiers (cdecl_) | VALA_CCODE_MODIFIERS_VOLATILE);
 	}
 	if (!plugin) {
 		vala_ccode_block_add_statement (type_block, (ValaCCodeNode*) cdecl_);
@@ -305,7 +305,7 @@ void vala_typeregister_function_init_from_type (ValaTypeRegisterFunction* self,
 		ctypedecl = vala_ccode_declaration_new ("const GTypeInfo");
 		vala_ccode_declaration_set_modifiers (ctypedecl, VALA_CCODE_MODIFIERS_STATIC);
 		_tmp43 = NULL;
-		if ((plugin)) {
+		if (plugin) {
 			char* _tmp44;
 			_tmp44 = NULL;
 			_tmp43 = (_tmp44 = vala_typeregister_function_get_base_finalize_func_name (self), _tmp43 = (g_free (_tmp43), NULL), _tmp44);
