@@ -21,9 +21,6 @@
  */
 
 #include <vala/valabaseaccess.h>
-#include <gee/collection.h>
-#include <gee/iterable.h>
-#include <gee/iterator.h>
 #include <vala/valasourcereference.h>
 #include <vala/valacodevisitor.h>
 #include <vala/valasemanticanalyzer.h>
@@ -104,48 +101,28 @@ static gboolean vala_base_access_real_check (ValaCodeNode* base, ValaSemanticAna
 		return FALSE;
 	}
 	if (analyzer->current_class == NULL) {
-		GeeList* _tmp8;
-		GeeIterator* _tmp9;
-		GeeIterator* base_type_it;
-		ValaDataType* _tmp10;
 		if (analyzer->current_struct == NULL) {
 			vala_code_node_set_error ((ValaCodeNode*) self, TRUE);
 			vala_report_error (vala_code_node_get_source_reference ((ValaCodeNode*) self), "Base access invalid outside of class and struct");
 			return FALSE;
 		} else {
-			GeeList* _tmp3;
-			gboolean _tmp4;
-			_tmp3 = NULL;
-			if ((_tmp4 = gee_collection_get_size ((GeeCollection*) (_tmp3 = vala_struct_get_base_types (analyzer->current_struct))) != 1, (_tmp3 == NULL) ? NULL : (_tmp3 = (gee_collection_object_unref (_tmp3), NULL)), _tmp4)) {
-				char* _tmp6;
-				GeeList* _tmp5;
+			if (vala_struct_get_base_type (analyzer->current_struct) == NULL) {
 				vala_code_node_set_error ((ValaCodeNode*) self, TRUE);
-				_tmp6 = NULL;
-				_tmp5 = NULL;
-				vala_report_error (vala_code_node_get_source_reference ((ValaCodeNode*) self), _tmp6 = g_strdup_printf ("Base access invalid without base type %d", gee_collection_get_size ((GeeCollection*) (_tmp5 = vala_struct_get_base_types (analyzer->current_struct)))));
-				_tmp6 = (g_free (_tmp6), NULL);
-				(_tmp5 == NULL) ? NULL : (_tmp5 = (gee_collection_object_unref (_tmp5), NULL));
+				vala_report_error (vala_code_node_get_source_reference ((ValaCodeNode*) self), "Base access invalid without base type");
 				return FALSE;
 			}
 		}
-		_tmp8 = NULL;
-		_tmp9 = NULL;
-		base_type_it = (_tmp9 = gee_iterable_iterator ((GeeIterable*) (_tmp8 = vala_struct_get_base_types (analyzer->current_struct))), (_tmp8 == NULL) ? NULL : (_tmp8 = (gee_collection_object_unref (_tmp8), NULL)), _tmp9);
-		gee_iterator_next (base_type_it);
-		_tmp10 = NULL;
-		vala_expression_set_value_type ((ValaExpression*) self, _tmp10 = (ValaDataType*) gee_iterator_get (base_type_it));
-		(_tmp10 == NULL) ? NULL : (_tmp10 = (vala_code_node_unref (_tmp10), NULL));
-		(base_type_it == NULL) ? NULL : (base_type_it = (gee_collection_object_unref (base_type_it), NULL));
+		vala_expression_set_value_type ((ValaExpression*) self, vala_struct_get_base_type (analyzer->current_struct));
 	} else {
 		if (vala_class_get_base_class (analyzer->current_class) == NULL) {
 			vala_code_node_set_error ((ValaCodeNode*) self, TRUE);
 			vala_report_error (vala_code_node_get_source_reference ((ValaCodeNode*) self), "Base access invalid without base class");
 			return FALSE;
 		} else {
-			ValaObjectType* _tmp12;
-			_tmp12 = NULL;
-			vala_expression_set_value_type ((ValaExpression*) self, (ValaDataType*) (_tmp12 = vala_object_type_new ((ValaObjectTypeSymbol*) vala_class_get_base_class (analyzer->current_class))));
-			(_tmp12 == NULL) ? NULL : (_tmp12 = (vala_code_node_unref (_tmp12), NULL));
+			ValaObjectType* _tmp5;
+			_tmp5 = NULL;
+			vala_expression_set_value_type ((ValaExpression*) self, (ValaDataType*) (_tmp5 = vala_object_type_new ((ValaObjectTypeSymbol*) vala_class_get_base_class (analyzer->current_class))));
+			(_tmp5 == NULL) ? NULL : (_tmp5 = (vala_code_node_unref (_tmp5), NULL));
 		}
 	}
 	vala_expression_set_symbol_reference ((ValaExpression*) self, (ValaSymbol*) vala_data_type_get_data_type (vala_expression_get_value_type ((ValaExpression*) self)));
