@@ -22,6 +22,7 @@
 
 #include <vala/valabasicblock.h>
 #include <gee/arraylist.h>
+#include <gee/hashset.h>
 #include <gee/collection.h>
 #include <gee/readonlylist.h>
 #include <gobject/gvaluecollector.h>
@@ -33,12 +34,17 @@ struct _ValaBasicBlockPrivate {
 	GeeList* nodes;
 	GeeList* predecessors;
 	GeeList* successors;
+	ValaBasicBlock* _parent;
+	GeeList* children;
+	GeeSet* df;
+	GeeSet* phi_functions;
 };
 
 #define VALA_BASIC_BLOCK_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VALA_TYPE_BASIC_BLOCK, ValaBasicBlockPrivate))
 enum  {
 	VALA_BASIC_BLOCK_DUMMY_PROPERTY
 };
+static void vala_basic_block_set_parent (ValaBasicBlock* self, ValaBasicBlock* value);
 static gpointer vala_basic_block_parent_class = NULL;
 static void vala_basic_block_finalize (ValaBasicBlock* obj);
 
@@ -46,7 +52,7 @@ static void vala_basic_block_finalize (ValaBasicBlock* obj);
 
 ValaBasicBlock* vala_basic_block_construct (GType object_type) {
 	ValaBasicBlock* self;
-	self = ((ValaBasicBlock*) (g_type_create_instance (object_type)));
+	self = (ValaBasicBlock*) g_type_create_instance (object_type);
 	return self;
 }
 
@@ -58,7 +64,7 @@ ValaBasicBlock* vala_basic_block_new (void) {
 
 ValaBasicBlock* vala_basic_block_construct_entry (GType object_type) {
 	ValaBasicBlock* self;
-	self = ((ValaBasicBlock*) (g_type_create_instance (object_type)));
+	self = (ValaBasicBlock*) g_type_create_instance (object_type);
 	return self;
 }
 
@@ -70,7 +76,7 @@ ValaBasicBlock* vala_basic_block_new_entry (void) {
 
 ValaBasicBlock* vala_basic_block_construct_exit (GType object_type) {
 	ValaBasicBlock* self;
-	self = ((ValaBasicBlock*) (g_type_create_instance (object_type)));
+	self = (ValaBasicBlock*) g_type_create_instance (object_type);
 	return self;
 }
 
@@ -83,31 +89,101 @@ ValaBasicBlock* vala_basic_block_new_exit (void) {
 void vala_basic_block_add_node (ValaBasicBlock* self, ValaCodeNode* node) {
 	g_return_if_fail (self != NULL);
 	g_return_if_fail (node != NULL);
-	gee_collection_add (((GeeCollection*) (self->priv->nodes)), node);
+	gee_collection_add ((GeeCollection*) self->priv->nodes, node);
+}
+
+
+GeeList* vala_basic_block_get_nodes (ValaBasicBlock* self) {
+	GeeList* _tmp0;
+	g_return_val_if_fail (self != NULL, NULL);
+	_tmp0 = NULL;
+	return (_tmp0 = self->priv->nodes, (_tmp0 == NULL) ? NULL : gee_collection_object_ref (_tmp0));
 }
 
 
 void vala_basic_block_connect (ValaBasicBlock* self, ValaBasicBlock* target) {
 	g_return_if_fail (self != NULL);
 	g_return_if_fail (target != NULL);
-	if (!gee_collection_contains (((GeeCollection*) (self->priv->successors)), target)) {
-		gee_collection_add (((GeeCollection*) (self->priv->successors)), target);
+	if (!gee_collection_contains ((GeeCollection*) self->priv->successors, target)) {
+		gee_collection_add ((GeeCollection*) self->priv->successors, target);
 	}
-	if (!gee_collection_contains (((GeeCollection*) (target->priv->predecessors)), self)) {
-		gee_collection_add (((GeeCollection*) (target->priv->predecessors)), self);
+	if (!gee_collection_contains ((GeeCollection*) target->priv->predecessors, self)) {
+		gee_collection_add ((GeeCollection*) target->priv->predecessors, self);
 	}
 }
 
 
 GeeList* vala_basic_block_get_predecessors (ValaBasicBlock* self) {
 	g_return_val_if_fail (self != NULL, NULL);
-	return ((GeeList*) (gee_read_only_list_new (VALA_TYPE_BASIC_BLOCK, NULL, NULL, self->priv->predecessors)));
+	return (GeeList*) gee_read_only_list_new (VALA_TYPE_BASIC_BLOCK, NULL, NULL, self->priv->predecessors);
 }
 
 
 GeeList* vala_basic_block_get_successors (ValaBasicBlock* self) {
 	g_return_val_if_fail (self != NULL, NULL);
-	return ((GeeList*) (gee_read_only_list_new (VALA_TYPE_BASIC_BLOCK, NULL, NULL, self->priv->successors)));
+	return (GeeList*) gee_read_only_list_new (VALA_TYPE_BASIC_BLOCK, NULL, NULL, self->priv->successors);
+}
+
+
+void vala_basic_block_add_child (ValaBasicBlock* self, ValaBasicBlock* block) {
+	g_return_if_fail (self != NULL);
+	g_return_if_fail (block != NULL);
+	gee_collection_add ((GeeCollection*) self->priv->children, block);
+	vala_basic_block_set_parent (block, self);
+}
+
+
+GeeList* vala_basic_block_get_children (ValaBasicBlock* self) {
+	GeeList* _tmp0;
+	g_return_val_if_fail (self != NULL, NULL);
+	_tmp0 = NULL;
+	return (_tmp0 = self->priv->children, (_tmp0 == NULL) ? NULL : gee_collection_object_ref (_tmp0));
+}
+
+
+void vala_basic_block_add_dominator_frontier (ValaBasicBlock* self, ValaBasicBlock* block) {
+	g_return_if_fail (self != NULL);
+	g_return_if_fail (block != NULL);
+	gee_collection_add ((GeeCollection*) self->priv->df, block);
+}
+
+
+GeeSet* vala_basic_block_get_dominator_frontier (ValaBasicBlock* self) {
+	GeeSet* _tmp0;
+	g_return_val_if_fail (self != NULL, NULL);
+	_tmp0 = NULL;
+	return (_tmp0 = self->priv->df, (_tmp0 == NULL) ? NULL : gee_collection_object_ref (_tmp0));
+}
+
+
+void vala_basic_block_add_phi_function (ValaBasicBlock* self, ValaPhiFunction* phi) {
+	g_return_if_fail (self != NULL);
+	g_return_if_fail (phi != NULL);
+	gee_collection_add ((GeeCollection*) self->priv->phi_functions, phi);
+}
+
+
+GeeSet* vala_basic_block_get_phi_functions (ValaBasicBlock* self) {
+	GeeSet* _tmp0;
+	g_return_val_if_fail (self != NULL, NULL);
+	_tmp0 = NULL;
+	return (_tmp0 = self->priv->phi_functions, (_tmp0 == NULL) ? NULL : gee_collection_object_ref (_tmp0));
+}
+
+
+ValaBasicBlock* vala_basic_block_get_parent (ValaBasicBlock* self) {
+	g_return_val_if_fail (self != NULL, NULL);
+	return self->priv->_parent;
+}
+
+
+static void vala_basic_block_set_parent (ValaBasicBlock* self, ValaBasicBlock* value) {
+	ValaBasicBlock* _tmp2;
+	ValaBasicBlock* _tmp1;
+	g_return_if_fail (self != NULL);
+	_tmp2 = NULL;
+	_tmp1 = NULL;
+	self->priv->_parent = (_tmp2 = (_tmp1 = value, (_tmp1 == NULL) ? NULL : vala_basic_block_ref (_tmp1)), (self->priv->_parent == NULL) ? NULL : (self->priv->_parent = (vala_basic_block_unref (self->priv->_parent), NULL)), _tmp2);
 }
 
 
@@ -140,12 +216,13 @@ static gpointer vala_value_basic_block_peek_pointer (const GValue* value) {
 static gchar* vala_value_basic_block_collect_value (GValue* value, guint n_collect_values, GTypeCValue* collect_values, guint collect_flags) {
 	if (collect_values[0].v_pointer) {
 		ValaBasicBlock* object;
-		object = value->data[0].v_pointer;
+		object = collect_values[0].v_pointer;
 		if (object->parent_instance.g_class == NULL) {
 			return g_strconcat ("invalid unclassed object pointer for value type `", G_VALUE_TYPE_NAME (value), "'", NULL);
-		} else if (!g_value_type_compatible (G_OBJECT_TYPE (object), G_VALUE_TYPE (value))) {
-			return g_strconcat ("invalid object type `", G_OBJECT_TYPE (object), "' for value type `", G_VALUE_TYPE_NAME (value), "'", NULL);
+		} else if (!g_value_type_compatible (G_TYPE_FROM_INSTANCE (object), G_VALUE_TYPE (value))) {
+			return g_strconcat ("invalid object type `", g_type_name (G_TYPE_FROM_INSTANCE (object)), "' for value type `", G_VALUE_TYPE_NAME (value), "'", NULL);
 		}
+		value->data[0].v_pointer = vala_basic_block_ref (object);
 	} else {
 		value->data[0].v_pointer = NULL;
 	}
@@ -157,6 +234,7 @@ static gchar* vala_value_basic_block_lcopy_value (const GValue* value, guint n_c
 	ValaBasicBlock** object_p;
 	object_p = collect_values[0].v_pointer;
 	if (!object_p) {
+		return g_strdup_printf ("value location for `%s' passed as NULL", G_VALUE_TYPE_NAME (value));
 	}
 	if (!value->data[0].v_pointer) {
 		*object_p = NULL;
@@ -211,9 +289,12 @@ static void vala_basic_block_class_init (ValaBasicBlockClass * klass) {
 
 static void vala_basic_block_instance_init (ValaBasicBlock * self) {
 	self->priv = VALA_BASIC_BLOCK_GET_PRIVATE (self);
-	self->priv->nodes = ((GeeList*) (gee_array_list_new (VALA_TYPE_CODE_NODE, ((GBoxedCopyFunc) (vala_code_node_ref)), vala_code_node_unref, g_direct_equal)));
-	self->priv->predecessors = ((GeeList*) (gee_array_list_new (VALA_TYPE_BASIC_BLOCK, NULL, NULL, g_direct_equal)));
-	self->priv->successors = ((GeeList*) (gee_array_list_new (VALA_TYPE_BASIC_BLOCK, ((GBoxedCopyFunc) (vala_basic_block_ref)), vala_basic_block_unref, g_direct_equal)));
+	self->priv->nodes = (GeeList*) gee_array_list_new (VALA_TYPE_CODE_NODE, (GBoxedCopyFunc) vala_code_node_ref, vala_code_node_unref, g_direct_equal);
+	self->priv->predecessors = (GeeList*) gee_array_list_new (VALA_TYPE_BASIC_BLOCK, NULL, NULL, g_direct_equal);
+	self->priv->successors = (GeeList*) gee_array_list_new (VALA_TYPE_BASIC_BLOCK, (GBoxedCopyFunc) vala_basic_block_ref, vala_basic_block_unref, g_direct_equal);
+	self->priv->children = (GeeList*) gee_array_list_new (VALA_TYPE_BASIC_BLOCK, (GBoxedCopyFunc) vala_basic_block_ref, vala_basic_block_unref, g_direct_equal);
+	self->priv->df = (GeeSet*) gee_hash_set_new (VALA_TYPE_BASIC_BLOCK, (GBoxedCopyFunc) vala_basic_block_ref, vala_basic_block_unref, g_direct_hash, g_direct_equal);
+	self->priv->phi_functions = (GeeSet*) gee_hash_set_new (VALA_TYPE_PHI_FUNCTION, (GBoxedCopyFunc) vala_phi_function_ref, vala_phi_function_unref, g_direct_hash, g_direct_equal);
 	self->ref_count = 1;
 }
 
@@ -221,9 +302,13 @@ static void vala_basic_block_instance_init (ValaBasicBlock * self) {
 static void vala_basic_block_finalize (ValaBasicBlock* obj) {
 	ValaBasicBlock * self;
 	self = VALA_BASIC_BLOCK (obj);
-	(self->priv->nodes == NULL ? NULL : (self->priv->nodes = (gee_collection_object_unref (self->priv->nodes), NULL)));
-	(self->priv->predecessors == NULL ? NULL : (self->priv->predecessors = (gee_collection_object_unref (self->priv->predecessors), NULL)));
-	(self->priv->successors == NULL ? NULL : (self->priv->successors = (gee_collection_object_unref (self->priv->successors), NULL)));
+	(self->priv->nodes == NULL) ? NULL : (self->priv->nodes = (gee_collection_object_unref (self->priv->nodes), NULL));
+	(self->priv->predecessors == NULL) ? NULL : (self->priv->predecessors = (gee_collection_object_unref (self->priv->predecessors), NULL));
+	(self->priv->successors == NULL) ? NULL : (self->priv->successors = (gee_collection_object_unref (self->priv->successors), NULL));
+	(self->priv->_parent == NULL) ? NULL : (self->priv->_parent = (vala_basic_block_unref (self->priv->_parent), NULL));
+	(self->priv->children == NULL) ? NULL : (self->priv->children = (gee_collection_object_unref (self->priv->children), NULL));
+	(self->priv->df == NULL) ? NULL : (self->priv->df = (gee_collection_object_unref (self->priv->df), NULL));
+	(self->priv->phi_functions == NULL) ? NULL : (self->priv->phi_functions = (gee_collection_object_unref (self->priv->phi_functions), NULL));
 }
 
 
@@ -252,7 +337,7 @@ void vala_basic_block_unref (gpointer instance) {
 	self = instance;
 	if (g_atomic_int_dec_and_test (&self->ref_count)) {
 		VALA_BASIC_BLOCK_GET_CLASS (self)->finalize (self);
-		g_type_free_instance (((GTypeInstance *) (self)));
+		g_type_free_instance ((GTypeInstance *) self);
 	}
 }
 
