@@ -28,11 +28,12 @@
 #include <gee/iterator.h>
 #include <stdlib.h>
 #include <string.h>
-#include <vala/valacodevisitor.h>
-#include <vala/valasourcereference.h>
 #include <vala/valasemanticanalyzer.h>
 #include <vala/valalocalvariable.h>
+#include <vala/valasourcereference.h>
+#include <vala/valacodevisitor.h>
 #include <vala/valastatementlist.h>
+#include <vala/valamethod.h>
 #include <vala/valascope.h>
 #include <vala/valareport.h>
 
@@ -145,32 +146,38 @@ GeeList* vala_block_get_statements (ValaBlock* self) {
  * @param decl a variable declarator
  */
 void vala_block_add_local_variable (ValaBlock* self, ValaLocalVariable* local) {
-	ValaBlock* _tmp1;
 	ValaSymbol* _tmp0;
-	ValaBlock* parent_block;
+	ValaSymbol* parent_block;
 	g_return_if_fail (self != NULL);
 	g_return_if_fail (local != NULL);
-	_tmp1 = NULL;
 	_tmp0 = NULL;
-	parent_block = (_tmp1 = (_tmp0 = vala_symbol_get_parent_symbol ((ValaSymbol*) self), VALA_IS_BLOCK (_tmp0) ? ((ValaBlock*) _tmp0) : NULL), (_tmp1 == NULL) ? NULL : vala_code_node_ref (_tmp1));
-	while (parent_block != NULL) {
+	parent_block = (_tmp0 = vala_symbol_get_parent_symbol ((ValaSymbol*) self), (_tmp0 == NULL) ? NULL : vala_code_node_ref (_tmp0));
+	while (TRUE) {
+		gboolean _tmp1;
 		ValaSymbol* _tmp2;
 		gboolean _tmp3;
-		ValaBlock* _tmp7;
-		ValaBlock* _tmp6;
+		ValaSymbol* _tmp6;
 		ValaSymbol* _tmp5;
+		_tmp1 = FALSE;
+		if (VALA_IS_BLOCK (parent_block)) {
+			_tmp1 = TRUE;
+		} else {
+			_tmp1 = VALA_IS_METHOD (parent_block);
+		}
+		if (!_tmp1) {
+			break;
+		}
 		_tmp2 = NULL;
-		if ((_tmp3 = (_tmp2 = vala_scope_lookup (vala_symbol_get_scope ((ValaSymbol*) parent_block), vala_symbol_get_name ((ValaSymbol*) local))) != NULL, (_tmp2 == NULL) ? NULL : (_tmp2 = (vala_code_node_unref (_tmp2), NULL)), _tmp3)) {
+		if ((_tmp3 = (_tmp2 = vala_scope_lookup (vala_symbol_get_scope (parent_block), vala_symbol_get_name ((ValaSymbol*) local))) != NULL, (_tmp2 == NULL) ? NULL : (_tmp2 = (vala_code_node_unref (_tmp2), NULL)), _tmp3)) {
 			char* _tmp4;
 			_tmp4 = NULL;
 			vala_report_error (vala_code_node_get_source_reference ((ValaCodeNode*) local), _tmp4 = g_strdup_printf ("Local variable `%s' conflicts with another local variable declared in a parent scope", vala_symbol_get_name ((ValaSymbol*) local)));
 			_tmp4 = (g_free (_tmp4), NULL);
 			break;
 		}
-		_tmp7 = NULL;
 		_tmp6 = NULL;
 		_tmp5 = NULL;
-		parent_block = (_tmp7 = (_tmp6 = (_tmp5 = vala_symbol_get_parent_symbol ((ValaSymbol*) parent_block), VALA_IS_BLOCK (_tmp5) ? ((ValaBlock*) _tmp5) : NULL), (_tmp6 == NULL) ? NULL : vala_code_node_ref (_tmp6)), (parent_block == NULL) ? NULL : (parent_block = (vala_code_node_unref (parent_block), NULL)), _tmp7);
+		parent_block = (_tmp6 = (_tmp5 = vala_symbol_get_parent_symbol (parent_block), (_tmp5 == NULL) ? NULL : vala_code_node_ref (_tmp5)), (parent_block == NULL) ? NULL : (parent_block = (vala_code_node_unref (parent_block), NULL)), _tmp6);
 	}
 	gee_collection_add ((GeeCollection*) self->priv->local_variables, local);
 	(parent_block == NULL) ? NULL : (parent_block = (vala_code_node_unref (parent_block), NULL));
@@ -253,7 +260,7 @@ static gboolean vala_block_real_check (ValaCodeNode* base, ValaSemanticAnalyzer*
 		for (; i < gee_collection_get_size ((GeeCollection*) self->priv->statement_list); i++) {
 			ValaStatement* _tmp5;
 			_tmp5 = NULL;
-			vala_code_node_check ((ValaCodeNode*) (_tmp5 = (ValaStatement*) gee_list_get ((GeeList*) self->priv->statement_list, i)), analyzer);
+			vala_code_node_check ((ValaCodeNode*) (_tmp5 = (ValaStatement*) gee_list_get (self->priv->statement_list, i)), analyzer);
 			(_tmp5 == NULL) ? NULL : (_tmp5 = (vala_code_node_unref (_tmp5), NULL));
 		}
 	}
@@ -305,7 +312,7 @@ void vala_block_insert_before (ValaBlock* self, ValaStatement* stmt, ValaStateme
 			ValaStatement* _tmp0;
 			ValaStatementList* stmt_list;
 			_tmp0 = NULL;
-			stmt_list = (_tmp0 = (ValaStatement*) gee_list_get ((GeeList*) self->priv->statement_list, i), VALA_IS_STATEMENT_LIST (_tmp0) ? ((ValaStatementList*) _tmp0) : NULL);
+			stmt_list = (_tmp0 = (ValaStatement*) gee_list_get (self->priv->statement_list, i), VALA_IS_STATEMENT_LIST (_tmp0) ? ((ValaStatementList*) _tmp0) : NULL);
 			if (stmt_list != NULL) {
 				{
 					gint j;
@@ -324,13 +331,13 @@ void vala_block_insert_before (ValaBlock* self, ValaStatement* stmt, ValaStateme
 				ValaStatement* _tmp3;
 				gboolean _tmp4;
 				_tmp3 = NULL;
-				if ((_tmp4 = (_tmp3 = (ValaStatement*) gee_list_get ((GeeList*) self->priv->statement_list, i)) == stmt, (_tmp3 == NULL) ? NULL : (_tmp3 = (vala_code_node_unref (_tmp3), NULL)), _tmp4)) {
+				if ((_tmp4 = (_tmp3 = (ValaStatement*) gee_list_get (self->priv->statement_list, i)) == stmt, (_tmp3 == NULL) ? NULL : (_tmp3 = (vala_code_node_unref (_tmp3), NULL)), _tmp4)) {
 					ValaStatementList* _tmp5;
 					_tmp5 = NULL;
 					stmt_list = (_tmp5 = vala_statement_list_new (vala_code_node_get_source_reference ((ValaCodeNode*) self)), (stmt_list == NULL) ? NULL : (stmt_list = (vala_code_node_unref (stmt_list), NULL)), _tmp5);
 					vala_statement_list_add (stmt_list, new_stmt);
 					vala_statement_list_add (stmt_list, stmt);
-					gee_list_set ((GeeList*) self->priv->statement_list, i, (ValaStatement*) stmt_list);
+					gee_list_set (self->priv->statement_list, i, (ValaStatement*) stmt_list);
 				}
 			}
 			(stmt_list == NULL) ? NULL : (stmt_list = (vala_code_node_unref (stmt_list), NULL));
@@ -350,7 +357,7 @@ void vala_block_replace_statement (ValaBlock* self, ValaStatement* old_stmt, Val
 			ValaStatement* _tmp0;
 			ValaStatementList* stmt_list;
 			_tmp0 = NULL;
-			stmt_list = (_tmp0 = (ValaStatement*) gee_list_get ((GeeList*) self->priv->statement_list, i), VALA_IS_STATEMENT_LIST (_tmp0) ? ((ValaStatementList*) _tmp0) : NULL);
+			stmt_list = (_tmp0 = (ValaStatement*) gee_list_get (self->priv->statement_list, i), VALA_IS_STATEMENT_LIST (_tmp0) ? ((ValaStatementList*) _tmp0) : NULL);
 			if (stmt_list != NULL) {
 				{
 					gint j;
@@ -369,8 +376,8 @@ void vala_block_replace_statement (ValaBlock* self, ValaStatement* old_stmt, Val
 				ValaStatement* _tmp3;
 				gboolean _tmp4;
 				_tmp3 = NULL;
-				if ((_tmp4 = (_tmp3 = (ValaStatement*) gee_list_get ((GeeList*) self->priv->statement_list, i)) == old_stmt, (_tmp3 == NULL) ? NULL : (_tmp3 = (vala_code_node_unref (_tmp3), NULL)), _tmp4)) {
-					gee_list_set ((GeeList*) self->priv->statement_list, i, new_stmt);
+				if ((_tmp4 = (_tmp3 = (ValaStatement*) gee_list_get (self->priv->statement_list, i)) == old_stmt, (_tmp3 == NULL) ? NULL : (_tmp3 = (vala_code_node_unref (_tmp3), NULL)), _tmp4)) {
+					gee_list_set (self->priv->statement_list, i, new_stmt);
 					(stmt_list == NULL) ? NULL : (stmt_list = (vala_code_node_unref (stmt_list), NULL));
 					break;
 				}
