@@ -1,4 +1,4 @@
-/* valacatchclause.vala
+/* valacatchvala
  *
  * Copyright (C) 2007-2008  Jürg Billeter
  *
@@ -26,6 +26,14 @@
 #include <vala/valadatatype.h>
 #include <vala/valasourcereference.h>
 #include <vala/valacodevisitor.h>
+#include <vala/valasemanticanalyzer.h>
+#include <vala/valaexpression.h>
+#include <vala/valasymbol.h>
+#include <vala/valascope.h>
+#include <vala/valaerrortype.h>
+#include <vala/valaerrordomain.h>
+#include <vala/valaerrorcode.h>
+#include <vala/valasourcefile.h>
 
 
 
@@ -45,13 +53,15 @@ enum  {
 static void vala_catch_clause_real_accept (ValaCodeNode* base, ValaCodeVisitor* visitor);
 static void vala_catch_clause_real_accept_children (ValaCodeNode* base, ValaCodeVisitor* visitor);
 static void vala_catch_clause_real_replace_type (ValaCodeNode* base, ValaDataType* old_type, ValaDataType* new_type);
+static gboolean vala_catch_clause_real_check (ValaCodeNode* base, ValaSemanticAnalyzer* analyzer);
+static void vala_catch_clause_real_get_defined_variables (ValaCodeNode* base, GeeCollection* collection);
 static gpointer vala_catch_clause_parent_class = NULL;
 static void vala_catch_clause_finalize (ValaCodeNode* obj);
 
 
 
 /**
- * Creates a new catch clause.
+ * Creates a new catch 
  *
  * @param type_reference   error type
  * @param variable_name    error variable name
@@ -62,11 +72,11 @@ static void vala_catch_clause_finalize (ValaCodeNode* obj);
 ValaCatchClause* vala_catch_clause_construct (GType object_type, ValaDataType* error_type, const char* variable_name, ValaBlock* body, ValaSourceReference* source_reference) {
 	ValaCatchClause* self;
 	g_return_val_if_fail (body != NULL, NULL);
-	self = ((ValaCatchClause*) (g_type_create_instance (object_type)));
+	self = (ValaCatchClause*) g_type_create_instance (object_type);
 	vala_catch_clause_set_error_type (self, error_type);
 	vala_catch_clause_set_variable_name (self, variable_name);
 	vala_catch_clause_set_body (self, body);
-	vala_code_node_set_source_reference (((ValaCodeNode*) (self)), source_reference);
+	vala_code_node_set_source_reference ((ValaCodeNode*) self, source_reference);
 	return self;
 }
 
@@ -78,7 +88,7 @@ ValaCatchClause* vala_catch_clause_new (ValaDataType* error_type, const char* va
 
 static void vala_catch_clause_real_accept (ValaCodeNode* base, ValaCodeVisitor* visitor) {
 	ValaCatchClause * self;
-	self = ((ValaCatchClause*) (base));
+	self = (ValaCatchClause*) base;
 	g_return_if_fail (visitor != NULL);
 	vala_code_visitor_visit_catch_clause (visitor, self);
 }
@@ -86,18 +96,18 @@ static void vala_catch_clause_real_accept (ValaCodeNode* base, ValaCodeVisitor*
 
 static void vala_catch_clause_real_accept_children (ValaCodeNode* base, ValaCodeVisitor* visitor) {
 	ValaCatchClause * self;
-	self = ((ValaCatchClause*) (base));
+	self = (ValaCatchClause*) base;
 	g_return_if_fail (visitor != NULL);
 	if (vala_catch_clause_get_error_type (self) != NULL) {
-		vala_code_node_accept (((ValaCodeNode*) (vala_catch_clause_get_error_type (self))), visitor);
+		vala_code_node_accept ((ValaCodeNode*) vala_catch_clause_get_error_type (self), visitor);
 	}
-	vala_code_node_accept (((ValaCodeNode*) (self->priv->_body)), visitor);
+	vala_code_node_accept ((ValaCodeNode*) self->priv->_body, visitor);
 }
 
 
 static void vala_catch_clause_real_replace_type (ValaCodeNode* base, ValaDataType* old_type, ValaDataType* new_type) {
 	ValaCatchClause * self;
-	self = ((ValaCatchClause*) (base));
+	self = (ValaCatchClause*) base;
 	g_return_if_fail (old_type != NULL);
 	g_return_if_fail (new_type != NULL);
 	if (vala_catch_clause_get_error_type (self) == old_type) {
@@ -106,6 +116,48 @@ static void vala_catch_clause_real_replace_type (ValaCodeNode* base, ValaDataTyp
 }
 
 
+static gboolean vala_catch_clause_real_check (ValaCodeNode* base, ValaSemanticAnalyzer* analyzer) {
+	ValaCatchClause * self;
+	self = (ValaCatchClause*) base;
+	g_return_val_if_fail (analyzer != NULL, FALSE);
+	if (vala_code_node_get_checked ((ValaCodeNode*) self)) {
+		return !vala_code_node_get_error ((ValaCodeNode*) self);
+	}
+	vala_code_node_set_checked ((ValaCodeNode*) self, TRUE);
+	if (vala_catch_clause_get_error_type (self) != NULL) {
+		ValaLocalVariable* _tmp2;
+		ValaDataType* _tmp1;
+		_tmp2 = NULL;
+		_tmp1 = NULL;
+		vala_catch_clause_set_error_variable (self, _tmp2 = vala_local_variable_new (_tmp1 = vala_data_type_copy (vala_catch_clause_get_error_type (self)), self->priv->_variable_name, NULL, NULL));
+		(_tmp2 == NULL) ? NULL : (_tmp2 = (vala_code_node_unref (_tmp2), NULL));
+		(_tmp1 == NULL) ? NULL : (_tmp1 = (vala_code_node_unref (_tmp1), NULL));
+		vala_scope_add (vala_symbol_get_scope ((ValaSymbol*) self->priv->_body), self->priv->_variable_name, (ValaSymbol*) self->priv->_error_variable);
+		vala_block_add_local_variable (self->priv->_body, self->priv->_error_variable);
+		vala_code_node_set_checked ((ValaCodeNode*) self->priv->_error_variable, TRUE);
+	} else {
+		ValaErrorType* _tmp3;
+		_tmp3 = NULL;
+		vala_catch_clause_set_error_type (self, (ValaDataType*) (_tmp3 = vala_error_type_new (NULL, NULL, vala_code_node_get_source_reference ((ValaCodeNode*) self))));
+		(_tmp3 == NULL) ? NULL : (_tmp3 = (vala_code_node_unref (_tmp3), NULL));
+	}
+	vala_code_node_check ((ValaCodeNode*) vala_catch_clause_get_error_type (self), analyzer);
+	vala_source_file_add_type_dependency (vala_semantic_analyzer_get_current_source_file (analyzer), vala_catch_clause_get_error_type (self), VALA_SOURCE_FILE_DEPENDENCY_TYPE_SOURCE);
+	vala_code_node_check ((ValaCodeNode*) self->priv->_body, analyzer);
+	return !vala_code_node_get_error ((ValaCodeNode*) self);
+}
+
+
+static void vala_catch_clause_real_get_defined_variables (ValaCodeNode* base, GeeCollection* collection) {
+	ValaCatchClause * self;
+	self = (ValaCatchClause*) base;
+	g_return_if_fail (collection != NULL);
+	if (self->priv->_error_variable != NULL) {
+		gee_collection_add (collection, self->priv->_error_variable);
+	}
+}
+
+
 ValaDataType* vala_catch_clause_get_error_type (ValaCatchClause* self) {
 	g_return_val_if_fail (self != NULL, NULL);
 	return self->priv->_data_type;
@@ -118,9 +170,9 @@ void vala_catch_clause_set_error_type (ValaCatchClause* self, ValaDataType* valu
 	g_return_if_fail (self != NULL);
 	_tmp2 = NULL;
 	_tmp1 = NULL;
-	self->priv->_data_type = (_tmp2 = (_tmp1 = value, (_tmp1 == NULL ? NULL : vala_code_node_ref (_tmp1))), (self->priv->_data_type == NULL ? NULL : (self->priv->_data_type = (vala_code_node_unref (self->priv->_data_type), NULL))), _tmp2);
+	self->priv->_data_type = (_tmp2 = (_tmp1 = value, (_tmp1 == NULL) ? NULL : vala_code_node_ref (_tmp1)), (self->priv->_data_type == NULL) ? NULL : (self->priv->_data_type = (vala_code_node_unref (self->priv->_data_type), NULL)), _tmp2);
 	if (self->priv->_data_type != NULL) {
-		vala_code_node_set_parent_node (((ValaCodeNode*) (self->priv->_data_type)), ((ValaCodeNode*) (self)));
+		vala_code_node_set_parent_node ((ValaCodeNode*) self->priv->_data_type, (ValaCodeNode*) self);
 	}
 }
 
@@ -137,7 +189,7 @@ void vala_catch_clause_set_variable_name (ValaCatchClause* self, const char* val
 	g_return_if_fail (self != NULL);
 	_tmp2 = NULL;
 	_tmp1 = NULL;
-	self->priv->_variable_name = (_tmp2 = (_tmp1 = value, (_tmp1 == NULL ? NULL : g_strdup (_tmp1))), (self->priv->_variable_name = (g_free (self->priv->_variable_name), NULL)), _tmp2);
+	self->priv->_variable_name = (_tmp2 = (_tmp1 = value, (_tmp1 == NULL) ? NULL : g_strdup (_tmp1)), self->priv->_variable_name = (g_free (self->priv->_variable_name), NULL), _tmp2);
 }
 
 
@@ -153,7 +205,7 @@ void vala_catch_clause_set_body (ValaCatchClause* self, ValaBlock* value) {
 	g_return_if_fail (self != NULL);
 	_tmp2 = NULL;
 	_tmp1 = NULL;
-	self->priv->_body = (_tmp2 = (_tmp1 = value, (_tmp1 == NULL ? NULL : vala_code_node_ref (_tmp1))), (self->priv->_body == NULL ? NULL : (self->priv->_body = (vala_code_node_unref (self->priv->_body), NULL))), _tmp2);
+	self->priv->_body = (_tmp2 = (_tmp1 = value, (_tmp1 == NULL) ? NULL : vala_code_node_ref (_tmp1)), (self->priv->_body == NULL) ? NULL : (self->priv->_body = (vala_code_node_unref (self->priv->_body), NULL)), _tmp2);
 }
 
 
@@ -169,7 +221,7 @@ void vala_catch_clause_set_error_variable (ValaCatchClause* self, ValaLocalVaria
 	g_return_if_fail (self != NULL);
 	_tmp2 = NULL;
 	_tmp1 = NULL;
-	self->priv->_error_variable = (_tmp2 = (_tmp1 = value, (_tmp1 == NULL ? NULL : vala_code_node_ref (_tmp1))), (self->priv->_error_variable == NULL ? NULL : (self->priv->_error_variable = (vala_code_node_unref (self->priv->_error_variable), NULL))), _tmp2);
+	self->priv->_error_variable = (_tmp2 = (_tmp1 = value, (_tmp1 == NULL) ? NULL : vala_code_node_ref (_tmp1)), (self->priv->_error_variable == NULL) ? NULL : (self->priv->_error_variable = (vala_code_node_unref (self->priv->_error_variable), NULL)), _tmp2);
 }
 
 
@@ -185,7 +237,7 @@ void vala_catch_clause_set_clabel_name (ValaCatchClause* self, const char* value
 	g_return_if_fail (self != NULL);
 	_tmp2 = NULL;
 	_tmp1 = NULL;
-	self->priv->_clabel_name = (_tmp2 = (_tmp1 = value, (_tmp1 == NULL ? NULL : g_strdup (_tmp1))), (self->priv->_clabel_name = (g_free (self->priv->_clabel_name), NULL)), _tmp2);
+	self->priv->_clabel_name = (_tmp2 = (_tmp1 = value, (_tmp1 == NULL) ? NULL : g_strdup (_tmp1)), self->priv->_clabel_name = (g_free (self->priv->_clabel_name), NULL), _tmp2);
 }
 
 
@@ -196,6 +248,8 @@ static void vala_catch_clause_class_init (ValaCatchClauseClass * klass) {
 	VALA_CODE_NODE_CLASS (klass)->accept = vala_catch_clause_real_accept;
 	VALA_CODE_NODE_CLASS (klass)->accept_children = vala_catch_clause_real_accept_children;
 	VALA_CODE_NODE_CLASS (klass)->replace_type = vala_catch_clause_real_replace_type;
+	VALA_CODE_NODE_CLASS (klass)->check = vala_catch_clause_real_check;
+	VALA_CODE_NODE_CLASS (klass)->get_defined_variables = vala_catch_clause_real_get_defined_variables;
 }
 
 
@@ -208,10 +262,10 @@ static void vala_catch_clause_finalize (ValaCodeNode* obj) {
 	ValaCatchClause * self;
 	self = VALA_CATCH_CLAUSE (obj);
 	self->priv->_variable_name = (g_free (self->priv->_variable_name), NULL);
-	(self->priv->_body == NULL ? NULL : (self->priv->_body = (vala_code_node_unref (self->priv->_body), NULL)));
-	(self->priv->_error_variable == NULL ? NULL : (self->priv->_error_variable = (vala_code_node_unref (self->priv->_error_variable), NULL)));
+	(self->priv->_body == NULL) ? NULL : (self->priv->_body = (vala_code_node_unref (self->priv->_body), NULL));
+	(self->priv->_error_variable == NULL) ? NULL : (self->priv->_error_variable = (vala_code_node_unref (self->priv->_error_variable), NULL));
 	self->priv->_clabel_name = (g_free (self->priv->_clabel_name), NULL);
-	(self->priv->_data_type == NULL ? NULL : (self->priv->_data_type = (vala_code_node_unref (self->priv->_data_type), NULL)));
+	(self->priv->_data_type == NULL) ? NULL : (self->priv->_data_type = (vala_code_node_unref (self->priv->_data_type), NULL));
 	VALA_CODE_NODE_CLASS (vala_catch_clause_parent_class)->finalize (obj);
 }
 
