@@ -145,7 +145,7 @@ void vala_character_literal_set_value (ValaCharacterLiteral* self, const char* v
 		} else {
 			_tmp4 = FALSE;
 		}
-		_tmp3 = (_tmp4);
+		_tmp3 = _tmp4;
 	}
 	if (_tmp3) {
 		vala_code_node_set_error ((ValaCodeNode*) self, TRUE);
