@@ -21,13 +21,14 @@
  */
 
 #include <vala/valacodecontext.h>
+#include <gee/collection.h>
 #include <gee/arraylist.h>
 #include <gee/iterable.h>
 #include <gee/iterator.h>
 #include <gee/readonlylist.h>
-#include <gee/collection.h>
 #include <glib/gstdio.h>
 #include <config.h>
+#include <vala/valareport.h>
 #include <vala/valanamespace.h>
 #include <vala/valasourcereference.h>
 #include <vala/valamethod.h>
@@ -62,6 +63,7 @@ struct _ValaCodeContextPrivate {
 	gboolean _save_temps;
 	gint _target_glib_major;
 	gint _target_glib_minor;
+	ValaReport* _report;
 	GeeList* source_files;
 	GeeList* c_source_files;
 	ValaNamespace* _root;
@@ -74,6 +76,7 @@ struct _ValaCodeContextPrivate {
 enum  {
 	VALA_CODE_CONTEXT_DUMMY_PROPERTY
 };
+static GeeList* vala_code_context_context_stack = NULL;
 static ValaSourceFile* vala_code_context_find_cycle_head (ValaCodeContext* self, ValaSourceFile* file);
 static void vala_code_context_visit (ValaCodeContext* self, ValaSourceFile* file, GeeList* chain);
 static gpointer vala_code_context_parent_class = NULL;
@@ -90,7 +93,7 @@ gboolean vala_code_context_require_glib_version (ValaCodeContext* self, gint maj
 	gboolean _tmp0;
 	g_return_val_if_fail (self != NULL, FALSE);
 	_tmp0 = FALSE;
-	if ((self->priv->_target_glib_major > major)) {
+	if (self->priv->_target_glib_major > major) {
 		_tmp0 = TRUE;
 	} else {
 		gboolean _tmp1;
@@ -100,7 +103,7 @@ gboolean vala_code_context_require_glib_version (ValaCodeContext* self, gint maj
 		} else {
 			_tmp1 = FALSE;
 		}
-		_tmp0 = (_tmp1);
+		_tmp0 = _tmp1;
 	}
 	return _tmp0;
 }
@@ -118,6 +121,36 @@ ValaCodeContext* vala_code_context_new (void) {
 }
 
 
+/**
+ * Return the topmost context from the context stack.
+ */
+ValaCodeContext* vala_code_context_get (void) {
+	return (ValaCodeContext*) gee_list_get (vala_code_context_context_stack, gee_collection_get_size ((GeeCollection*) vala_code_context_context_stack) - 1);
+}
+
+
+/**
+ * Push the specified context to the context stack.
+ */
+void vala_code_context_push (ValaCodeContext* context) {
+	g_return_if_fail (context != NULL);
+	if (vala_code_context_context_stack == NULL) {
+		GeeList* _tmp0;
+		_tmp0 = NULL;
+		vala_code_context_context_stack = (_tmp0 = (GeeList*) gee_array_list_new (VALA_TYPE_CODE_CONTEXT, (GBoxedCopyFunc) vala_code_context_ref, vala_code_context_unref, g_direct_equal), (vala_code_context_context_stack == NULL) ? NULL : (vala_code_context_context_stack = (gee_collection_object_unref (vala_code_context_context_stack), NULL)), _tmp0);
+	}
+	gee_collection_add ((GeeCollection*) vala_code_context_context_stack, context);
+}
+
+
+/**
+ * Remove the topmost context from the context stack.
+ */
+void vala_code_context_pop (void) {
+	gee_list_remove_at (vala_code_context_context_stack, gee_collection_get_size ((GeeCollection*) vala_code_context_context_stack) - 1);
+}
+
+
 /**
  * Returns a copy of the list of source files.
  *
@@ -771,6 +804,12 @@ void vala_code_context_set_directory (ValaCodeContext* self, const char* value)
 }
 
 
+gboolean vala_code_context_get_legacy_headers (ValaCodeContext* self) {
+	g_return_val_if_fail (self != NULL, FALSE);
+	return self->priv->_header_filename == NULL;
+}
+
+
 gboolean vala_code_context_get_debug (ValaCodeContext* self) {
 	g_return_val_if_fail (self != NULL, FALSE);
 	return self->priv->_debug;
@@ -878,6 +917,22 @@ gboolean vala_code_context_get_save_cheaders (ValaCodeContext* self) {
 }
 
 
+ValaReport* vala_code_context_get_report (ValaCodeContext* self) {
+	g_return_val_if_fail (self != NULL, NULL);
+	return self->priv->_report;
+}
+
+
+void vala_code_context_set_report (ValaCodeContext* self, ValaReport* value) {
+	ValaReport* _tmp2;
+	ValaReport* _tmp1;
+	g_return_if_fail (self != NULL);
+	_tmp2 = NULL;
+	_tmp1 = NULL;
+	self->priv->_report = (_tmp2 = (_tmp1 = value, (_tmp1 == NULL) ? NULL : g_object_ref (_tmp1)), (self->priv->_report == NULL) ? NULL : (self->priv->_report = (g_object_unref (self->priv->_report), NULL)), _tmp2);
+}
+
+
 ValaNamespace* vala_code_context_get_root (ValaCodeContext* self) {
 	g_return_val_if_fail (self != NULL, NULL);
 	return self->priv->_root;
@@ -1002,6 +1057,7 @@ static void vala_code_context_class_init (ValaCodeContextClass * klass) {
 
 static void vala_code_context_instance_init (ValaCodeContext * self) {
 	self->priv = VALA_CODE_CONTEXT_GET_PRIVATE (self);
+	self->priv->_report = vala_report_new ();
 	self->priv->source_files = (GeeList*) gee_array_list_new (VALA_TYPE_SOURCE_FILE, (GBoxedCopyFunc) vala_source_file_ref, vala_source_file_unref, g_direct_equal);
 	self->priv->c_source_files = (GeeList*) gee_array_list_new (G_TYPE_STRING, (GBoxedCopyFunc) g_strdup, g_free, g_direct_equal);
 	self->priv->_root = vala_namespace_new (NULL, NULL);
@@ -1021,6 +1077,7 @@ static void vala_code_context_finalize (ValaCodeContext* obj) {
 	self->priv->_basedir = (g_free (self->priv->_basedir), NULL);
 	self->priv->_directory = (g_free (self->priv->_directory), NULL);
 	(self->priv->_module_init_method == NULL) ? NULL : (self->priv->_module_init_method = (vala_code_node_unref (self->priv->_module_init_method), NULL));
+	(self->priv->_report == NULL) ? NULL : (self->priv->_report = (g_object_unref (self->priv->_report), NULL));
 	(self->priv->source_files == NULL) ? NULL : (self->priv->source_files = (gee_collection_object_unref (self->priv->source_files), NULL));
 	(self->priv->c_source_files == NULL) ? NULL : (self->priv->c_source_files = (gee_collection_object_unref (self->priv->c_source_files), NULL));
 	(self->priv->_root == NULL) ? NULL : (self->priv->_root = (vala_code_node_unref (self->priv->_root), NULL));
