@@ -171,7 +171,7 @@ void vala_code_node_accept_children (ValaCodeNode* self, ValaCodeVisitor* visito
 static gboolean vala_code_node_real_check (ValaCodeNode* self, ValaSemanticAnalyzer* analyzer) {
 	g_return_val_if_fail (self != NULL, FALSE);
 	g_return_val_if_fail (analyzer != NULL, FALSE);
-	return FALSE;
+	return TRUE;
 }
 
 
@@ -292,13 +292,7 @@ char* vala_code_node_get_temp_name (ValaCodeNode* self) {
 	g_return_val_if_fail (self != NULL, NULL);
 	_tmp0 = NULL;
 	_tmp1 = NULL;
-	return (_tmp1 = g_strconcat (".", _tmp0 = g_strdup_printf ("%i", ((vala_code_node_last_temp_nr = vala_code_node_last_temp_nr + 1))), NULL), _tmp0 = (g_free (_tmp0), NULL), _tmp1);
-}
-
-
-const char* vala_code_node_get_type_name (ValaCodeNode* self) {
-	g_return_val_if_fail (self != NULL, NULL);
-	return g_type_name (G_TYPE_FROM_INSTANCE (self));
+	return (_tmp1 = g_strconcat (".", _tmp0 = g_strdup_printf ("%i", vala_code_node_last_temp_nr = vala_code_node_last_temp_nr + 1), NULL), _tmp0 = (g_free (_tmp0), NULL), _tmp1);
 }
 
 
@@ -330,6 +324,12 @@ void vala_code_node_set_source_reference (ValaCodeNode* self, ValaSourceReferenc
 }
 
 
+const char* vala_code_node_get_type_name (ValaCodeNode* self) {
+	g_return_val_if_fail (self != NULL, NULL);
+	return g_type_name (G_TYPE_FROM_INSTANCE (self));
+}
+
+
 ValaCCodeNode* vala_code_node_get_ccodenode (ValaCodeNode* self) {
 	g_return_val_if_fail (self != NULL, NULL);
 	return self->priv->_ccodenode;
