@@ -23,10 +23,11 @@
 #include <vala/valadestructor.h>
 #include <stdlib.h>
 #include <string.h>
-#include <vala/valaformalparameter.h>
-#include <vala/valacodevisitor.h>
 #include <vala/valablock.h>
+#include <vala/valaformalparameter.h>
 #include <vala/valasourcereference.h>
+#include <vala/valacodevisitor.h>
+#include <vala/valasemanticanalyzer.h>
 
 
 
@@ -43,6 +44,7 @@ enum  {
 };
 static void vala_destructor_real_accept (ValaCodeNode* base, ValaCodeVisitor* visitor);
 static void vala_destructor_real_accept_children (ValaCodeNode* base, ValaCodeVisitor* visitor);
+static gboolean vala_destructor_real_check (ValaCodeNode* base, ValaSemanticAnalyzer* analyzer);
 static gpointer vala_destructor_parent_class = NULL;
 static void vala_destructor_finalize (ValaCodeNode* obj);
 
@@ -56,8 +58,7 @@ static void vala_destructor_finalize (ValaCodeNode* obj);
  */
 ValaDestructor* vala_destructor_construct (GType object_type, ValaSourceReference* source_reference) {
 	ValaDestructor* self;
-	self = ((ValaDestructor*) (g_type_create_instance (object_type)));
-	self = ((ValaDestructor*) (vala_symbol_construct (object_type, NULL, source_reference)));
+	self = (ValaDestructor*) vala_symbol_construct (object_type, NULL, source_reference);
 	return self;
 }
 
@@ -69,7 +70,7 @@ ValaDestructor* vala_destructor_new (ValaSourceReference* source_reference) {
 
 static void vala_destructor_real_accept (ValaCodeNode* base, ValaCodeVisitor* visitor) {
 	ValaDestructor * self;
-	self = ((ValaDestructor*) (base));
+	self = (ValaDestructor*) base;
 	g_return_if_fail (visitor != NULL);
 	vala_code_visitor_visit_destructor (visitor, self);
 }
@@ -77,14 +78,32 @@ static void vala_destructor_real_accept (ValaCodeNode* base, ValaCodeVisitor* vi
 
 static void vala_destructor_real_accept_children (ValaCodeNode* base, ValaCodeVisitor* visitor) {
 	ValaDestructor * self;
-	self = ((ValaDestructor*) (base));
+	self = (ValaDestructor*) base;
 	g_return_if_fail (visitor != NULL);
 	if (self->priv->_body != NULL) {
-		vala_code_node_accept (((ValaCodeNode*) (self->priv->_body)), visitor);
+		vala_code_node_accept ((ValaCodeNode*) self->priv->_body, visitor);
 	}
 }
 
 
+static gboolean vala_destructor_real_check (ValaCodeNode* base, ValaSemanticAnalyzer* analyzer) {
+	ValaDestructor * self;
+	self = (ValaDestructor*) base;
+	g_return_val_if_fail (analyzer != NULL, FALSE);
+	if (vala_code_node_get_checked ((ValaCodeNode*) self)) {
+		return !vala_code_node_get_error ((ValaCodeNode*) self);
+	}
+	vala_code_node_set_checked ((ValaCodeNode*) self, TRUE);
+	vala_symbol_set_owner ((ValaSymbol*) self, vala_symbol_get_scope (vala_semantic_analyzer_get_current_symbol (analyzer)));
+	vala_semantic_analyzer_set_current_symbol (analyzer, (ValaSymbol*) self);
+	if (self->priv->_body != NULL) {
+		vala_code_node_check ((ValaCodeNode*) self->priv->_body, analyzer);
+	}
+	vala_semantic_analyzer_set_current_symbol (analyzer, vala_symbol_get_parent_symbol (vala_semantic_analyzer_get_current_symbol (analyzer)));
+	return !vala_code_node_get_error ((ValaCodeNode*) self);
+}
+
+
 ValaBlock* vala_destructor_get_body (ValaDestructor* self) {
 	g_return_val_if_fail (self != NULL, NULL);
 	return self->priv->_body;
@@ -97,7 +116,7 @@ void vala_destructor_set_body (ValaDestructor* self, ValaBlock* value) {
 	g_return_if_fail (self != NULL);
 	_tmp2 = NULL;
 	_tmp1 = NULL;
-	self->priv->_body = (_tmp2 = (_tmp1 = value, (_tmp1 == NULL ? NULL : vala_code_node_ref (_tmp1))), (self->priv->_body == NULL ? NULL : (self->priv->_body = (vala_code_node_unref (self->priv->_body), NULL))), _tmp2);
+	self->priv->_body = (_tmp2 = (_tmp1 = value, (_tmp1 == NULL) ? NULL : vala_code_node_ref (_tmp1)), (self->priv->_body == NULL) ? NULL : (self->priv->_body = (vala_code_node_unref (self->priv->_body), NULL)), _tmp2);
 }
 
 
@@ -113,7 +132,7 @@ void vala_destructor_set_this_parameter (ValaDestructor* self, ValaFormalParamet
 	g_return_if_fail (self != NULL);
 	_tmp2 = NULL;
 	_tmp1 = NULL;
-	self->priv->_this_parameter = (_tmp2 = (_tmp1 = value, (_tmp1 == NULL ? NULL : vala_code_node_ref (_tmp1))), (self->priv->_this_parameter == NULL ? NULL : (self->priv->_this_parameter = (vala_code_node_unref (self->priv->_this_parameter), NULL))), _tmp2);
+	self->priv->_this_parameter = (_tmp2 = (_tmp1 = value, (_tmp1 == NULL) ? NULL : vala_code_node_ref (_tmp1)), (self->priv->_this_parameter == NULL) ? NULL : (self->priv->_this_parameter = (vala_code_node_unref (self->priv->_this_parameter), NULL)), _tmp2);
 }
 
 
@@ -135,6 +154,7 @@ static void vala_destructor_class_init (ValaDestructorClass * klass) {
 	g_type_class_add_private (klass, sizeof (ValaDestructorPrivate));
 	VALA_CODE_NODE_CLASS (klass)->accept = vala_destructor_real_accept;
 	VALA_CODE_NODE_CLASS (klass)->accept_children = vala_destructor_real_accept_children;
+	VALA_CODE_NODE_CLASS (klass)->check = vala_destructor_real_check;
 }
 
 
@@ -147,8 +167,8 @@ static void vala_destructor_instance_init (ValaDestructor * self) {
 static void vala_destructor_finalize (ValaCodeNode* obj) {
 	ValaDestructor * self;
 	self = VALA_DESTRUCTOR (obj);
-	(self->priv->_body == NULL ? NULL : (self->priv->_body = (vala_code_node_unref (self->priv->_body), NULL)));
-	(self->priv->_this_parameter == NULL ? NULL : (self->priv->_this_parameter = (vala_code_node_unref (self->priv->_this_parameter), NULL)));
+	(self->priv->_body == NULL) ? NULL : (self->priv->_body = (vala_code_node_unref (self->priv->_body), NULL));
+	(self->priv->_this_parameter == NULL) ? NULL : (self->priv->_this_parameter = (vala_code_node_unref (self->priv->_this_parameter), NULL));
 	VALA_CODE_NODE_CLASS (vala_destructor_parent_class)->finalize (obj);
 }
 
