@@ -69,6 +69,7 @@ static char* vala_enum_get_lower_case_csuffix (ValaEnum* self);
 static char* vala_enum_real_get_lower_case_cname (ValaSymbol* base, const char* infix);
 static char* vala_enum_real_get_upper_case_cname (ValaTypeSymbol* base, const char* infix);
 static gboolean vala_enum_real_is_reference_type (ValaTypeSymbol* base);
+static char* vala_enum_real_get_cprefix (ValaSymbol* base);
 static void vala_enum_process_ccode_attribute (ValaEnum* self, ValaAttribute* a);
 static char* vala_enum_real_get_type_id (ValaTypeSymbol* base);
 static char* vala_enum_real_get_marshaller_type_name (ValaTypeSymbol* base);
@@ -232,6 +233,17 @@ static char* vala_enum_real_get_cname (ValaTypeSymbol* base, gboolean const_type
 }
 
 
+void vala_enum_set_cname (ValaEnum* self, const char* cname) {
+	char* _tmp1;
+	const char* _tmp0;
+	g_return_if_fail (self != NULL);
+	g_return_if_fail (cname != NULL);
+	_tmp1 = NULL;
+	_tmp0 = NULL;
+	self->priv->cname = (_tmp1 = (_tmp0 = cname, (_tmp0 == NULL) ? NULL : g_strdup (_tmp0)), self->priv->cname = (g_free (self->priv->cname), NULL), _tmp1);
+}
+
+
 static char* vala_enum_real_get_lower_case_cprefix (ValaSymbol* base) {
 	ValaEnum * self;
 	const char* _tmp2;
@@ -296,15 +308,10 @@ static gboolean vala_enum_real_is_reference_type (ValaTypeSymbol* base) {
 }
 
 
-/**
- * Returns the string to be prepended to the name of members of this
- * enum when used in C code.
- *
- * @return the prefix to be used in C code
- */
-char* vala_enum_get_cprefix (ValaEnum* self) {
+static char* vala_enum_real_get_cprefix (ValaSymbol* base) {
+	ValaEnum * self;
 	const char* _tmp2;
-	g_return_val_if_fail (self != NULL, NULL);
+	self = (ValaEnum*) base;
 	if (self->priv->cprefix == NULL) {
 		char* _tmp1;
 		char* _tmp0;
@@ -606,6 +613,7 @@ static void vala_enum_class_init (ValaEnumClass * klass) {
 	VALA_SYMBOL_CLASS (klass)->get_lower_case_cname = vala_enum_real_get_lower_case_cname;
 	VALA_TYPESYMBOL_CLASS (klass)->get_upper_case_cname = vala_enum_real_get_upper_case_cname;
 	VALA_TYPESYMBOL_CLASS (klass)->is_reference_type = vala_enum_real_is_reference_type;
+	VALA_SYMBOL_CLASS (klass)->get_cprefix = vala_enum_real_get_cprefix;
 	VALA_TYPESYMBOL_CLASS (klass)->get_type_id = vala_enum_real_get_type_id;
 	VALA_TYPESYMBOL_CLASS (klass)->get_marshaller_type_name = vala_enum_real_get_marshaller_type_name;
 	VALA_TYPESYMBOL_CLASS (klass)->get_get_value_function = vala_enum_real_get_get_value_function;
