@@ -121,7 +121,7 @@ static char* vala_error_code_real_get_cname (ValaTypeSymbol* base, gboolean cons
 		edomain = (_tmp0 = VALA_ERROR_DOMAIN (vala_symbol_get_parent_symbol ((ValaSymbol*) self)), (_tmp0 == NULL) ? NULL : vala_code_node_ref (_tmp0));
 		_tmp2 = NULL;
 		_tmp1 = NULL;
-		self->priv->cname = (_tmp2 = g_strdup_printf ("%s%s", _tmp1 = vala_error_domain_get_cprefix (edomain), vala_symbol_get_name ((ValaSymbol*) self)), self->priv->cname = (g_free (self->priv->cname), NULL), _tmp2);
+		self->priv->cname = (_tmp2 = g_strdup_printf ("%s%s", _tmp1 = vala_symbol_get_cprefix ((ValaSymbol*) edomain), vala_symbol_get_name ((ValaSymbol*) self)), self->priv->cname = (g_free (self->priv->cname), NULL), _tmp2);
 		_tmp1 = (g_free (_tmp1), NULL);
 		(edomain == NULL) ? NULL : (edomain = (vala_code_node_unref (edomain), NULL));
 	}
