@@ -48,6 +48,7 @@ struct _ValaFormalParameterPrivate {
 	double _cparameter_position;
 	double _carray_length_parameter_position;
 	double _cdelegate_target_parameter_position;
+	gboolean _async_only;
 	char* _ctype;
 	ValaDataType* _data_type;
 };
@@ -162,6 +163,9 @@ static void vala_formal_parameter_process_ccode_attribute (ValaFormalParameter*
 	if (vala_attribute_has_argument (a, "delegate_target_pos")) {
 		vala_formal_parameter_set_cdelegate_target_parameter_position (self, vala_attribute_get_double (a, "delegate_target_pos"));
 	}
+	if (vala_attribute_has_argument (a, "async_only")) {
+		vala_formal_parameter_set_async_only (self, vala_attribute_get_bool (a, "async_only"));
+	}
 }
 
 
@@ -233,7 +237,7 @@ static gboolean vala_formal_parameter_real_check (ValaCodeNode* base, ValaSemant
 		vala_code_node_check ((ValaCodeNode*) vala_formal_parameter_get_parameter_type (self), analyzer);
 		_tmp3 = FALSE;
 		if (self->priv->_params_array) {
-			_tmp3 = !(VALA_IS_ARRAY_TYPE (vala_formal_parameter_get_parameter_type (self)));
+			_tmp3 = !VALA_IS_ARRAY_TYPE (vala_formal_parameter_get_parameter_type (self));
 		} else {
 			_tmp3 = FALSE;
 		}
@@ -438,6 +442,18 @@ void vala_formal_parameter_set_cdelegate_target_parameter_position (ValaFormalPa
 }
 
 
+gboolean vala_formal_parameter_get_async_only (ValaFormalParameter* self) {
+	g_return_val_if_fail (self != NULL, FALSE);
+	return self->priv->_async_only;
+}
+
+
+void vala_formal_parameter_set_async_only (ValaFormalParameter* self, gboolean value) {
+	g_return_if_fail (self != NULL);
+	self->priv->_async_only = value;
+}
+
+
 const char* vala_formal_parameter_get_ctype (ValaFormalParameter* self) {
 	g_return_val_if_fail (self != NULL, NULL);
 	return self->priv->_ctype;
