@@ -200,13 +200,13 @@ static gboolean vala_initializer_list_real_check (ValaCodeNode* base, ValaSemant
 			array_type = (_tmp2 = VALA_ARRAY_TYPE (vala_expression_get_target_type ((ValaExpression*) self)), (_tmp2 == NULL) ? NULL : vala_code_node_ref (_tmp2));
 			_tmp3 = FALSE;
 			_tmp4 = FALSE;
-			if (!(VALA_IS_ARRAY_CREATION_EXPRESSION (vala_code_node_get_parent_node ((ValaCodeNode*) self)))) {
-				_tmp4 = !(VALA_IS_CONSTANT (vala_code_node_get_parent_node ((ValaCodeNode*) self)));
+			if (!VALA_IS_ARRAY_CREATION_EXPRESSION (vala_code_node_get_parent_node ((ValaCodeNode*) self))) {
+				_tmp4 = !VALA_IS_CONSTANT (vala_code_node_get_parent_node ((ValaCodeNode*) self));
 			} else {
 				_tmp4 = FALSE;
 			}
 			if (_tmp4) {
-				_tmp3 = !(VALA_IS_INITIALIZER_LIST (vala_code_node_get_parent_node ((ValaCodeNode*) self)));
+				_tmp3 = !VALA_IS_INITIALIZER_LIST (vala_code_node_get_parent_node ((ValaCodeNode*) self));
 			} else {
 				_tmp3 = FALSE;
 			}
@@ -386,7 +386,7 @@ static gboolean vala_initializer_list_real_check (ValaCodeNode* base, ValaSemant
 				} else {
 					_tmp34 = vala_unary_expression_get_operator (unary) == VALA_UNARY_OPERATOR_OUT;
 				}
-				_tmp33 = (_tmp34);
+				_tmp33 = _tmp34;
 			} else {
 				_tmp33 = FALSE;
 			}
