@@ -1,6 +1,6 @@
 /* valanamespace.vala
  *
- * Copyright (C) 2006-2008  Jürg Billeter
+ * Copyright (C) 2006-2009  Jürg Billeter
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -71,7 +71,7 @@ GeeList* vala_namespace_get_structs (ValaNamespace* self);
 GeeList* vala_namespace_get_classes (ValaNamespace* self);
 GeeList* vala_namespace_get_interfaces (ValaNamespace* self);
 GeeList* vala_namespace_get_enums (ValaNamespace* self);
-GeeList* vala_namespace_get_error_types (ValaNamespace* self);
+GeeList* vala_namespace_get_error_domains (ValaNamespace* self);
 GeeList* vala_namespace_get_fields (ValaNamespace* self);
 GeeList* vala_namespace_get_constants (ValaNamespace* self);
 GeeList* vala_namespace_get_delegates (ValaNamespace* self);
