@@ -1,6 +1,6 @@
 /* valapointertype.vala
  *
- * Copyright (C) 2007-2008  Jürg Billeter
+ * Copyright (C) 2007-2009  Jürg Billeter
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -23,7 +23,6 @@
 #include <vala/valapointertype.h>
 #include <vala/valacodenode.h>
 #include <vala/valatypesymbol.h>
-#include <vala/valasemanticanalyzer.h>
 
 
 
@@ -46,6 +45,7 @@ static char* vala_pointer_type_real_get_type_id (ValaDataType* base);
 static void vala_pointer_type_real_accept_children (ValaCodeNode* base, ValaCodeVisitor* visitor);
 static void vala_pointer_type_real_replace_type (ValaCodeNode* base, ValaDataType* old_type, ValaDataType* new_type);
 static gboolean vala_pointer_type_real_is_disposable (ValaDataType* base);
+static gboolean vala_pointer_type_real_check (ValaCodeNode* base, ValaSemanticAnalyzer* analyzer);
 static gpointer vala_pointer_type_parent_class = NULL;
 static void vala_pointer_type_finalize (ValaCodeNode* obj);
 
@@ -130,7 +130,7 @@ static gboolean vala_pointer_type_real_compatible (ValaDataType* base, ValaDataT
 		} else {
 			_tmp1 = FALSE;
 		}
-		_tmp0 = (_tmp1);
+		_tmp0 = _tmp1;
 	}
 	if (_tmp0) {
 		return TRUE;
@@ -206,6 +206,15 @@ static gboolean vala_pointer_type_real_is_disposable (ValaDataType* base) {
 }
 
 
+static gboolean vala_pointer_type_real_check (ValaCodeNode* base, ValaSemanticAnalyzer* analyzer) {
+	ValaPointerType * self;
+	self = (ValaPointerType*) base;
+	g_return_val_if_fail (analyzer != NULL, FALSE);
+	vala_code_node_set_error ((ValaCodeNode*) self, !vala_code_node_check ((ValaCodeNode*) vala_pointer_type_get_base_type (self), analyzer));
+	return !vala_code_node_get_error ((ValaCodeNode*) self);
+}
+
+
 ValaDataType* vala_pointer_type_get_base_type (ValaPointerType* self) {
 	g_return_val_if_fail (self != NULL, NULL);
 	return self->priv->_base_type;
@@ -237,6 +246,7 @@ static void vala_pointer_type_class_init (ValaPointerTypeClass * klass) {
 	VALA_CODE_NODE_CLASS (klass)->accept_children = vala_pointer_type_real_accept_children;
 	VALA_CODE_NODE_CLASS (klass)->replace_type = vala_pointer_type_real_replace_type;
 	VALA_DATA_TYPE_CLASS (klass)->is_disposable = vala_pointer_type_real_is_disposable;
+	VALA_CODE_NODE_CLASS (klass)->check = vala_pointer_type_real_check;
 }
 
 
