@@ -24,15 +24,18 @@
 #include <gee/arraylist.h>
 #include <gee/collection.h>
 #include <gee/readonlylist.h>
-#include <vala/valacodevisitor.h>
-#include <vala/valadatatype.h>
-#include <vala/valasourcereference.h>
+#include <gee/iterable.h>
+#include <gee/iterator.h>
 #include <vala/valaformalparameter.h>
 #include <vala/valamethod.h>
+#include <vala/valadatatype.h>
+#include <vala/valasourcereference.h>
 #include <vala/valasymbol.h>
 #include <vala/valascope.h>
 #include <vala/valadelegate.h>
+#include <vala/valacodevisitor.h>
 #include <vala/valaattribute.h>
+#include <vala/valasemanticanalyzer.h>
 
 
 
@@ -56,6 +59,7 @@ static void vala_signal_real_accept_children (ValaCodeNode* base, ValaCodeVisito
 static gboolean vala_signal_real_get_lock_used (ValaLockable* base);
 static void vala_signal_real_set_lock_used (ValaLockable* base, gboolean used);
 static void vala_signal_real_replace_type (ValaCodeNode* base, ValaDataType* old_type, ValaDataType* new_type);
+static gboolean vala_signal_real_check (ValaCodeNode* base, ValaSemanticAnalyzer* analyzer);
 static gpointer vala_signal_parent_class = NULL;
 static ValaLockableIface* vala_signal_vala_lockable_parent_iface = NULL;
 static void vala_signal_finalize (ValaCodeNode* obj);
@@ -75,8 +79,7 @@ ValaSignal* vala_signal_construct (GType object_type, const char* name, ValaData
 	ValaSignal* self;
 	g_return_val_if_fail (name != NULL, NULL);
 	g_return_val_if_fail (return_type != NULL, NULL);
-	self = ((ValaSignal*) (g_type_create_instance (object_type)));
-	self = ((ValaSignal*) (vala_member_construct (object_type, name, source_reference)));
+	self = (ValaSignal*) vala_member_construct (object_type, name, source_reference);
 	vala_signal_set_return_type (self, return_type);
 	return self;
 }
@@ -96,17 +99,17 @@ void vala_signal_add_parameter (ValaSignal* self, ValaFormalParameter* param) {
 	g_return_if_fail (self != NULL);
 	g_return_if_fail (param != NULL);
 	/* default C parameter position*/
-	vala_formal_parameter_set_cparameter_position (param, ((double) (gee_collection_get_size (((GeeCollection*) (self->priv->parameters))) + 1)));
+	vala_formal_parameter_set_cparameter_position (param, (double) (gee_collection_get_size ((GeeCollection*) self->priv->parameters) + 1));
 	vala_formal_parameter_set_carray_length_parameter_position (param, vala_formal_parameter_get_cparameter_position (param) + 0.1);
 	vala_formal_parameter_set_cdelegate_target_parameter_position (param, vala_formal_parameter_get_cparameter_position (param) + 0.1);
-	gee_collection_add (((GeeCollection*) (self->priv->parameters)), param);
-	vala_scope_add (vala_symbol_get_scope (((ValaSymbol*) (self))), vala_symbol_get_name (((ValaSymbol*) (param))), ((ValaSymbol*) (param)));
+	gee_collection_add ((GeeCollection*) self->priv->parameters, param);
+	vala_scope_add (vala_symbol_get_scope ((ValaSymbol*) self), vala_symbol_get_name ((ValaSymbol*) param), (ValaSymbol*) param);
 }
 
 
 GeeList* vala_signal_get_parameters (ValaSignal* self) {
 	g_return_val_if_fail (self != NULL, NULL);
-	return ((GeeList*) (gee_read_only_list_new (VALA_TYPE_FORMAL_PARAMETER, ((GBoxedCopyFunc) (vala_code_node_ref)), vala_code_node_unref, self->priv->parameters)));
+	return (GeeList*) gee_read_only_list_new (VALA_TYPE_FORMAL_PARAMETER, (GBoxedCopyFunc) vala_code_node_ref, vala_code_node_unref, self->priv->parameters);
 }
 
 
@@ -115,14 +118,17 @@ GeeList* vala_signal_get_parameters (ValaSignal* self) {
  *
  * @return delegate
  */
-ValaDelegate* vala_signal_get_delegate (ValaSignal* self, ValaDataType* sender_type) {
+ValaDelegate* vala_signal_get_delegate (ValaSignal* self, ValaDataType* sender_type, ValaCodeNode* node_reference) {
+	ValaDataType* actual_return_type;
 	ValaDelegate* generated_delegate;
 	ValaDataType* sender_param_type;
 	ValaFormalParameter* sender_param;
 	ValaDelegate* _tmp1;
 	g_return_val_if_fail (self != NULL, NULL);
 	g_return_val_if_fail (sender_type != NULL, NULL);
-	generated_delegate = vala_delegate_new (NULL, vala_signal_get_return_type (self), NULL);
+	g_return_val_if_fail (node_reference != NULL, NULL);
+	actual_return_type = vala_data_type_get_actual_type (vala_signal_get_return_type (self), sender_type, node_reference);
+	generated_delegate = vala_delegate_new (NULL, actual_return_type, NULL);
 	vala_delegate_set_has_target (generated_delegate, TRUE);
 	/* sender parameter is never null and doesn't own its value*/
 	sender_param_type = vala_data_type_copy (sender_type);
@@ -131,24 +137,26 @@ ValaDelegate* vala_signal_get_delegate (ValaSignal* self, ValaDataType* sender_t
 	sender_param = vala_formal_parameter_new ("_sender", sender_param_type, NULL);
 	vala_delegate_add_parameter (generated_delegate, sender_param);
 	{
-		GeeList* param_collection;
-		int param_it;
-		param_collection = self->priv->parameters;
-		for (param_it = 0; param_it < gee_collection_get_size (GEE_COLLECTION (param_collection)); param_it = param_it + 1) {
+		GeeIterator* _param_it;
+		_param_it = gee_iterable_iterator ((GeeIterable*) self->priv->parameters);
+		while (gee_iterator_next (_param_it)) {
 			ValaFormalParameter* param;
-			param = ((ValaFormalParameter*) (gee_list_get (GEE_LIST (param_collection), param_it)));
-			{
-				ValaFormalParameter* _tmp0;
-				_tmp0 = NULL;
-				vala_delegate_add_parameter (generated_delegate, (_tmp0 = vala_formal_parameter_copy (param)));
-				(_tmp0 == NULL ? NULL : (_tmp0 = (vala_code_node_unref (_tmp0), NULL)));
-				(param == NULL ? NULL : (param = (vala_code_node_unref (param), NULL)));
-			}
+			ValaFormalParameter* actual_param;
+			ValaDataType* _tmp0;
+			param = (ValaFormalParameter*) gee_iterator_get (_param_it);
+			actual_param = vala_formal_parameter_copy (param);
+			_tmp0 = NULL;
+			vala_formal_parameter_set_parameter_type (actual_param, _tmp0 = vala_data_type_get_actual_type (vala_formal_parameter_get_parameter_type (actual_param), sender_type, node_reference));
+			(_tmp0 == NULL) ? NULL : (_tmp0 = (vala_code_node_unref (_tmp0), NULL));
+			vala_delegate_add_parameter (generated_delegate, actual_param);
+			(param == NULL) ? NULL : (param = (vala_code_node_unref (param), NULL));
+			(actual_param == NULL) ? NULL : (actual_param = (vala_code_node_unref (actual_param), NULL));
 		}
+		(_param_it == NULL) ? NULL : (_param_it = (gee_collection_object_unref (_param_it), NULL));
 	}
-	vala_scope_add (vala_symbol_get_scope (((ValaSymbol*) (self))), NULL, ((ValaSymbol*) (generated_delegate)));
+	vala_scope_add (vala_symbol_get_scope ((ValaSymbol*) self), NULL, (ValaSymbol*) generated_delegate);
 	_tmp1 = NULL;
-	return (_tmp1 = generated_delegate, (sender_param_type == NULL ? NULL : (sender_param_type = (vala_code_node_unref (sender_param_type), NULL))), (sender_param == NULL ? NULL : (sender_param = (vala_code_node_unref (sender_param), NULL))), _tmp1);
+	return (_tmp1 = generated_delegate, (actual_return_type == NULL) ? NULL : (actual_return_type = (vala_code_node_unref (actual_return_type), NULL)), (sender_param_type == NULL) ? NULL : (sender_param_type = (vala_code_node_unref (sender_param_type), NULL)), (sender_param == NULL) ? NULL : (sender_param = (vala_code_node_unref (sender_param), NULL)), _tmp1);
 }
 
 
@@ -163,10 +171,10 @@ char* vala_signal_get_cname (ValaSignal* self) {
 	if (self->priv->cname == NULL) {
 		char* _tmp0;
 		_tmp0 = NULL;
-		self->priv->cname = (_tmp0 = vala_symbol_camel_case_to_lower_case (vala_symbol_get_name (((ValaSymbol*) (self)))), (self->priv->cname = (g_free (self->priv->cname), NULL)), _tmp0);
+		self->priv->cname = (_tmp0 = vala_symbol_camel_case_to_lower_case (vala_symbol_get_name ((ValaSymbol*) self)), self->priv->cname = (g_free (self->priv->cname), NULL), _tmp0);
 	}
 	_tmp1 = NULL;
-	return (_tmp1 = self->priv->cname, (_tmp1 == NULL ? NULL : g_strdup (_tmp1)));
+	return (_tmp1 = self->priv->cname, (_tmp1 == NULL) ? NULL : g_strdup (_tmp1));
 }
 
 
@@ -177,7 +185,7 @@ void vala_signal_set_cname (ValaSignal* self, const char* cname) {
 	g_return_if_fail (cname != NULL);
 	_tmp1 = NULL;
 	_tmp0 = NULL;
-	self->priv->cname = (_tmp1 = (_tmp0 = cname, (_tmp0 == NULL ? NULL : g_strdup (_tmp0))), (self->priv->cname = (g_free (self->priv->cname), NULL)), _tmp1);
+	self->priv->cname = (_tmp1 = (_tmp0 = cname, (_tmp0 == NULL) ? NULL : g_strdup (_tmp0)), self->priv->cname = (g_free (self->priv->cname), NULL), _tmp1);
 }
 
 
@@ -205,7 +213,7 @@ ValaCCodeConstant* vala_signal_get_canonical_cconstant (ValaSignal* self, const
 		}
 		_tmp1 = NULL;
 		_tmp0 = NULL;
-		i = (_tmp1 = (_tmp0 = g_utf8_next_char (i), (_tmp0 == NULL ? NULL : g_strdup (_tmp0))), (i = (g_free (i), NULL)), _tmp1);
+		i = (_tmp1 = (_tmp0 = g_utf8_next_char (i), (_tmp0 == NULL) ? NULL : g_strdup (_tmp0)), i = (g_free (i), NULL), _tmp1);
 	}
 	if (detail != NULL) {
 		g_string_append (str, "::");
@@ -213,36 +221,34 @@ ValaCCodeConstant* vala_signal_get_canonical_cconstant (ValaSignal* self, const
 	}
 	g_string_append_c (str, '"');
 	_tmp2 = NULL;
-	return (_tmp2 = vala_ccode_constant_new (str->str), (str == NULL ? NULL : (str = (g_string_free (str, TRUE), NULL))), (i = (g_free (i), NULL)), _tmp2);
+	return (_tmp2 = vala_ccode_constant_new (str->str), (str == NULL) ? NULL : (str = (g_string_free (str, TRUE), NULL)), i = (g_free (i), NULL), _tmp2);
 }
 
 
 static void vala_signal_real_accept (ValaCodeNode* base, ValaCodeVisitor* visitor) {
 	ValaSignal * self;
-	self = ((ValaSignal*) (base));
+	self = (ValaSignal*) base;
 	g_return_if_fail (visitor != NULL);
-	vala_code_visitor_visit_member (visitor, ((ValaMember*) (self)));
+	vala_code_visitor_visit_member (visitor, (ValaMember*) self);
 	vala_code_visitor_visit_signal (visitor, self);
 }
 
 
 static void vala_signal_real_accept_children (ValaCodeNode* base, ValaCodeVisitor* visitor) {
 	ValaSignal * self;
-	self = ((ValaSignal*) (base));
+	self = (ValaSignal*) base;
 	g_return_if_fail (visitor != NULL);
-	vala_code_node_accept (((ValaCodeNode*) (vala_signal_get_return_type (self))), visitor);
+	vala_code_node_accept ((ValaCodeNode*) vala_signal_get_return_type (self), visitor);
 	{
-		GeeList* param_collection;
-		int param_it;
-		param_collection = self->priv->parameters;
-		for (param_it = 0; param_it < gee_collection_get_size (GEE_COLLECTION (param_collection)); param_it = param_it + 1) {
+		GeeIterator* _param_it;
+		_param_it = gee_iterable_iterator ((GeeIterable*) self->priv->parameters);
+		while (gee_iterator_next (_param_it)) {
 			ValaFormalParameter* param;
-			param = ((ValaFormalParameter*) (gee_list_get (GEE_LIST (param_collection), param_it)));
-			{
-				vala_code_node_accept (((ValaCodeNode*) (param)), visitor);
-				(param == NULL ? NULL : (param = (vala_code_node_unref (param), NULL)));
-			}
+			param = (ValaFormalParameter*) gee_iterator_get (_param_it);
+			vala_code_node_accept ((ValaCodeNode*) param, visitor);
+			(param == NULL) ? NULL : (param = (vala_code_node_unref (param), NULL));
 		}
+		(_param_it == NULL) ? NULL : (_param_it = (gee_collection_object_unref (_param_it), NULL));
 	}
 }
 
@@ -255,17 +261,17 @@ void vala_signal_process_attributes (ValaSignal* self) {
 	{
 		GList* a_collection;
 		GList* a_it;
-		a_collection = ((ValaCodeNode*) (self))->attributes;
+		a_collection = ((ValaCodeNode*) self)->attributes;
 		for (a_it = a_collection; a_it != NULL; a_it = a_it->next) {
 			ValaAttribute* _tmp0;
 			ValaAttribute* a;
 			_tmp0 = NULL;
-			a = (_tmp0 = ((ValaAttribute*) (a_it->data)), (_tmp0 == NULL ? NULL : vala_code_node_ref (_tmp0)));
+			a = (_tmp0 = (ValaAttribute*) a_it->data, (_tmp0 == NULL) ? NULL : vala_code_node_ref (_tmp0));
 			{
 				if (_vala_strcmp0 (vala_attribute_get_name (a), "HasEmitter") == 0) {
 					vala_signal_set_has_emitter (self, TRUE);
 				}
-				(a == NULL ? NULL : (a = (vala_code_node_unref (a), NULL)));
+				(a == NULL) ? NULL : (a = (vala_code_node_unref (a), NULL));
 			}
 		}
 	}
@@ -274,21 +280,21 @@ void vala_signal_process_attributes (ValaSignal* self) {
 
 static gboolean vala_signal_real_get_lock_used (ValaLockable* base) {
 	ValaSignal * self;
-	self = ((ValaSignal*) (base));
+	self = (ValaSignal*) base;
 	return self->priv->lock_used;
 }
 
 
 static void vala_signal_real_set_lock_used (ValaLockable* base, gboolean used) {
 	ValaSignal * self;
-	self = ((ValaSignal*) (base));
+	self = (ValaSignal*) base;
 	self->priv->lock_used = used;
 }
 
 
 static void vala_signal_real_replace_type (ValaCodeNode* base, ValaDataType* old_type, ValaDataType* new_type) {
 	ValaSignal * self;
-	self = ((ValaSignal*) (base));
+	self = (ValaSignal*) base;
 	g_return_if_fail (old_type != NULL);
 	g_return_if_fail (new_type != NULL);
 	if (vala_signal_get_return_type (self) == old_type) {
@@ -304,26 +310,49 @@ ValaMethod* vala_signal_get_method_handler (ValaSignal* self) {
 	if (self->priv->generated_method == NULL) {
 		ValaMethod* _tmp0;
 		_tmp0 = NULL;
-		self->priv->generated_method = (_tmp0 = vala_method_new (vala_symbol_get_name (((ValaSymbol*) (self))), vala_signal_get_return_type (self), vala_code_node_get_source_reference (((ValaCodeNode*) (self)))), (self->priv->generated_method == NULL ? NULL : (self->priv->generated_method = (vala_code_node_unref (self->priv->generated_method), NULL))), _tmp0);
+		self->priv->generated_method = (_tmp0 = vala_method_new (vala_symbol_get_name ((ValaSymbol*) self), vala_signal_get_return_type (self), vala_code_node_get_source_reference ((ValaCodeNode*) self)), (self->priv->generated_method == NULL) ? NULL : (self->priv->generated_method = (vala_code_node_unref (self->priv->generated_method), NULL)), _tmp0);
 		vala_method_set_is_virtual (self->priv->generated_method, TRUE);
-		vala_method_set_vfunc_name (self->priv->generated_method, vala_symbol_get_name (((ValaSymbol*) (self))));
+		vala_method_set_vfunc_name (self->priv->generated_method, vala_symbol_get_name ((ValaSymbol*) self));
 		{
-			GeeList* param_collection;
-			int param_it;
-			param_collection = self->priv->parameters;
-			for (param_it = 0; param_it < gee_collection_get_size (GEE_COLLECTION (param_collection)); param_it = param_it + 1) {
+			GeeIterator* _param_it;
+			_param_it = gee_iterable_iterator ((GeeIterable*) self->priv->parameters);
+			while (gee_iterator_next (_param_it)) {
 				ValaFormalParameter* param;
-				param = ((ValaFormalParameter*) (gee_list_get (GEE_LIST (param_collection), param_it)));
-				{
-					vala_method_add_parameter (self->priv->generated_method, param);
-					(param == NULL ? NULL : (param = (vala_code_node_unref (param), NULL)));
-				}
+				param = (ValaFormalParameter*) gee_iterator_get (_param_it);
+				vala_method_add_parameter (self->priv->generated_method, param);
+				(param == NULL) ? NULL : (param = (vala_code_node_unref (param), NULL));
 			}
+			(_param_it == NULL) ? NULL : (_param_it = (gee_collection_object_unref (_param_it), NULL));
 		}
-		vala_scope_add (vala_symbol_get_scope (vala_symbol_get_parent_symbol (((ValaSymbol*) (self)))), NULL, ((ValaSymbol*) (self->priv->generated_method)));
+		vala_scope_add (vala_symbol_get_scope (vala_symbol_get_parent_symbol ((ValaSymbol*) self)), NULL, (ValaSymbol*) self->priv->generated_method);
 	}
 	_tmp1 = NULL;
-	return (_tmp1 = self->priv->generated_method, (_tmp1 == NULL ? NULL : vala_code_node_ref (_tmp1)));
+	return (_tmp1 = self->priv->generated_method, (_tmp1 == NULL) ? NULL : vala_code_node_ref (_tmp1));
+}
+
+
+static gboolean vala_signal_real_check (ValaCodeNode* base, ValaSemanticAnalyzer* analyzer) {
+	ValaSignal * self;
+	self = (ValaSignal*) base;
+	g_return_val_if_fail (analyzer != NULL, FALSE);
+	if (vala_code_node_get_checked ((ValaCodeNode*) self)) {
+		return !vala_code_node_get_error ((ValaCodeNode*) self);
+	}
+	vala_code_node_set_checked ((ValaCodeNode*) self, TRUE);
+	vala_signal_process_attributes (self);
+	vala_code_node_check ((ValaCodeNode*) vala_signal_get_return_type (self), analyzer);
+	{
+		GeeIterator* _param_it;
+		_param_it = gee_iterable_iterator ((GeeIterable*) self->priv->parameters);
+		while (gee_iterator_next (_param_it)) {
+			ValaFormalParameter* param;
+			param = (ValaFormalParameter*) gee_iterator_get (_param_it);
+			vala_code_node_check ((ValaCodeNode*) param, analyzer);
+			(param == NULL) ? NULL : (param = (vala_code_node_unref (param), NULL));
+		}
+		(_param_it == NULL) ? NULL : (_param_it = (gee_collection_object_unref (_param_it), NULL));
+	}
+	return !vala_code_node_get_error ((ValaCodeNode*) self);
 }
 
 
@@ -339,8 +368,8 @@ void vala_signal_set_return_type (ValaSignal* self, ValaDataType* value) {
 	g_return_if_fail (self != NULL);
 	_tmp2 = NULL;
 	_tmp1 = NULL;
-	self->priv->_return_type = (_tmp2 = (_tmp1 = value, (_tmp1 == NULL ? NULL : vala_code_node_ref (_tmp1))), (self->priv->_return_type == NULL ? NULL : (self->priv->_return_type = (vala_code_node_unref (self->priv->_return_type), NULL))), _tmp2);
-	vala_code_node_set_parent_node (((ValaCodeNode*) (self->priv->_return_type)), ((ValaCodeNode*) (self)));
+	self->priv->_return_type = (_tmp2 = (_tmp1 = value, (_tmp1 == NULL) ? NULL : vala_code_node_ref (_tmp1)), (self->priv->_return_type == NULL) ? NULL : (self->priv->_return_type = (vala_code_node_unref (self->priv->_return_type), NULL)), _tmp2);
+	vala_code_node_set_parent_node ((ValaCodeNode*) self->priv->_return_type, (ValaCodeNode*) self);
 }
 
 
@@ -375,6 +404,7 @@ static void vala_signal_class_init (ValaSignalClass * klass) {
 	VALA_CODE_NODE_CLASS (klass)->accept = vala_signal_real_accept;
 	VALA_CODE_NODE_CLASS (klass)->accept_children = vala_signal_real_accept_children;
 	VALA_CODE_NODE_CLASS (klass)->replace_type = vala_signal_real_replace_type;
+	VALA_CODE_NODE_CLASS (klass)->check = vala_signal_real_check;
 }
 
 
@@ -387,7 +417,7 @@ static void vala_signal_vala_lockable_interface_init (ValaLockableIface * iface)
 
 static void vala_signal_instance_init (ValaSignal * self) {
 	self->priv = VALA_SIGNAL_GET_PRIVATE (self);
-	self->priv->parameters = ((GeeList*) (gee_array_list_new (VALA_TYPE_FORMAL_PARAMETER, ((GBoxedCopyFunc) (vala_code_node_ref)), vala_code_node_unref, g_direct_equal)));
+	self->priv->parameters = (GeeList*) gee_array_list_new (VALA_TYPE_FORMAL_PARAMETER, (GBoxedCopyFunc) vala_code_node_ref, vala_code_node_unref, g_direct_equal);
 	self->priv->lock_used = FALSE;
 }
 
@@ -395,10 +425,10 @@ static void vala_signal_instance_init (ValaSignal * self) {
 static void vala_signal_finalize (ValaCodeNode* obj) {
 	ValaSignal * self;
 	self = VALA_SIGNAL (obj);
-	(self->priv->parameters == NULL ? NULL : (self->priv->parameters = (gee_collection_object_unref (self->priv->parameters), NULL)));
-	(self->priv->generated_method == NULL ? NULL : (self->priv->generated_method = (vala_code_node_unref (self->priv->generated_method), NULL)));
+	(self->priv->parameters == NULL) ? NULL : (self->priv->parameters = (gee_collection_object_unref (self->priv->parameters), NULL));
+	(self->priv->generated_method == NULL) ? NULL : (self->priv->generated_method = (vala_code_node_unref (self->priv->generated_method), NULL));
 	self->priv->cname = (g_free (self->priv->cname), NULL);
-	(self->priv->_return_type == NULL ? NULL : (self->priv->_return_type = (vala_code_node_unref (self->priv->_return_type), NULL)));
+	(self->priv->_return_type == NULL) ? NULL : (self->priv->_return_type = (vala_code_node_unref (self->priv->_return_type), NULL));
 	VALA_CODE_NODE_CLASS (vala_signal_parent_class)->finalize (obj);
 }
 
@@ -420,7 +450,7 @@ static int _vala_strcmp0 (const char * str1, const char * str2) {
 		return -(str1 != str2);
 	}
 	if (str2 == NULL) {
-		return (str1 != str2);
+		return str1 != str2;
 	}
 	return strcmp (str1, str2);
 }
