@@ -25,6 +25,7 @@
 #include <vala/valasourcereference.h>
 #include <vala/valacodevisitor.h>
 #include <vala/valasemanticanalyzer.h>
+#include <vala/valasourcefile.h>
 
 
 
@@ -107,6 +108,7 @@ static gboolean vala_sizeof_expression_real_check (ValaCodeNode* base, ValaSeman
 	vala_code_node_set_checked ((ValaCodeNode*) self, TRUE);
 	vala_code_node_check ((ValaCodeNode*) vala_sizeof_expression_get_type_reference (self), analyzer);
 	vala_expression_set_value_type ((ValaExpression*) self, analyzer->ulong_type);
+	vala_source_file_add_type_dependency (vala_semantic_analyzer_get_current_source_file (analyzer), vala_sizeof_expression_get_type_reference (self), VALA_SOURCE_FILE_DEPENDENCY_TYPE_SOURCE);
 	return !vala_code_node_get_error ((ValaCodeNode*) self);
 }
 
