@@ -25,7 +25,7 @@
 
 #include <glib.h>
 #include <glib-object.h>
-#include <gee/collection.h>
+#include <gee/list.h>
 #include <vala/valacodenode.h>
 #include <vala/valastatement.h>
 
@@ -48,13 +48,16 @@ struct _ValaSwitchStatement {
 	ValaCodeNode parent_instance;
 	ValaSwitchStatementPrivate * priv;
 };
+
 struct _ValaSwitchStatementClass {
 	ValaCodeNodeClass parent_class;
 };
 
+
+ValaSwitchStatement* vala_switch_statement_construct (GType object_type, ValaExpression* expression, ValaSourceReference* source_reference);
 ValaSwitchStatement* vala_switch_statement_new (ValaExpression* expression, ValaSourceReference* source_reference);
 void vala_switch_statement_add_section (ValaSwitchStatement* self, ValaSwitchSection* section);
-GeeCollection* vala_switch_statement_get_sections (ValaSwitchStatement* self);
+GeeList* vala_switch_statement_get_sections (ValaSwitchStatement* self);
 ValaExpression* vala_switch_statement_get_expression (ValaSwitchStatement* self);
 void vala_switch_statement_set_expression (ValaSwitchStatement* self, ValaExpression* value);
 GType vala_switch_statement_get_type (void);
