@@ -91,6 +91,36 @@ gboolean vala_symbol_is_internal_symbol (ValaSymbol* self) {
 }
 
 
+gboolean vala_symbol_is_library_internal_symbol (ValaSymbol* self) {
+	g_return_val_if_fail (self != NULL, FALSE);
+	{
+		ValaSymbol* _tmp0;
+		ValaSymbol* sym;
+		ValaSymbol* _tmp2;
+		ValaSymbol* _tmp1;
+		_tmp0 = NULL;
+		sym = (_tmp0 = self, (_tmp0 == NULL) ? NULL : vala_code_node_ref (_tmp0));
+		_tmp2 = NULL;
+		_tmp1 = NULL;
+		for (; NULL != sym; sym = (_tmp2 = (_tmp1 = vala_symbol_get_parent_symbol (sym), (_tmp1 == NULL) ? NULL : vala_code_node_ref (_tmp1)), (sym == NULL) ? NULL : (sym = (vala_code_node_unref (sym), NULL)), _tmp2)) {
+			gboolean _tmp3;
+			_tmp3 = FALSE;
+			if (sym->priv->_access == VALA_SYMBOL_ACCESSIBILITY_PRIVATE) {
+				_tmp3 = TRUE;
+			} else {
+				_tmp3 = sym->priv->_access == VALA_SYMBOL_ACCESSIBILITY_INTERNAL;
+			}
+			if (_tmp3) {
+				gboolean _tmp4;
+				return (_tmp4 = TRUE, (sym == NULL) ? NULL : (sym = (vala_code_node_unref (sym), NULL)), _tmp4);
+			}
+		}
+		(sym == NULL) ? NULL : (sym = (vala_code_node_unref (sym), NULL));
+	}
+	return FALSE;
+}
+
+
 ValaSymbol* vala_symbol_construct (GType object_type, const char* name, ValaSourceReference* source_reference) {
 	ValaSymbol* self;
 	ValaScope* _tmp0;
@@ -265,7 +295,7 @@ char* vala_symbol_camel_case_to_lower_case (const char* camel_case) {
 				} else {
 					_tmp2 = FALSE;
 				}
-				_tmp1 = (_tmp2);
+				_tmp1 = _tmp2;
 			}
 			if (_tmp1) {
 				glong len;
@@ -473,7 +503,7 @@ gboolean vala_symbol_get_external_package (ValaSymbol* self) {
 	} else {
 		_tmp0 = FALSE;
 	}
-	return (_tmp0);
+	return _tmp0;
 }
 
 
