@@ -1,6 +1,6 @@
 /* valaunaryexpression.vala
  *
- * Copyright (C) 2006-2008  Jürg Billeter
+ * Copyright (C) 2006-2009  Jürg Billeter
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -26,9 +26,10 @@
 #include <vala/valadatatype.h>
 #include <vala/valastruct.h>
 #include <vala/valamemberaccess.h>
-#include <vala/valaparenthesizedexpression.h>
 #include <vala/valasemanticanalyzer.h>
+#include <vala/valafieldprototype.h>
 #include <vala/valareport.h>
+#include <vala/valasymbol.h>
 #include <vala/valabinaryexpression.h>
 #include <vala/valaintegerliteral.h>
 #include <vala/valaassignment.h>
@@ -195,7 +196,7 @@ static gboolean vala_unary_expression_is_numeric_type (ValaUnaryExpression* self
 	gboolean _tmp3;
 	g_return_val_if_fail (self != NULL, FALSE);
 	g_return_val_if_fail (type != NULL, FALSE);
-	if (!(VALA_IS_STRUCT (vala_data_type_get_data_type (type)))) {
+	if (!VALA_IS_STRUCT (vala_data_type_get_data_type (type))) {
 		return FALSE;
 	}
 	_tmp1 = NULL;
@@ -216,7 +217,7 @@ static gboolean vala_unary_expression_is_integer_type (ValaUnaryExpression* self
 	gboolean _tmp2;
 	g_return_val_if_fail (self != NULL, FALSE);
 	g_return_val_if_fail (type != NULL, FALSE);
-	if (!(VALA_IS_STRUCT (vala_data_type_get_data_type (type)))) {
+	if (!VALA_IS_STRUCT (vala_data_type_get_data_type (type))) {
 		return FALSE;
 	}
 	_tmp1 = NULL;
@@ -228,19 +229,10 @@ static gboolean vala_unary_expression_is_integer_type (ValaUnaryExpression* self
 static ValaMemberAccess* vala_unary_expression_find_member_access (ValaUnaryExpression* self, ValaExpression* expr) {
 	g_return_val_if_fail (self != NULL, NULL);
 	g_return_val_if_fail (expr != NULL, NULL);
-	if (VALA_IS_PARENTHESIZED_EXPRESSION (expr)) {
-		ValaParenthesizedExpression* _tmp0;
-		ValaParenthesizedExpression* pe;
-		ValaMemberAccess* _tmp1;
-		_tmp0 = NULL;
-		pe = (_tmp0 = VALA_PARENTHESIZED_EXPRESSION (expr), (_tmp0 == NULL) ? NULL : vala_code_node_ref (_tmp0));
-		_tmp1 = NULL;
-		return (_tmp1 = vala_unary_expression_find_member_access (self, vala_parenthesized_expression_get_inner (pe)), (pe == NULL) ? NULL : (pe = (vala_code_node_unref (pe), NULL)), _tmp1);
-	}
 	if (VALA_IS_MEMBER_ACCESS (expr)) {
-		ValaMemberAccess* _tmp2;
-		_tmp2 = NULL;
-		return (_tmp2 = VALA_MEMBER_ACCESS (expr), (_tmp2 == NULL) ? NULL : vala_code_node_ref (_tmp2));
+		ValaMemberAccess* _tmp0;
+		_tmp0 = NULL;
+		return (_tmp0 = VALA_MEMBER_ACCESS (expr), (_tmp0 == NULL) ? NULL : vala_code_node_ref (_tmp0));
 	}
 	return NULL;
 }
@@ -249,7 +241,7 @@ static ValaMemberAccess* vala_unary_expression_find_member_access (ValaUnaryExpr
 static gboolean vala_unary_expression_real_check (ValaCodeNode* base, ValaSemanticAnalyzer* analyzer) {
 	ValaUnaryExpression * self;
 	gboolean _tmp1;
-	gboolean _tmp3;
+	gboolean _tmp6;
 	self = (ValaUnaryExpression*) base;
 	g_return_val_if_fail (analyzer != NULL, FALSE);
 	if (vala_code_node_get_checked ((ValaCodeNode*) self)) {
@@ -271,23 +263,34 @@ static gboolean vala_unary_expression_real_check (ValaCodeNode* base, ValaSemant
 		vala_code_node_set_error ((ValaCodeNode*) self, TRUE);
 		return FALSE;
 	}
-	_tmp3 = FALSE;
-	if (self->priv->_operator == VALA_UNARY_OPERATOR_PLUS) {
-		_tmp3 = TRUE;
-	} else {
-		_tmp3 = self->priv->_operator == VALA_UNARY_OPERATOR_MINUS;
+	if (VALA_IS_FIELD_PROTOTYPE (vala_expression_get_value_type (vala_unary_expression_get_inner (self)))) {
+		char* _tmp4;
+		char* _tmp3;
+		vala_code_node_set_error ((ValaCodeNode*) self, TRUE);
+		_tmp4 = NULL;
+		_tmp3 = NULL;
+		vala_report_error (vala_code_node_get_source_reference ((ValaCodeNode*) vala_unary_expression_get_inner (self)), _tmp4 = g_strdup_printf ("Access to instance member `%s' denied", _tmp3 = vala_symbol_get_full_name (vala_expression_get_symbol_reference (vala_unary_expression_get_inner (self)))));
+		_tmp4 = (g_free (_tmp4), NULL);
+		_tmp3 = (g_free (_tmp3), NULL);
+		return FALSE;
 	}
-	if (_tmp3) {
+	_tmp6 = FALSE;
+	if (self->priv->_operator == VALA_UNARY_OPERATOR_PLUS) {
+		_tmp6 = TRUE;
+	} else {
+		_tmp6 = self->priv->_operator == VALA_UNARY_OPERATOR_MINUS;
+	}
+	if (_tmp6) {
 		/* integer or floating point type*/
 		if (!vala_unary_expression_is_numeric_type (self, vala_expression_get_value_type (vala_unary_expression_get_inner (self)))) {
-			char* _tmp5;
-			char* _tmp4;
+			char* _tmp8;
+			char* _tmp7;
 			vala_code_node_set_error ((ValaCodeNode*) self, TRUE);
-			_tmp5 = NULL;
-			_tmp4 = NULL;
-			vala_report_error (vala_code_node_get_source_reference ((ValaCodeNode*) self), _tmp5 = g_strdup_printf ("Operator not supported for `%s'", _tmp4 = vala_code_node_to_string ((ValaCodeNode*) vala_expression_get_value_type (vala_unary_expression_get_inner (self)))));
-			_tmp5 = (g_free (_tmp5), NULL);
-			_tmp4 = (g_free (_tmp4), NULL);
+			_tmp8 = NULL;
+			_tmp7 = NULL;
+			vala_report_error (vala_code_node_get_source_reference ((ValaCodeNode*) self), _tmp8 = g_strdup_printf ("Operator not supported for `%s'", _tmp7 = vala_code_node_to_string ((ValaCodeNode*) vala_expression_get_value_type (vala_unary_expression_get_inner (self)))));
+			_tmp8 = (g_free (_tmp8), NULL);
+			_tmp7 = (g_free (_tmp7), NULL);
 			return FALSE;
 		}
 		vala_expression_set_value_type ((ValaExpression*) self, vala_expression_get_value_type (vala_unary_expression_get_inner (self)));
@@ -295,14 +298,14 @@ static gboolean vala_unary_expression_real_check (ValaCodeNode* base, ValaSemant
 		if (self->priv->_operator == VALA_UNARY_OPERATOR_LOGICAL_NEGATION) {
 			/* boolean type*/
 			if (!vala_data_type_compatible (vala_expression_get_value_type (vala_unary_expression_get_inner (self)), analyzer->bool_type)) {
-				char* _tmp8;
-				char* _tmp7;
+				char* _tmp11;
+				char* _tmp10;
 				vala_code_node_set_error ((ValaCodeNode*) self, TRUE);
-				_tmp8 = NULL;
-				_tmp7 = NULL;
-				vala_report_error (vala_code_node_get_source_reference ((ValaCodeNode*) self), _tmp8 = g_strdup_printf ("Operator not supported for `%s'", _tmp7 = vala_code_node_to_string ((ValaCodeNode*) vala_expression_get_value_type (vala_unary_expression_get_inner (self)))));
-				_tmp8 = (g_free (_tmp8), NULL);
-				_tmp7 = (g_free (_tmp7), NULL);
+				_tmp11 = NULL;
+				_tmp10 = NULL;
+				vala_report_error (vala_code_node_get_source_reference ((ValaCodeNode*) self), _tmp11 = g_strdup_printf ("Operator not supported for `%s'", _tmp10 = vala_code_node_to_string ((ValaCodeNode*) vala_expression_get_value_type (vala_unary_expression_get_inner (self)))));
+				_tmp11 = (g_free (_tmp11), NULL);
+				_tmp10 = (g_free (_tmp10), NULL);
 				return FALSE;
 			}
 			vala_expression_set_value_type ((ValaExpression*) self, vala_expression_get_value_type (vala_unary_expression_get_inner (self)));
@@ -310,95 +313,93 @@ static gboolean vala_unary_expression_real_check (ValaCodeNode* base, ValaSemant
 			if (self->priv->_operator == VALA_UNARY_OPERATOR_BITWISE_COMPLEMENT) {
 				/* integer type*/
 				if (!vala_unary_expression_is_integer_type (self, vala_expression_get_value_type (vala_unary_expression_get_inner (self)))) {
-					char* _tmp11;
-					char* _tmp10;
+					char* _tmp14;
+					char* _tmp13;
 					vala_code_node_set_error ((ValaCodeNode*) self, TRUE);
-					_tmp11 = NULL;
-					_tmp10 = NULL;
-					vala_report_error (vala_code_node_get_source_reference ((ValaCodeNode*) self), _tmp11 = g_strdup_printf ("Operator not supported for `%s'", _tmp10 = vala_code_node_to_string ((ValaCodeNode*) vala_expression_get_value_type (vala_unary_expression_get_inner (self)))));
-					_tmp11 = (g_free (_tmp11), NULL);
-					_tmp10 = (g_free (_tmp10), NULL);
+					_tmp14 = NULL;
+					_tmp13 = NULL;
+					vala_report_error (vala_code_node_get_source_reference ((ValaCodeNode*) self), _tmp14 = g_strdup_printf ("Operator not supported for `%s'", _tmp13 = vala_code_node_to_string ((ValaCodeNode*) vala_expression_get_value_type (vala_unary_expression_get_inner (self)))));
+					_tmp14 = (g_free (_tmp14), NULL);
+					_tmp13 = (g_free (_tmp13), NULL);
 					return FALSE;
 				}
 				vala_expression_set_value_type ((ValaExpression*) self, vala_expression_get_value_type (vala_unary_expression_get_inner (self)));
 			} else {
-				gboolean _tmp13;
-				_tmp13 = FALSE;
+				gboolean _tmp16;
+				_tmp16 = FALSE;
 				if (self->priv->_operator == VALA_UNARY_OPERATOR_INCREMENT) {
-					_tmp13 = TRUE;
+					_tmp16 = TRUE;
 				} else {
-					_tmp13 = self->priv->_operator == VALA_UNARY_OPERATOR_DECREMENT;
+					_tmp16 = self->priv->_operator == VALA_UNARY_OPERATOR_DECREMENT;
 				}
-				if (_tmp13) {
+				if (_tmp16) {
 					ValaMemberAccess* ma;
 					ValaMemberAccess* old_value;
-					ValaBinaryOperator _tmp18;
-					ValaIntegerLiteral* _tmp19;
-					ValaBinaryExpression* _tmp20;
+					ValaBinaryOperator _tmp21;
+					ValaIntegerLiteral* _tmp22;
+					ValaBinaryExpression* _tmp23;
 					ValaBinaryExpression* bin;
 					ValaAssignment* assignment;
-					ValaParenthesizedExpression* parenthexp;
-					gboolean _tmp21;
+					gboolean _tmp24;
 					/* integer type*/
 					if (!vala_unary_expression_is_integer_type (self, vala_expression_get_value_type (vala_unary_expression_get_inner (self)))) {
-						char* _tmp15;
-						char* _tmp14;
+						char* _tmp18;
+						char* _tmp17;
 						vala_code_node_set_error ((ValaCodeNode*) self, TRUE);
-						_tmp15 = NULL;
-						_tmp14 = NULL;
-						vala_report_error (vala_code_node_get_source_reference ((ValaCodeNode*) self), _tmp15 = g_strdup_printf ("Operator not supported for `%s'", _tmp14 = vala_code_node_to_string ((ValaCodeNode*) vala_expression_get_value_type (vala_unary_expression_get_inner (self)))));
-						_tmp15 = (g_free (_tmp15), NULL);
-						_tmp14 = (g_free (_tmp14), NULL);
+						_tmp18 = NULL;
+						_tmp17 = NULL;
+						vala_report_error (vala_code_node_get_source_reference ((ValaCodeNode*) self), _tmp18 = g_strdup_printf ("Operator not supported for `%s'", _tmp17 = vala_code_node_to_string ((ValaCodeNode*) vala_expression_get_value_type (vala_unary_expression_get_inner (self)))));
+						_tmp18 = (g_free (_tmp18), NULL);
+						_tmp17 = (g_free (_tmp17), NULL);
 						return FALSE;
 					}
 					ma = vala_unary_expression_find_member_access (self, vala_unary_expression_get_inner (self));
 					if (ma == NULL) {
-						gboolean _tmp17;
+						gboolean _tmp20;
 						vala_code_node_set_error ((ValaCodeNode*) self, TRUE);
 						vala_report_error (vala_code_node_get_source_reference ((ValaCodeNode*) self), "Prefix operators not supported for this expression");
-						return (_tmp17 = FALSE, (ma == NULL) ? NULL : (ma = (vala_code_node_unref (ma), NULL)), _tmp17);
+						return (_tmp20 = FALSE, (ma == NULL) ? NULL : (ma = (vala_code_node_unref (ma), NULL)), _tmp20);
 					}
 					old_value = vala_member_access_new (vala_member_access_get_inner (ma), vala_member_access_get_member_name (ma), vala_code_node_get_source_reference ((ValaCodeNode*) vala_unary_expression_get_inner (self)));
-					_tmp18 = 0;
+					_tmp21 = 0;
 					if (self->priv->_operator == VALA_UNARY_OPERATOR_INCREMENT) {
-						_tmp18 = VALA_BINARY_OPERATOR_PLUS;
+						_tmp21 = VALA_BINARY_OPERATOR_PLUS;
 					} else {
-						_tmp18 = VALA_BINARY_OPERATOR_MINUS;
+						_tmp21 = VALA_BINARY_OPERATOR_MINUS;
 					}
-					_tmp19 = NULL;
-					_tmp20 = NULL;
-					bin = (_tmp20 = vala_binary_expression_new (_tmp18, (ValaExpression*) old_value, (ValaExpression*) (_tmp19 = vala_integer_literal_new ("1", NULL)), vala_code_node_get_source_reference ((ValaCodeNode*) self)), (_tmp19 == NULL) ? NULL : (_tmp19 = (vala_code_node_unref (_tmp19), NULL)), _tmp20);
+					_tmp22 = NULL;
+					_tmp23 = NULL;
+					bin = (_tmp23 = vala_binary_expression_new (_tmp21, (ValaExpression*) old_value, (ValaExpression*) (_tmp22 = vala_integer_literal_new ("1", NULL)), vala_code_node_get_source_reference ((ValaCodeNode*) self)), (_tmp22 == NULL) ? NULL : (_tmp22 = (vala_code_node_unref (_tmp22), NULL)), _tmp23);
 					assignment = vala_assignment_new ((ValaExpression*) ma, (ValaExpression*) bin, VALA_ASSIGNMENT_OPERATOR_SIMPLE, vala_code_node_get_source_reference ((ValaCodeNode*) self));
-					parenthexp = vala_parenthesized_expression_new ((ValaExpression*) assignment, vala_code_node_get_source_reference ((ValaCodeNode*) self));
-					vala_expression_set_target_type ((ValaExpression*) parenthexp, vala_expression_get_target_type ((ValaExpression*) self));
+					vala_expression_set_target_type ((ValaExpression*) assignment, vala_expression_get_target_type ((ValaExpression*) self));
 					gee_collection_add ((GeeCollection*) analyzer->replaced_nodes, (ValaCodeNode*) self);
-					vala_code_node_replace_expression (vala_code_node_get_parent_node ((ValaCodeNode*) self), (ValaExpression*) self, (ValaExpression*) parenthexp);
-					vala_code_node_check ((ValaCodeNode*) parenthexp, analyzer);
-					return (_tmp21 = TRUE, (ma == NULL) ? NULL : (ma = (vala_code_node_unref (ma), NULL)), (old_value == NULL) ? NULL : (old_value = (vala_code_node_unref (old_value), NULL)), (bin == NULL) ? NULL : (bin = (vala_code_node_unref (bin), NULL)), (assignment == NULL) ? NULL : (assignment = (vala_code_node_unref (assignment), NULL)), (parenthexp == NULL) ? NULL : (parenthexp = (vala_code_node_unref (parenthexp), NULL)), _tmp21);
+					vala_code_node_replace_expression (vala_code_node_get_parent_node ((ValaCodeNode*) self), (ValaExpression*) self, (ValaExpression*) assignment);
+					vala_code_node_check ((ValaCodeNode*) assignment, analyzer);
+					return (_tmp24 = TRUE, (ma == NULL) ? NULL : (ma = (vala_code_node_unref (ma), NULL)), (old_value == NULL) ? NULL : (old_value = (vala_code_node_unref (old_value), NULL)), (bin == NULL) ? NULL : (bin = (vala_code_node_unref (bin), NULL)), (assignment == NULL) ? NULL : (assignment = (vala_code_node_unref (assignment), NULL)), _tmp24);
 				} else {
-					gboolean _tmp22;
-					_tmp22 = FALSE;
+					gboolean _tmp25;
+					_tmp25 = FALSE;
 					if (self->priv->_operator == VALA_UNARY_OPERATOR_REF) {
-						_tmp22 = TRUE;
+						_tmp25 = TRUE;
 					} else {
-						_tmp22 = self->priv->_operator == VALA_UNARY_OPERATOR_OUT;
+						_tmp25 = self->priv->_operator == VALA_UNARY_OPERATOR_OUT;
 					}
-					if (_tmp22) {
-						gboolean _tmp23;
-						gboolean _tmp24;
-						_tmp23 = FALSE;
-						_tmp24 = FALSE;
+					if (_tmp25) {
+						gboolean _tmp26;
+						gboolean _tmp27;
+						_tmp26 = FALSE;
+						_tmp27 = FALSE;
 						if (VALA_IS_FIELD (vala_expression_get_symbol_reference (vala_unary_expression_get_inner (self)))) {
-							_tmp24 = TRUE;
+							_tmp27 = TRUE;
 						} else {
-							_tmp24 = VALA_IS_FORMAL_PARAMETER (vala_expression_get_symbol_reference (vala_unary_expression_get_inner (self)));
+							_tmp27 = VALA_IS_FORMAL_PARAMETER (vala_expression_get_symbol_reference (vala_unary_expression_get_inner (self)));
 						}
-						if (_tmp24) {
-							_tmp23 = TRUE;
+						if (_tmp27) {
+							_tmp26 = TRUE;
 						} else {
-							_tmp23 = VALA_IS_LOCAL_VARIABLE (vala_expression_get_symbol_reference (vala_unary_expression_get_inner (self)));
+							_tmp26 = VALA_IS_LOCAL_VARIABLE (vala_expression_get_symbol_reference (vala_unary_expression_get_inner (self)));
 						}
-						if (_tmp23) {
+						if (_tmp26) {
 							/* ref and out can only be used with fields, parameters, and local variables*/
 							vala_expression_set_lvalue ((ValaExpression*) self, TRUE);
 							vala_expression_set_value_type ((ValaExpression*) self, vala_expression_get_value_type (vala_unary_expression_get_inner (self)));
