@@ -44,10 +44,10 @@ static void vala_unresolved_symbol_finalize (ValaCodeNode* obj);
 ValaUnresolvedSymbol* vala_unresolved_symbol_construct (GType object_type, ValaUnresolvedSymbol* inner, const char* name, ValaSourceReference* source_reference) {
 	ValaUnresolvedSymbol* self;
 	g_return_val_if_fail (name != NULL, NULL);
-	self = ((ValaUnresolvedSymbol*) (g_type_create_instance (object_type)));
+	self = (ValaUnresolvedSymbol*) g_type_create_instance (object_type);
 	vala_unresolved_symbol_set_inner (self, inner);
-	vala_symbol_set_name (((ValaSymbol*) (self)), name);
-	vala_code_node_set_source_reference (((ValaCodeNode*) (self)), source_reference);
+	vala_symbol_set_name ((ValaSymbol*) self, name);
+	vala_code_node_set_source_reference ((ValaCodeNode*) self, source_reference);
 	return self;
 }
 
@@ -59,24 +59,24 @@ ValaUnresolvedSymbol* vala_unresolved_symbol_new (ValaUnresolvedSymbol* inner, c
 
 static char* vala_unresolved_symbol_real_to_string (ValaCodeNode* base) {
 	ValaUnresolvedSymbol * self;
-	self = ((ValaUnresolvedSymbol*) (base));
+	self = (ValaUnresolvedSymbol*) base;
 	if (self->priv->_inner == NULL) {
 		const char* _tmp0;
 		_tmp0 = NULL;
-		return (_tmp0 = vala_symbol_get_name (((ValaSymbol*) (self))), (_tmp0 == NULL ? NULL : g_strdup (_tmp0)));
+		return (_tmp0 = vala_symbol_get_name ((ValaSymbol*) self), (_tmp0 == NULL) ? NULL : g_strdup (_tmp0));
 	} else {
 		char* _tmp2;
 		char* _tmp3;
 		_tmp2 = NULL;
 		_tmp3 = NULL;
-		return (_tmp3 = g_strdup_printf ("%s.%s", (_tmp2 = vala_code_node_to_string (((ValaCodeNode*) (self->priv->_inner)))), vala_symbol_get_name (((ValaSymbol*) (self)))), (_tmp2 = (g_free (_tmp2), NULL)), _tmp3);
+		return (_tmp3 = g_strdup_printf ("%s.%s", _tmp2 = vala_code_node_to_string ((ValaCodeNode*) self->priv->_inner), vala_symbol_get_name ((ValaSymbol*) self)), _tmp2 = (g_free (_tmp2), NULL), _tmp3);
 	}
 }
 
 
 ValaUnresolvedSymbol* vala_unresolved_symbol_copy (ValaUnresolvedSymbol* self) {
 	g_return_val_if_fail (self != NULL, NULL);
-	return vala_unresolved_symbol_new (self->priv->_inner, vala_symbol_get_name (((ValaSymbol*) (self))), vala_code_node_get_source_reference (((ValaCodeNode*) (self))));
+	return vala_unresolved_symbol_new (self->priv->_inner, vala_symbol_get_name ((ValaSymbol*) self), vala_code_node_get_source_reference ((ValaCodeNode*) self));
 }
 
 
@@ -92,7 +92,7 @@ void vala_unresolved_symbol_set_inner (ValaUnresolvedSymbol* self, ValaUnresolve
 	g_return_if_fail (self != NULL);
 	_tmp2 = NULL;
 	_tmp1 = NULL;
-	self->priv->_inner = (_tmp2 = (_tmp1 = value, (_tmp1 == NULL ? NULL : vala_code_node_ref (_tmp1))), (self->priv->_inner == NULL ? NULL : (self->priv->_inner = (vala_code_node_unref (self->priv->_inner), NULL))), _tmp2);
+	self->priv->_inner = (_tmp2 = (_tmp1 = value, (_tmp1 == NULL) ? NULL : vala_code_node_ref (_tmp1)), (self->priv->_inner == NULL) ? NULL : (self->priv->_inner = (vala_code_node_unref (self->priv->_inner), NULL)), _tmp2);
 }
 
 
@@ -124,7 +124,7 @@ static void vala_unresolved_symbol_instance_init (ValaUnresolvedSymbol * self) {
 static void vala_unresolved_symbol_finalize (ValaCodeNode* obj) {
 	ValaUnresolvedSymbol * self;
 	self = VALA_UNRESOLVED_SYMBOL (obj);
-	(self->priv->_inner == NULL ? NULL : (self->priv->_inner = (vala_code_node_unref (self->priv->_inner), NULL)));
+	(self->priv->_inner == NULL) ? NULL : (self->priv->_inner = (vala_code_node_unref (self->priv->_inner), NULL));
 	VALA_CODE_NODE_CLASS (vala_unresolved_symbol_parent_class)->finalize (obj);
 }
 
