@@ -1,6 +1,6 @@
 /* valavaluetype.vala
  *
- * Copyright (C) 2007-2008  Jürg Billeter
+ * Copyright (C) 2007-2009  Jürg Billeter
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -21,10 +21,8 @@
  */
 
 #include <vala/valavaluetype.h>
-#include <gee/iterable.h>
-#include <gee/iterator.h>
-#include <vala/valacodenode.h>
 #include <vala/valastruct.h>
+#include <vala/valacodenode.h>
 
 
 
@@ -37,7 +35,6 @@ struct _ValaValueTypePrivate {
 enum  {
 	VALA_VALUE_TYPE_DUMMY_PROPERTY
 };
-static ValaDataType* vala_value_type_real_copy (ValaDataType* base);
 static char* vala_value_type_real_get_cname (ValaDataType* base);
 static gboolean vala_value_type_real_is_disposable (ValaDataType* base);
 static gboolean vala_value_type_real_check (ValaCodeNode* base, ValaSemanticAnalyzer* analyzer);
@@ -61,38 +58,6 @@ ValaValueType* vala_value_type_new (ValaTypeSymbol* type_symbol) {
 }
 
 
-static ValaDataType* vala_value_type_real_copy (ValaDataType* base) {
-	ValaValueType * self;
-	ValaValueType* result;
-	self = (ValaValueType*) base;
-	result = vala_value_type_new (self->priv->_type_symbol);
-	vala_code_node_set_source_reference ((ValaCodeNode*) result, vala_code_node_get_source_reference ((ValaCodeNode*) self));
-	vala_data_type_set_value_owned ((ValaDataType*) result, vala_data_type_get_value_owned ((ValaDataType*) self));
-	vala_data_type_set_nullable ((ValaDataType*) result, vala_data_type_get_nullable ((ValaDataType*) self));
-	vala_data_type_set_is_dynamic ((ValaDataType*) result, vala_data_type_get_is_dynamic ((ValaDataType*) self));
-	vala_data_type_set_floating_reference ((ValaDataType*) result, vala_data_type_get_floating_reference ((ValaDataType*) self));
-	{
-		GeeList* _tmp0;
-		GeeIterator* _tmp1;
-		GeeIterator* _arg_it;
-		_tmp0 = NULL;
-		_tmp1 = NULL;
-		_arg_it = (_tmp1 = gee_iterable_iterator ((GeeIterable*) (_tmp0 = vala_data_type_get_type_arguments ((ValaDataType*) self))), (_tmp0 == NULL) ? NULL : (_tmp0 = (gee_collection_object_unref (_tmp0), NULL)), _tmp1);
-		while (gee_iterator_next (_arg_it)) {
-			ValaDataType* arg;
-			ValaDataType* _tmp2;
-			arg = (ValaDataType*) gee_iterator_get (_arg_it);
-			_tmp2 = NULL;
-			vala_data_type_add_type_argument ((ValaDataType*) result, _tmp2 = vala_data_type_copy (arg));
-			(_tmp2 == NULL) ? NULL : (_tmp2 = (vala_code_node_unref (_tmp2), NULL));
-			(arg == NULL) ? NULL : (arg = (vala_code_node_unref (arg), NULL));
-		}
-		(_arg_it == NULL) ? NULL : (_arg_it = (gee_collection_object_unref (_arg_it), NULL));
-	}
-	return (ValaDataType*) result;
-}
-
-
 static char* vala_value_type_real_get_cname (ValaDataType* base) {
 	ValaValueType * self;
 	char* ptr;
@@ -162,7 +127,6 @@ static void vala_value_type_class_init (ValaValueTypeClass * klass) {
 	vala_value_type_parent_class = g_type_class_peek_parent (klass);
 	VALA_CODE_NODE_CLASS (klass)->finalize = vala_value_type_finalize;
 	g_type_class_add_private (klass, sizeof (ValaValueTypePrivate));
-	VALA_DATA_TYPE_CLASS (klass)->copy = vala_value_type_real_copy;
 	VALA_DATA_TYPE_CLASS (klass)->get_cname = vala_value_type_real_get_cname;
 	VALA_DATA_TYPE_CLASS (klass)->is_disposable = vala_value_type_real_is_disposable;
 	VALA_CODE_NODE_CLASS (klass)->check = vala_value_type_real_check;
@@ -185,7 +149,7 @@ GType vala_value_type_get_type (void) {
 	static GType vala_value_type_type_id = 0;
 	if (vala_value_type_type_id == 0) {
 		static const GTypeInfo g_define_type_info = { sizeof (ValaValueTypeClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vala_value_type_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ValaValueType), 0, (GInstanceInitFunc) vala_value_type_instance_init, NULL };
-		vala_value_type_type_id = g_type_register_static (VALA_TYPE_DATA_TYPE, "ValaValueType", &g_define_type_info, 0);
+		vala_value_type_type_id = g_type_register_static (VALA_TYPE_DATA_TYPE, "ValaValueType", &g_define_type_info, G_TYPE_FLAG_ABSTRACT);
 	}
 	return vala_value_type_type_id;
 }
