@@ -1,6 +1,7 @@
 /* valagidlparser.vala
  *
- * Copyright (C) 2006-2007  Jürg Billeter, Raffaele Sandrini
+ * Copyright (C) 2006-2009  Jürg Billeter
+ * Copyright (C) 2006-2008  Raffaele Sandrini
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -18,7 +19,7 @@
  *
  * Author:
  * 	Jürg Billeter <j@bitron.ch>
- *	Raffaele Sandrini <rasa@gmx.ch>
+ *	Raffaele Sandrini <raffaele@sandrini.ch>
  */
 
 #ifndef __VAPIGEN_VALAGIDLPARSER_H__
@@ -33,7 +34,8 @@
 #include <vala/valastruct.h>
 #include <vala/valainterface.h>
 #include <vala/valaenum.h>
-#include <vala/valacallback.h>
+#include <vala/valaerrordomain.h>
+#include <vala/valadelegate.h>
 #include <vala/valasourcefile.h>
 
 G_BEGIN_DECLS
@@ -54,17 +56,21 @@ typedef struct _ValaGIdlParserPrivate ValaGIdlParserPrivate;
  * Code visitor parsing all GIDL files.
  */
 struct _ValaGIdlParser {
-	ValaCodeVisitor parent;
+	ValaCodeVisitor parent_instance;
 	ValaGIdlParserPrivate * priv;
 };
+
 struct _ValaGIdlParserClass {
-	ValaCodeVisitorClass parent;
+	ValaCodeVisitorClass parent_class;
 };
 
+
 void vala_gidl_parser_parse (ValaGIdlParser* self, ValaCodeContext* context);
+ValaGIdlParser* vala_gidl_parser_construct (GType object_type);
 ValaGIdlParser* vala_gidl_parser_new (void);
 GType vala_gidl_parser_get_type (void);
 
+
 G_END_DECLS
 
 #endif
