@@ -1,6 +1,6 @@
 /* valavapigen.vala
  *
- * Copyright (C) 2006-2008  Jürg Billeter
+ * Copyright (C) 2006-2009  Jürg Billeter
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -40,26 +40,6 @@
 #include "vapigen/valagidlparser.h"
 
 
-#define VALA_TYPE_VAPI_GEN (vala_vapi_gen_get_type ())
-#define VALA_VAPI_GEN(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_VAPI_GEN, ValaVAPIGen))
-#define VALA_VAPI_GEN_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_VAPI_GEN, ValaVAPIGenClass))
-#define VALA_IS_VAPI_GEN(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_VAPI_GEN))
-#define VALA_IS_VAPI_GEN_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_VAPI_GEN))
-#define VALA_VAPI_GEN_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_VAPI_GEN, ValaVAPIGenClass))
-
-typedef struct _ValaVAPIGen ValaVAPIGen;
-typedef struct _ValaVAPIGenClass ValaVAPIGenClass;
-typedef struct _ValaVAPIGenPrivate ValaVAPIGenPrivate;
-
-struct _ValaVAPIGen {
-	GObject parent_instance;
-	ValaVAPIGenPrivate * priv;
-};
-
-struct _ValaVAPIGenClass {
-	GObjectClass parent_class;
-};
-
 
 
 static char* string_substring (const char* self, glong offset, glong len);
@@ -71,25 +51,23 @@ struct _ValaVAPIGenPrivate {
 enum  {
 	VALA_VAPI_GEN_DUMMY_PROPERTY
 };
-static char* vala_vapi_gen_directory;
-static gboolean vala_vapi_gen_version;
-static gboolean vala_vapi_gen_quiet_mode;
-static char** vala_vapi_gen_sources;
-static char** vala_vapi_gen_vapi_directories;
-static char* vala_vapi_gen_library;
-static char** vala_vapi_gen_packages;
-static char* vala_vapi_gen_metadata_filename;
+static char* vala_vapi_gen_directory = NULL;
+static gboolean vala_vapi_gen_version = FALSE;
+static gboolean vala_vapi_gen_quiet_mode = FALSE;
+static char** vala_vapi_gen_sources = NULL;
+static char** vala_vapi_gen_vapi_directories = NULL;
+static char* vala_vapi_gen_library = NULL;
+static char** vala_vapi_gen_packages = NULL;
+static char* vala_vapi_gen_metadata_filename = NULL;
 static gint vala_vapi_gen_quit (ValaVAPIGen* self);
 static gboolean vala_vapi_gen_add_package (ValaVAPIGen* self, const char* pkg);
 static char** vala_vapi_gen_get_packages_from_depsfile (const char* depsfile, int* result_length1);
 static gint vala_vapi_gen_run (ValaVAPIGen* self);
 static gint vala_vapi_gen_main (char** args, int args_length1);
-static ValaVAPIGen* vala_vapi_gen_construct (GType object_type);
-static ValaVAPIGen* vala_vapi_gen_new (void);
 static gpointer vala_vapi_gen_parent_class = NULL;
 static void vala_vapi_gen_finalize (GObject* obj);
-static GType vala_vapi_gen_get_type (void);
 static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func);
+static gint _vala_array_length (gpointer array);
 
 static const GOptionEntry VALA_VAPI_GEN_options[] = {{"vapidir", (gchar) 0, 0, G_OPTION_ARG_FILENAME_ARRAY, &vala_vapi_gen_vapi_directories, "Look for package bindings in DIRECTORY", "DIRECTORY..."}, {"pkg", (gchar) 0, 0, G_OPTION_ARG_STRING_ARRAY, &vala_vapi_gen_packages, "Include binding for PACKAGE", "PACKAGE..."}, {"library", (gchar) 0, 0, G_OPTION_ARG_STRING, &vala_vapi_gen_library, "Library name", "NAME"}, {"metadata", (gchar) 0, 0, G_OPTION_ARG_FILENAME, &vala_vapi_gen_metadata_filename, "Metadata filename", "FILE"}, {"directory", 'd', 0, G_OPTION_ARG_FILENAME, &vala_vapi_gen_directory, "Output directory", "DIRECTORY"}, {"version", (gchar) 0, 0, G_OPTION_ARG_NONE, &vala_vapi_gen_version, "Display version number", NULL}, {"quiet", 'q', 0, G_OPTION_ARG_NONE, &vala_vapi_gen_quiet_mode, "Do not print messages to the console", NULL}, {"", (gchar) 0, 0, G_OPTION_ARG_FILENAME_ARRAY, &vala_vapi_gen_sources, NULL, "FILE..."}, {NULL}};
 
@@ -101,14 +79,14 @@ static char* string_substring (const char* self, glong offset, glong len) {
 	string_length = g_utf8_strlen (self, -1);
 	if (offset < 0) {
 		offset = string_length + offset;
-		g_warn_if_fail (offset >= 0);
+		g_return_val_if_fail (offset >= 0, NULL);
 	} else {
-		g_warn_if_fail (offset <= string_length);
+		g_return_val_if_fail (offset <= string_length, NULL);
 	}
 	if (len < 0) {
 		len = string_length - offset;
 	}
-	g_warn_if_fail ((offset + len) <= string_length);
+	g_return_val_if_fail ((offset + len) <= string_length, NULL);
 	start = g_utf8_offset_to_pointer (self, offset);
 	return g_strndup (start, ((gchar*) g_utf8_offset_to_pointer (start, len)) - ((gchar*) start));
 }
@@ -116,14 +94,14 @@ static char* string_substring (const char* self, glong offset, glong len) {
 
 static gint vala_vapi_gen_quit (ValaVAPIGen* self) {
 	g_return_val_if_fail (self != NULL, 0);
-	if (vala_report_get_errors () == 0) {
+	if (vala_report_get_errors (vala_code_context_get_report (self->priv->context)) == 0) {
 		if (!vala_vapi_gen_quiet_mode) {
-			fprintf (stdout, "Generation succeeded - %d warning(s)\n", vala_report_get_warnings ());
+			fprintf (stdout, "Generation succeeded - %d warning(s)\n", vala_report_get_warnings (vala_code_context_get_report (self->priv->context)));
 		}
 		return 0;
 	} else {
 		if (!vala_vapi_gen_quiet_mode) {
-			fprintf (stdout, "Generation failed: %d error(s), %d warning(s)\n", vala_report_get_errors (), vala_report_get_warnings ());
+			fprintf (stdout, "Generation failed: %d error(s), %d warning(s)\n", vala_report_get_errors (vala_code_context_get_report (self->priv->context)), vala_report_get_warnings (vala_code_context_get_report (self->priv->context)));
 		}
 		return 1;
 	}
@@ -140,7 +118,7 @@ static gboolean vala_vapi_gen_add_package (ValaVAPIGen* self, const char* pkg) {
 		/* ignore multiple occurences of the same package*/
 		return TRUE;
 	}
-	package_path = vala_code_context_get_package_path (self->priv->context, pkg, -1, vala_vapi_gen_vapi_directories);
+	package_path = vala_code_context_get_package_path (self->priv->context, pkg, vala_vapi_gen_vapi_directories, _vala_array_length (vala_vapi_gen_vapi_directories));
 	if (package_path == NULL) {
 		gboolean _tmp1;
 		return (_tmp1 = FALSE, package_path = (g_free (package_path), NULL), _tmp1);
@@ -164,6 +142,7 @@ static char** vala_vapi_gen_get_packages_from_depsfile (const char* depsfile, in
 		char* _tmp0;
 		char** _tmp3;
 		char** _tmp4;
+		char** _tmp5;
 		contents = NULL;
 		_tmp2 = NULL;
 		_tmp0 = NULL;
@@ -171,15 +150,16 @@ static char** vala_vapi_gen_get_packages_from_depsfile (const char* depsfile, in
 		contents = (_tmp2 = _tmp0, contents = (g_free (contents), NULL), _tmp2);
 		_tmp1;
 		if (inner_error != NULL) {
+			contents = (g_free (contents), NULL);
 			if (inner_error->domain == G_FILE_ERROR) {
 				goto __catch4_g_file_error;
 			}
-			g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, inner_error->message);
-			g_clear_error (&inner_error);
+			goto __finally4;
 		}
 		_tmp3 = NULL;
 		_tmp4 = NULL;
-		return (_tmp4 = (_tmp3 = g_strsplit (g_strstrip (contents), "\n", 0), *result_length1 = -1, _tmp3), contents = (g_free (contents), NULL), _tmp4);
+		_tmp5 = NULL;
+		return (_tmp5 = (_tmp4 = _tmp3 = g_strsplit (g_strstrip (contents), "\n", 0), *result_length1 = _vala_array_length (_tmp3), _tmp4), contents = (g_free (contents), NULL), _tmp5);
 	}
 	goto __finally4;
 	__catch4_g_file_error:
@@ -188,15 +168,19 @@ static char** vala_vapi_gen_get_packages_from_depsfile (const char* depsfile, in
 		e = inner_error;
 		inner_error = NULL;
 		{
-			gpointer _tmp5;
-			char** _tmp6;
+			gpointer _tmp6;
+			char** _tmp7;
 			/* deps files are optional*/
-			_tmp6 = NULL;
-			return (_tmp6 = (_tmp5 = NULL, *result_length1 = 0, _tmp5), (e == NULL) ? NULL : (e = (g_error_free (e), NULL)), _tmp6);
+			_tmp7 = NULL;
+			return (_tmp7 = (_tmp6 = NULL, *result_length1 = 0, _tmp6), (e == NULL) ? NULL : (e = (g_error_free (e), NULL)), _tmp7);
 		}
 	}
 	__finally4:
-	;
+	if (inner_error != NULL) {
+		g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, inner_error->message);
+		g_clear_error (&inner_error);
+		return NULL;
+	}
 }
 
 
@@ -212,6 +196,7 @@ static gint vala_vapi_gen_run (ValaVAPIGen* self) {
 	g_return_val_if_fail (self != NULL, 0);
 	_tmp0 = NULL;
 	self->priv->context = (_tmp0 = vala_code_context_new (), (self->priv->context == NULL) ? NULL : (self->priv->context = (vala_code_context_unref (self->priv->context), NULL)), _tmp0);
+	vala_code_context_push (self->priv->context);
 	/* default package */
 	if (!vala_vapi_gen_add_package (self, "glib-2.0")) {
 		vala_report_error (NULL, "glib-2.0 not found in specified Vala API directories");
@@ -223,19 +208,20 @@ static gint vala_vapi_gen_run (ValaVAPIGen* self) {
 	{
 		char** source_collection;
 		int source_collection_length1;
-		char** source_it;
+		int source_it;
 		source_collection = vala_vapi_gen_sources;
-		source_collection_length1 = -1;
-		for (source_it = source_collection; *source_it != NULL; source_it = source_it + 1) {
+		source_collection_length1 = _vala_array_length (vala_vapi_gen_sources);
+		for (source_it = 0; source_it < _vala_array_length (vala_vapi_gen_sources); source_it = source_it + 1) {
 			const char* _tmp7;
 			char* source;
 			_tmp7 = NULL;
-			source = (_tmp7 = *source_it, (_tmp7 == NULL) ? NULL : g_strdup (_tmp7));
+			source = (_tmp7 = source_collection[source_it], (_tmp7 == NULL) ? NULL : g_strdup (_tmp7));
 			{
 				char* _tmp1;
 				char* _tmp2;
 				char* depsfile;
 				char** _tmp4;
+				gint deps_size;
 				gint deps_length1;
 				gint _tmp3;
 				char** deps;
@@ -252,14 +238,14 @@ static gint vala_vapi_gen_run (ValaVAPIGen* self) {
 					continue;
 				}
 				_tmp4 = NULL;
-				deps = (_tmp4 = vala_vapi_gen_get_packages_from_depsfile (depsfile, &_tmp3), deps_length1 = _tmp3, _tmp4);
+				deps = (_tmp4 = vala_vapi_gen_get_packages_from_depsfile (depsfile, &_tmp3), deps_length1 = _tmp3, deps_size = deps_length1, _tmp4);
 				{
 					char** dep_collection;
 					int dep_collection_length1;
 					int dep_it;
 					dep_collection = deps;
 					dep_collection_length1 = deps_length1;
-					for (dep_it = 0; ((deps_length1 != -1) && (dep_it < deps_length1)) || ((deps_length1 == -1) && (dep_collection[dep_it] != NULL)); dep_it = dep_it + 1) {
+					for (dep_it = 0; dep_it < deps_length1; dep_it = dep_it + 1) {
 						const char* _tmp6;
 						char* dep;
 						_tmp6 = NULL;
@@ -286,14 +272,14 @@ static gint vala_vapi_gen_run (ValaVAPIGen* self) {
 		{
 			char** package_collection;
 			int package_collection_length1;
-			char** package_it;
+			int package_it;
 			package_collection = vala_vapi_gen_packages;
-			package_collection_length1 = -1;
-			for (package_it = package_collection; *package_it != NULL; package_it = package_it + 1) {
+			package_collection_length1 = _vala_array_length (vala_vapi_gen_packages);
+			for (package_it = 0; package_it < _vala_array_length (vala_vapi_gen_packages); package_it = package_it + 1) {
 				const char* _tmp9;
 				char* package;
 				_tmp9 = NULL;
-				package = (_tmp9 = *package_it, (_tmp9 == NULL) ? NULL : g_strdup (_tmp9));
+				package = (_tmp9 = package_collection[package_it], (_tmp9 == NULL) ? NULL : g_strdup (_tmp9));
 				{
 					if (!vala_vapi_gen_add_package (self, package)) {
 						char* _tmp8;
@@ -306,22 +292,22 @@ static gint vala_vapi_gen_run (ValaVAPIGen* self) {
 			}
 		}
 		_tmp10 = NULL;
-		vala_vapi_gen_packages = (_tmp10 = NULL, vala_vapi_gen_packages = (_vala_array_free (vala_vapi_gen_packages, -1, (GDestroyNotify) g_free), NULL), _tmp10);
+		vala_vapi_gen_packages = (_tmp10 = NULL, vala_vapi_gen_packages = (_vala_array_free (vala_vapi_gen_packages, _vala_array_length (vala_vapi_gen_packages), (GDestroyNotify) g_free), NULL), _tmp10);
 	}
-	if (vala_report_get_errors () > 0) {
+	if (vala_report_get_errors (vala_code_context_get_report (self->priv->context)) > 0) {
 		return vala_vapi_gen_quit (self);
 	}
 	{
 		char** source_collection;
 		int source_collection_length1;
-		char** source_it;
+		int source_it;
 		source_collection = vala_vapi_gen_sources;
-		source_collection_length1 = -1;
-		for (source_it = source_collection; *source_it != NULL; source_it = source_it + 1) {
+		source_collection_length1 = _vala_array_length (vala_vapi_gen_sources);
+		for (source_it = 0; source_it < _vala_array_length (vala_vapi_gen_sources); source_it = source_it + 1) {
 			const char* _tmp14;
 			char* source;
 			_tmp14 = NULL;
-			source = (_tmp14 = *source_it, (_tmp14 == NULL) ? NULL : g_strdup (_tmp14));
+			source = (_tmp14 = source_collection[source_it], (_tmp14 == NULL) ? NULL : g_strdup (_tmp14));
 			{
 				if (g_file_test (source, G_FILE_TEST_EXISTS)) {
 					ValaSourceFile* _tmp12;
@@ -339,13 +325,13 @@ static gint vala_vapi_gen_run (ValaVAPIGen* self) {
 		}
 	}
 	_tmp15 = NULL;
-	vala_vapi_gen_sources = (_tmp15 = NULL, vala_vapi_gen_sources = (_vala_array_free (vala_vapi_gen_sources, -1, (GDestroyNotify) g_free), NULL), _tmp15);
-	if (vala_report_get_errors () > 0) {
+	vala_vapi_gen_sources = (_tmp15 = NULL, vala_vapi_gen_sources = (_vala_array_free (vala_vapi_gen_sources, _vala_array_length (vala_vapi_gen_sources), (GDestroyNotify) g_free), NULL), _tmp15);
+	if (vala_report_get_errors (vala_code_context_get_report (self->priv->context)) > 0) {
 		return vala_vapi_gen_quit (self);
 	}
 	parser = vala_parser_new ();
 	vala_parser_parse (parser, self->priv->context);
-	if (vala_report_get_errors () > 0) {
+	if (vala_report_get_errors (vala_code_context_get_report (self->priv->context)) > 0) {
 		gint _tmp17;
 		return (_tmp17 = vala_vapi_gen_quit (self), (parser == NULL) ? NULL : (parser = (vala_code_visitor_unref (parser), NULL)), _tmp17);
 	}
@@ -354,25 +340,25 @@ static gint vala_vapi_gen_run (ValaVAPIGen* self) {
 		vala_gir_parser_parse_metadata (girparser, vala_vapi_gen_metadata_filename);
 	}
 	vala_gir_parser_parse (girparser, self->priv->context);
-	if (vala_report_get_errors () > 0) {
+	if (vala_report_get_errors (vala_code_context_get_report (self->priv->context)) > 0) {
 		gint _tmp18;
 		return (_tmp18 = vala_vapi_gen_quit (self), (parser == NULL) ? NULL : (parser = (vala_code_visitor_unref (parser), NULL)), (girparser == NULL) ? NULL : (girparser = (vala_code_visitor_unref (girparser), NULL)), _tmp18);
 	}
 	gidlparser = vala_gidl_parser_new ();
 	vala_gidl_parser_parse (gidlparser, self->priv->context);
-	if (vala_report_get_errors () > 0) {
+	if (vala_report_get_errors (vala_code_context_get_report (self->priv->context)) > 0) {
 		gint _tmp19;
 		return (_tmp19 = vala_vapi_gen_quit (self), (parser == NULL) ? NULL : (parser = (vala_code_visitor_unref (parser), NULL)), (girparser == NULL) ? NULL : (girparser = (vala_code_visitor_unref (girparser), NULL)), (gidlparser == NULL) ? NULL : (gidlparser = (vala_code_visitor_unref (gidlparser), NULL)), _tmp19);
 	}
 	resolver = vala_symbol_resolver_new ();
 	vala_symbol_resolver_resolve (resolver, self->priv->context);
-	if (vala_report_get_errors () > 0) {
+	if (vala_report_get_errors (vala_code_context_get_report (self->priv->context)) > 0) {
 		gint _tmp20;
 		return (_tmp20 = vala_vapi_gen_quit (self), (parser == NULL) ? NULL : (parser = (vala_code_visitor_unref (parser), NULL)), (girparser == NULL) ? NULL : (girparser = (vala_code_visitor_unref (girparser), NULL)), (gidlparser == NULL) ? NULL : (gidlparser = (vala_code_visitor_unref (gidlparser), NULL)), (resolver == NULL) ? NULL : (resolver = (vala_code_visitor_unref (resolver), NULL)), _tmp20);
 	}
 	analyzer = vala_semantic_analyzer_new ();
 	vala_semantic_analyzer_analyze (analyzer, self->priv->context);
-	if (vala_report_get_errors () > 0) {
+	if (vala_report_get_errors (vala_code_context_get_report (self->priv->context)) > 0) {
 		gint _tmp21;
 		return (_tmp21 = vala_vapi_gen_quit (self), (parser == NULL) ? NULL : (parser = (vala_code_visitor_unref (parser), NULL)), (girparser == NULL) ? NULL : (girparser = (vala_code_visitor_unref (girparser), NULL)), (gidlparser == NULL) ? NULL : (gidlparser = (vala_code_visitor_unref (gidlparser), NULL)), (resolver == NULL) ? NULL : (resolver = (vala_code_visitor_unref (resolver), NULL)), (analyzer == NULL) ? NULL : (analyzer = (vala_code_visitor_unref (analyzer), NULL)), _tmp21);
 	}
@@ -425,11 +411,11 @@ static gint vala_vapi_gen_main (char** args, int args_length1) {
 		g_option_context_add_main_entries (opt_context, VALA_VAPI_GEN_options, NULL);
 		g_option_context_parse (opt_context, &args_length1, &args, &inner_error);
 		if (inner_error != NULL) {
+			(opt_context == NULL) ? NULL : (opt_context = (g_option_context_free (opt_context), NULL));
 			if (inner_error->domain == G_OPTION_ERROR) {
 				goto __catch5_g_option_error;
 			}
-			g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, inner_error->message);
-			g_clear_error (&inner_error);
+			goto __finally5;
 		}
 		(opt_context == NULL) ? NULL : (opt_context = (g_option_context_free (opt_context), NULL));
 	}
@@ -447,7 +433,11 @@ static gint vala_vapi_gen_main (char** args, int args_length1) {
 		}
 	}
 	__finally5:
-	;
+	if (inner_error != NULL) {
+		g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, inner_error->message);
+		g_clear_error (&inner_error);
+		return 0;
+	}
 	if (vala_vapi_gen_version) {
 		fprintf (stdout, "Vala API Generator %s\n", PACKAGE_VERSION);
 		return 0;
@@ -467,14 +457,14 @@ int main (int argc, char ** argv) {
 }
 
 
-static ValaVAPIGen* vala_vapi_gen_construct (GType object_type) {
+ValaVAPIGen* vala_vapi_gen_construct (GType object_type) {
 	ValaVAPIGen * self;
 	self = g_object_newv (object_type, 0, NULL);
 	return self;
 }
 
 
-static ValaVAPIGen* vala_vapi_gen_new (void) {
+ValaVAPIGen* vala_vapi_gen_new (void) {
 	return vala_vapi_gen_construct (VALA_TYPE_VAPI_GEN);
 }
 
@@ -499,7 +489,7 @@ static void vala_vapi_gen_finalize (GObject* obj) {
 }
 
 
-static GType vala_vapi_gen_get_type (void) {
+GType vala_vapi_gen_get_type (void) {
 	static GType vala_vapi_gen_type_id = 0;
 	if (vala_vapi_gen_type_id == 0) {
 		static const GTypeInfo g_define_type_info = { sizeof (ValaVAPIGenClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vala_vapi_gen_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ValaVAPIGen), 0, (GInstanceInitFunc) vala_vapi_gen_instance_init, NULL };
@@ -512,19 +502,27 @@ static GType vala_vapi_gen_get_type (void) {
 static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func) {
 	if ((array != NULL) && (destroy_func != NULL)) {
 		int i;
-		if (array_length >= 0)
 		for (i = 0; i < array_length; i = i + 1) {
-			if (((gpointer*) array)[i] != NULL)
-			destroy_func (((gpointer*) array)[i]);
-		}
-		else
-		for (i = 0; ((gpointer*) array)[i] != NULL; i = i + 1) {
-			destroy_func (((gpointer*) array)[i]);
+			if (((gpointer*) array)[i] != NULL) {
+				destroy_func (((gpointer*) array)[i]);
+			}
 		}
 	}
 	g_free (array);
 }
 
 
+static gint _vala_array_length (gpointer array) {
+	int length;
+	length = 0;
+	if (array) {
+		while (((gpointer*) array)[length]) {
+			length++;
+		}
+	}
+	return length;
+}
+
+
 
 
